@@ -28,6 +28,7 @@ STRING_EXTERN(WAL_SEGMENT_REGEXP_STR,                               WAL_SEGMENT_
 STRING_EXTERN(WAL_SEGMENT_PARTIAL_REGEXP_STR,                       WAL_SEGMENT_PARTIAL_REGEXP);
 STRING_EXTERN(WAL_SEGMENT_DIR_REGEXP_STR,                           WAL_SEGMENT_DIR_REGEXP);
 STRING_EXTERN(WAL_SEGMENT_FILE_REGEXP_STR,                          WAL_SEGMENT_FILE_REGEXP);
+STRING_EXTERN(WAL_TIMELINE_HISTORY_REGEXP_STR,                      WAL_TIMELINE_HISTORY_REGEXP);
 
 /***********************************************************************************************************************************
 Global error file constant
@@ -45,7 +46,7 @@ static const String *
 archiveAsyncSpoolQueue(ArchiveMode archiveMode)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(ENUM, archiveMode);
+        FUNCTION_TEST_PARAM(STRING_ID, archiveMode);
     FUNCTION_TEST_END();
 
     FUNCTION_TEST_RETURN((archiveMode == archiveModeGet ? STORAGE_SPOOL_ARCHIVE_IN_STR : STORAGE_SPOOL_ARCHIVE_OUT_STR));
@@ -56,7 +57,7 @@ void
 archiveAsyncErrorClear(ArchiveMode archiveMode, const String *archiveFile)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING_ID, archiveMode);
         FUNCTION_LOG_PARAM(STRING, archiveFile);
     FUNCTION_LOG_END();
 
@@ -70,12 +71,13 @@ archiveAsyncErrorClear(ArchiveMode archiveMode, const String *archiveFile)
 
 /**********************************************************************************************************************************/
 bool
-archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool throwOnError)
+archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool throwOnError, bool warnOnOk)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING_ID, archiveMode);
         FUNCTION_LOG_PARAM(STRING, walSegment);
         FUNCTION_LOG_PARAM(BOOL, throwOnError);
+        FUNCTION_LOG_PARAM(BOOL, warnOnOk);
     FUNCTION_LOG_END();
 
     ASSERT(walSegment != NULL);
@@ -134,15 +136,15 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool throw
                     THROW_FMT(FormatError, "%s message must be > 0", strZ(statusFile));
 
                 // Get contents
-                code = varIntForce(VARSTR(strNewN(strZ(content), (size_t)(linefeedPtr - strZ(content)))));
-                message = strTrim(strNew(linefeedPtr + 1));
+                code = varIntForce(VARSTR(strNewZN(strZ(content), (size_t)(linefeedPtr - strZ(content)))));
+                message = strTrim(strNewZ(linefeedPtr + 1));
             }
 
             // Process OK files
             if (okFileExists)
             {
                 // If there is content in the status file it is a warning
-                if (strSize(content) != 0)
+                if (strSize(content) != 0 && warnOnOk)
                 {
                     // If error code is not success, then this was a renamed error file
                     if (code != 0)
@@ -178,7 +180,7 @@ void
 archiveAsyncStatusErrorWrite(ArchiveMode archiveMode, const String *walSegment, int code, const String *message)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING_ID, archiveMode);
         FUNCTION_LOG_PARAM(STRING, walSegment);
         FUNCTION_LOG_PARAM(INT, code);
         FUNCTION_LOG_PARAM(STRING, message);
@@ -207,7 +209,7 @@ void
 archiveAsyncStatusOkWrite(ArchiveMode archiveMode, const String *walSegment, const String *warning)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING_ID, archiveMode);
         FUNCTION_LOG_PARAM(STRING, walSegment);
         FUNCTION_LOG_PARAM(STRING, warning);
     FUNCTION_LOG_END();
@@ -232,7 +234,7 @@ void
 archiveAsyncExec(ArchiveMode archiveMode, const StringList *commandExec)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING_ID, archiveMode);
         FUNCTION_LOG_PARAM(STRING_LIST, commandExec);
     FUNCTION_LOG_END();
 
@@ -332,10 +334,10 @@ walPath(const String *walFile, const String *pgPath, const String *command)
         {
             THROW_FMT(
                 OptionRequiredError,
-                "option '" CFGOPT_PG1_PATH "' must be specified when relative wal paths are used\n"
+                "option '%s' must be specified when relative wal paths are used\n"
                     "HINT: is %%f passed to %s instead of %%p?\n"
                     "HINT: PostgreSQL may pass relative paths even with %%p depending on the environment.",
-                strZ(command));
+                cfgOptionName(cfgOptPgPath), strZ(command));
         }
 
         // Get the working directory
@@ -358,9 +360,9 @@ walPath(const String *walFile, const String *pgPath, const String *command)
             {
                 THROW_FMT(
                     OptionInvalidValueError,
-                    PG_NAME " working directory '%s' is not the same as option " CFGOPT_PG1_PATH " '%s'\n"
-                        "HINT: is the " PG_NAME " data_directory configured the same as the " CFGOPT_PG1_PATH " option?",
-                    currentWorkDir, strZ(pgPath));
+                    PG_NAME " working directory '%s' is not the same as option %s '%s'\n"
+                        "HINT: is the " PG_NAME " data_directory configured the same as the %s option?",
+                    currentWorkDir, cfgOptionName(cfgOptPgPath), strZ(pgPath), cfgOptionName(cfgOptPgPath));
             }
         }
 
@@ -430,7 +432,7 @@ walSegmentFind(const Storage *storage, const String *archiveId, const String *wa
                 .nullOnMissing = true);
 
             // If there are results
-            if (list != NULL && strLstSize(list) > 0)
+            if (list != NULL && !strLstEmpty(list))
             {
                 // Error if there is more than one match
                 if (strLstSize(list) > 1)
