@@ -25,6 +25,11 @@ Archive Push Command
 #include "protocol/parallel.h"
 #include "storage/helper.h"
 
+/***********************************************************************************************************************************
+Constants for log messages that are used multiple times to keep them consistent
+***********************************************************************************************************************************/
+#define UNABLE_TO_FIND_VALID_REPO_MSG                               "unable to find a valid repository"
+
 /***********************************************************************************************************************************
 Ready file extension constants
 ***********************************************************************************************************************************/
@@ -191,7 +196,8 @@ typedef struct ArchivePushCheckResult
 {
     unsigned int pgVersion;                                         // PostgreSQL version
     uint64_t pgSystemId;                                            // PostgreSQL system id
-    ArchivePushFileRepoData *repoData;                              // Data for each repo
+    List *repoList;                                                 // Data for each repo
+    StringList *errorList;                                          // Errors while checking repos
 } ArchivePushCheckResult;
 
 static ArchivePushCheckResult
@@ -201,7 +207,7 @@ archivePushCheck(bool pgPathSet)
         FUNCTION_LOG_PARAM(BOOL, pgPathSet);
     FUNCTION_LOG_END();
 
-    ArchivePushCheckResult result = {.repoData = memNew(cfgOptionGroupIdxTotal(cfgOptGrpRepo) * sizeof(ArchivePushFileRepoData))};
+    ArchivePushCheckResult result = {.repoList = lstNewP(sizeof(ArchivePushFileRepoData)), .errorList = strLstNew()};
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
@@ -217,50 +223,78 @@ archivePushCheck(bool pgPathSet)
 
         for (unsigned int repoIdx = 0; repoIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); repoIdx++)
         {
-            // Get the repo storage in case it is remote and encryption settings need to be pulled down
-            storageRepoIdx(repoIdx);
-
-            // Set cipher type in repo data
-            result.repoData[repoIdx].cipherType = cipherType(cfgOptionIdxStr(cfgOptRepoCipherType, repoIdx));
-
-            // Attempt to load the archive info file
-            InfoArchive *info = infoArchiveLoadFile(
-                storageRepoIdx(repoIdx), INFO_ARCHIVE_PATH_FILE_STR, result.repoData[repoIdx].cipherType,
-                cfgOptionIdxStrNull(cfgOptRepoCipherPass, repoIdx));
-
-            // Get archive id for the most recent version -- archive-push will only operate against the most recent version
-            String *archiveId = infoPgArchiveId(infoArchivePg(info), infoPgDataCurrentId(infoArchivePg(info)));
-            InfoPgData archiveInfo = infoPgData(infoArchivePg(info), infoPgDataCurrentId(infoArchivePg(info)));
-
-            // Ensure that stanza version and system identifier match pg_control when available or the other repos when pg_control
-            // is not available
-            if (pgPathSet || repoIdx > 0)
+            TRY_BEGIN()
             {
-                if (result.pgVersion != archiveInfo.version || result.pgSystemId != archiveInfo.systemId)
+                // Get the repo storage in case it is remote and encryption settings need to be pulled down
+                storageRepoIdx(repoIdx);
+
+                // Get cipher type
+                CipherType repoCipherType = cfgOptionIdxStrId(cfgOptRepoCipherType, repoIdx);
+
+                // Attempt to load the archive info file
+                InfoArchive *info = infoArchiveLoadFile(
+                    storageRepoIdx(repoIdx), INFO_ARCHIVE_PATH_FILE_STR, repoCipherType,
+                    cfgOptionIdxStrNull(cfgOptRepoCipherPass, repoIdx));
+
+                // Get archive id for the most recent version -- archive-push will only operate against the most recent version
+                String *archiveId = infoPgArchiveId(infoArchivePg(info), infoPgDataCurrentId(infoArchivePg(info)));
+                InfoPgData archiveInfo = infoPgData(infoArchivePg(info), infoPgDataCurrentId(infoArchivePg(info)));
+
+                // Ensure that stanza version and system identifier match pg_control when available or the other repos when
+                // pg_control is not available
+                if (pgPathSet || repoIdx > 0)
                 {
-                    THROW_FMT(
-                        ArchiveMismatchError,
-                        "%s version %s, system-id %" PRIu64 " do not match %s stanza version %s, system-id %" PRIu64
-                        "\nHINT: are you archiving to the correct stanza?",
-                        pgPathSet ? PG_NAME : strZ(strNewFmt("repo%u stanza", cfgOptionGroupIdxToKey(cfgOptGrpRepo, 0))),
-                        strZ(pgVersionToStr(result.pgVersion)), result.pgSystemId,
-                        strZ(strNewFmt("repo%u", cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx))),
-                        strZ(pgVersionToStr(archiveInfo.version)), archiveInfo.systemId);
+                    if (result.pgVersion != archiveInfo.version || result.pgSystemId != archiveInfo.systemId)
+                    {
+                        THROW_FMT(
+                            ArchiveMismatchError,
+                            "%s version %s, system-id %" PRIu64 " do not match %s stanza version %s, system-id %" PRIu64
+                            "\nHINT: are you archiving to the correct stanza?",
+                            pgPathSet ? PG_NAME : strZ(strNewFmt("repo%u stanza", cfgOptionGroupIdxToKey(cfgOptGrpRepo, 0))),
+                            strZ(pgVersionToStr(result.pgVersion)), result.pgSystemId,
+                            strZ(strNewFmt("repo%u", cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx))),
+                            strZ(pgVersionToStr(archiveInfo.version)), archiveInfo.systemId);
+                    }
                 }
-            }
 
-            MEM_CONTEXT_PRIOR_BEGIN()
-            {
-                result.pgVersion = archiveInfo.version;
-                result.pgSystemId = archiveInfo.systemId;
-                result.repoData[repoIdx].archiveId = strDup(archiveId);
-                result.repoData[repoIdx].cipherPass = strDup(infoArchiveCipherPass(info));
+                MEM_CONTEXT_PRIOR_BEGIN()
+                {
+                    result.pgVersion = archiveInfo.version;
+                    result.pgSystemId = archiveInfo.systemId;
+
+                    lstAdd(
+                        result.repoList,
+                        &(ArchivePushFileRepoData)
+                        {
+                            .repoIdx = repoIdx,
+                            .archiveId = strDup(archiveId),
+                            .cipherType = repoCipherType,
+                            .cipherPass = strDup(infoArchiveCipherPass(info)),
+                        });
+                }
+                MEM_CONTEXT_PRIOR_END();
             }
-            MEM_CONTEXT_PRIOR_END();
+            CATCH_ANY()
+            {
+                strLstAdd(
+                    result.errorList,
+                    strNewFmt(
+                        "repo%u: [%s] %s", cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx), errorTypeName(errorType()),
+                        errorMessage()));
+            }
+            TRY_END();
         }
     }
     MEM_CONTEXT_TEMP_END();
 
+    // If no valid repos were found then error
+    if (lstEmpty(result.repoList))
+    {
+        ASSERT(strLstSize(result.errorList) > 0);
+
+        THROW_FMT(RepoInvalidError, UNABLE_TO_FIND_VALID_REPO_MSG ":\n%s", strZ(strLstJoin(result.errorList, "\n")));
+    }
+
     FUNCTION_LOG_RETURN_STRUCT(result);
 }
 
@@ -287,7 +321,7 @@ cmdArchivePush(void)
         lockStopTest();
 
         // Get the segment name
-        String *walFile = walPath(strLstGet(commandParam, 0), cfgOptionStrNull(cfgOptPgPath), STR(cfgCommandName(cfgCommand())));
+        String *walFile = walPath(strLstGet(commandParam, 0), cfgOptionStrNull(cfgOptPgPath), STR(cfgCommandName()));
         String *archiveFile = strBase(walFile);
 
         if (cfgOptionBool(cfgOptArchiveAsync))
@@ -311,7 +345,7 @@ cmdArchivePush(void)
             {
                 // Check if the WAL segment has been pushed.  Errors will not be thrown on the first try to allow the async process
                 // a chance to fix them.
-                pushed = archiveAsyncStatus(archiveModePush, archiveFile, throwOnError);
+                pushed = archiveAsyncStatus(archiveModePush, archiveFile, throwOnError, true);
 
                 // If the WAL segment has not already been pushed then start the async process to push it.  There's no point in
                 // forking the async process off more than once so track that as well.  Use an archive lock to prevent more than
@@ -324,8 +358,8 @@ cmdArchivePush(void)
                     // The async process should not output on the console at all
                     KeyValue *optionReplace = kvNew();
 
-                    kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_CONSOLE_STR), VARSTRDEF("off"));
-                    kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_STDERR_STR), VARSTRDEF("off"));
+                    kvPut(optionReplace, VARSTRDEF(CFGOPT_LOG_LEVEL_CONSOLE), VARSTRDEF("off"));
+                    kvPut(optionReplace, VARSTRDEF(CFGOPT_LOG_LEVEL_STDERR), VARSTRDEF("off"));
 
                     // Generate command options
                     StringList *commandExec = cfgExecParam(cfgCmdArchivePush, cfgCmdRoleAsync, optionReplace, true, false);
@@ -356,7 +390,7 @@ cmdArchivePush(void)
             {
                 THROW_FMT(
                     ArchiveTimeoutError, "unable to push WAL file '%s' to the archive asynchronously after %s second(s)",
-                    strZ(archiveFile), strZ(strNewDbl((double)cfgOptionInt64(cfgOptArchiveTimeout) / MSEC_PER_SEC)));
+                    strZ(archiveFile), strZ(cfgOptionDisplay(cfgOptArchiveTimeout)));
             }
 
             // Log success
@@ -377,13 +411,14 @@ cmdArchivePush(void)
                 ArchivePushCheckResult archiveInfo = archivePushCheck(cfgOptionTest(cfgOptPgPath));
 
                 // Push the file to the archive
-                String *warning = archivePushFile(
-                    walFile, archiveInfo.pgVersion, archiveInfo.pgSystemId, archiveFile,
-                    compressTypeEnum(cfgOptionStr(cfgOptCompressType)), cfgOptionInt(cfgOptCompressLevel), archiveInfo.repoData);
+                ArchivePushFileResult fileResult = archivePushFile(
+                    walFile, cfgOptionBool(cfgOptArchiveHeaderCheck), archiveInfo.pgVersion, archiveInfo.pgSystemId, archiveFile,
+                    compressTypeEnum(cfgOptionStr(cfgOptCompressType)), cfgOptionInt(cfgOptCompressLevel), archiveInfo.repoList,
+                    archiveInfo.errorList);
 
                 // If a warning was returned then log it
-                if (warning != NULL)
-                    LOG_WARN(strZ(warning));
+                for (unsigned int warnIdx = 0; warnIdx < strLstSize(fileResult.warnList); warnIdx++)
+                    LOG_WARN(strZ(strLstGet(fileResult.warnList, warnIdx)));
 
                 // Log success
                 LOG_INFO_FMT("pushed WAL file '%s' to the archive", strZ(archiveFile));
@@ -414,37 +449,60 @@ archivePushAsyncCallback(void *data, unsigned int clientIdx)
         FUNCTION_TEST_PARAM(UINT, clientIdx);
     FUNCTION_TEST_END();
 
-    // No special logic based on the client, we'll just get the next job
-    (void)clientIdx;
+    ProtocolParallelJob *result = NULL;
 
-    // Get a new job if there are any left
-    ArchivePushAsyncData *jobData = data;
-
-    if (jobData->walFileIdx < strLstSize(jobData->walFileList))
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        const String *walFile = strLstGet(jobData->walFileList, jobData->walFileIdx);
-        jobData->walFileIdx++;
+        // No special logic based on the client, we'll just get the next job
+        (void)clientIdx;
 
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_ARCHIVE_PUSH_STR);
-        protocolCommandParamAdd(command, VARSTR(strNewFmt("%s/%s", strZ(jobData->walPath), strZ(walFile))));
-        protocolCommandParamAdd(command, VARUINT(jobData->archiveInfo.pgVersion));
-        protocolCommandParamAdd(command, VARUINT64(jobData->archiveInfo.pgSystemId));
-        protocolCommandParamAdd(command, VARSTR(walFile));
-        protocolCommandParamAdd(command, VARUINT(jobData->compressType));
-        protocolCommandParamAdd(command, VARINT(jobData->compressLevel));
+        // Get a new job if there are any left
+        ArchivePushAsyncData *jobData = data;
 
-        // Add data for each repo to push to
-        for (unsigned int repoIdx = 0; repoIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); repoIdx++)
+        if (jobData->walFileIdx < strLstSize(jobData->walFileList))
         {
-            protocolCommandParamAdd(command, VARSTR(jobData->archiveInfo.repoData[repoIdx].archiveId));
-            protocolCommandParamAdd(command, VARUINT(jobData->archiveInfo.repoData[repoIdx].cipherType));
-            protocolCommandParamAdd(command, VARSTR(jobData->archiveInfo.repoData[repoIdx].cipherPass));
+            const String *walFile = strLstGet(jobData->walFileList, jobData->walFileIdx);
+            jobData->walFileIdx++;
+
+            ProtocolCommand *const command = protocolCommandNew(PROTOCOL_COMMAND_ARCHIVE_PUSH_FILE);
+            PackWrite *const param = protocolCommandParam(command);
+
+            pckWriteStrP(param, strNewFmt("%s/%s", strZ(jobData->walPath), strZ(walFile)));
+            pckWriteBoolP(param, cfgOptionBool(cfgOptArchiveHeaderCheck));
+            pckWriteU32P(param, jobData->archiveInfo.pgVersion);
+            pckWriteU64P(param, jobData->archiveInfo.pgSystemId);
+            pckWriteStrP(param, walFile);
+            pckWriteU32P(param, jobData->compressType);
+            pckWriteI32P(param, jobData->compressLevel);
+            pckWriteStrLstP(param, jobData->archiveInfo.errorList);
+
+            // Add data for each repo to push to
+            pckWriteArrayBeginP(param);
+
+            for (unsigned int repoListIdx = 0; repoListIdx < lstSize(jobData->archiveInfo.repoList); repoListIdx++)
+            {
+                ArchivePushFileRepoData *data = lstGet(jobData->archiveInfo.repoList, repoListIdx);
+
+                pckWriteObjBeginP(param);
+                pckWriteU32P(param, data->repoIdx);
+                pckWriteStrP(param, data->archiveId);
+                pckWriteU64P(param, data->cipherType);
+                pckWriteStrP(param, data->cipherPass);
+                pckWriteObjEndP(param);
+            }
+
+            pckWriteArrayEndP(param);
+
+            MEM_CONTEXT_PRIOR_BEGIN()
+            {
+                result = protocolParallelJobNew(VARSTR(walFile), command);
+            }
+            MEM_CONTEXT_PRIOR_END();
         }
-
-        FUNCTION_TEST_RETURN(protocolParallelJobNew(VARSTR(walFile), command));
     }
+    MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(NULL);
+    FUNCTION_TEST_RETURN(result);
 }
 
 void
@@ -515,50 +573,59 @@ cmdArchivePushAsync(void)
                     protocolParallelClientAdd(parallelExec, protocolLocalGet(protocolStorageTypeRepo, 0, processIdx));
 
                 // Process jobs
-                do
+                MEM_CONTEXT_TEMP_RESET_BEGIN()
                 {
-                    unsigned int completed = protocolParallelProcess(parallelExec);
-
-                    for (unsigned int jobIdx = 0; jobIdx < completed; jobIdx++)
+                    do
                     {
-                        protocolKeepAlive();
+                        unsigned int completed = protocolParallelProcess(parallelExec);
 
-                        // Get the job and job key
-                        ProtocolParallelJob *job = protocolParallelResult(parallelExec);
-                        unsigned int processId = protocolParallelJobProcessId(job);
-                        const String *walFile = varStr(protocolParallelJobKey(job));
-
-                        // The job was successful
-                        if (protocolParallelJobErrorCode(job) == 0)
+                        for (unsigned int jobIdx = 0; jobIdx < completed; jobIdx++)
                         {
-                            // If there was a warning then output it to the log
-                            const String *warning = varStr(protocolParallelJobResult(job));
+                            protocolKeepAlive();
 
-                            if (warning != NULL)
-                                LOG_WARN_PID(processId, strZ(warning));
+                            // Get the job and job key
+                            ProtocolParallelJob *job = protocolParallelResult(parallelExec);
+                            unsigned int processId = protocolParallelJobProcessId(job);
+                            const String *walFile = varStr(protocolParallelJobKey(job));
 
-                            // Log success
-                            LOG_DETAIL_PID_FMT(processId, "pushed WAL file '%s' to the archive", strZ(walFile));
+                            // The job was successful
+                            if (protocolParallelJobErrorCode(job) == 0)
+                            {
+                                // Output file warnings
+                                StringList *fileWarnList = pckReadStrLstP(protocolParallelJobResult(job));
 
-                            // Write the status file
-                            archiveAsyncStatusOkWrite(archiveModePush, walFile, warning);
-                        }
-                        // Else the job errored
-                        else
-                        {
-                            LOG_WARN_PID_FMT(
-                                processId,
-                                "could not push WAL file '%s' to the archive (will be retried): [%d] %s", strZ(walFile),
-                                protocolParallelJobErrorCode(job), strZ(protocolParallelJobErrorMessage(job)));
+                                for (unsigned int warnIdx = 0; warnIdx < strLstSize(fileWarnList); warnIdx++)
+                                    LOG_WARN_PID(processId, strZ(strLstGet(fileWarnList, warnIdx)));
 
-                            archiveAsyncStatusErrorWrite(
-                                archiveModePush, walFile, protocolParallelJobErrorCode(job), protocolParallelJobErrorMessage(job));
+                                // Log success
+                                LOG_DETAIL_PID_FMT(processId, "pushed WAL file '%s' to the archive", strZ(walFile));
+
+                                // Write the status file
+                                archiveAsyncStatusOkWrite(
+                                    archiveModePush, walFile, strLstEmpty(fileWarnList) ? NULL : strLstJoin(fileWarnList, "\n"));
+                            }
+                            // Else the job errored
+                            else
+                            {
+                                LOG_WARN_PID_FMT(
+                                    processId,
+                                    "could not push WAL file '%s' to the archive (will be retried): [%d] %s", strZ(walFile),
+                                    protocolParallelJobErrorCode(job), strZ(protocolParallelJobErrorMessage(job)));
+
+                                archiveAsyncStatusErrorWrite(
+                                    archiveModePush, walFile, protocolParallelJobErrorCode(job),
+                                    protocolParallelJobErrorMessage(job));
+                            }
+
+                            protocolParallelJobFree(job);
                         }
 
-                        protocolParallelJobFree(job);
+                        // Reset the memory context occasionally so we don't use too much memory or slow down processing
+                        MEM_CONTEXT_TEMP_RESET(1000);
                     }
+                    while (!protocolParallelDone(parallelExec));
                 }
-                while (!protocolParallelDone(parallelExec));
+                MEM_CONTEXT_TEMP_END();
             }
         }
         // On any global error write a single error file to cover all unprocessed files
