@@ -3,13 +3,13 @@ Error Handler
 ***********************************************************************************************************************************/
 #include "build.auto.h"
 
+#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "common/error.h"
-#include "common/memContext.h"
 #include "common/stackTrace.h"
 
 /***********************************************************************************************************************************
@@ -27,7 +27,7 @@ struct ErrorType
     const ErrorType name = {code, #name, &parentType}
 
 // Define test error
-#ifndef NDEBUG
+#ifdef DEBUG
     ERROR_DEFINE(1, TestError, RuntimeError);
 #endif
 
@@ -47,11 +47,25 @@ typedef enum {errorStateBegin, errorStateTry, errorStateCatch, errorStateFinal,
 /***********************************************************************************************************************************
 Track error handling
 ***********************************************************************************************************************************/
+typedef struct Error
+{
+    const ErrorType *errorType;                                     // Error type
+    const char *fileName;                                           // Source file where the error occurred
+    const char *functionName;                                       // Function where the error occurred
+    int fileLine;                                                   // Source file line where the error occurred
+    const char *message;                                            // Description of the error
+    const char *stackTrace;                                         // Stack trace
+} Error;
+
 static struct
 {
     // Array of jump buffers
     jmp_buf jumpList[ERROR_TRY_MAX];
 
+    // Handler list
+    const ErrorHandlerFunction *handlerList;
+    unsigned int handlerTotal;
+
     // State of each try
     int tryTotal;
 
@@ -62,15 +76,7 @@ static struct
     } tryList[ERROR_TRY_MAX + 1];
 
     // Last error
-    struct
-    {
-        const ErrorType *errorType;                                     // Error type
-        const char *fileName;                                           // Source file where the error occurred
-        const char *functionName;                                       // Function where the error occurred
-        int fileLine;                                                   // Source file line where the error occurred
-        const char *message;                                            // Description of the error
-        const char *stackTrace;                                         // Stack trace
-    } error;
+    Error error;
 } errorContext;
 
 /***********************************************************************************************************************************
@@ -82,12 +88,23 @@ situations.
 
 The temp buffer is required because the error message being passed might be the error already stored in the message buffer.
 ***********************************************************************************************************************************/
-#define ERROR_MESSAGE_BUFFER_SIZE                                   8192
+#ifndef ERROR_MESSAGE_BUFFER_SIZE
+    #define ERROR_MESSAGE_BUFFER_SIZE                                   8192
+#endif
 
 static char messageBuffer[ERROR_MESSAGE_BUFFER_SIZE];
 static char messageBufferTemp[ERROR_MESSAGE_BUFFER_SIZE];
 static char stackTraceBuffer[ERROR_MESSAGE_BUFFER_SIZE];
 
+/**********************************************************************************************************************************/
+void errorHandlerSet(ErrorHandlerFunction *list, unsigned int total)
+{
+    assert(total == 0 || list != NULL);
+
+    errorContext.handlerList = list;
+    errorContext.handlerTotal = total;
+}
+
 /**********************************************************************************************************************************/
 int
 errorTypeCode(const ErrorType *errorType)
@@ -162,6 +179,8 @@ errorTypeExtends(const ErrorType *child, const ErrorType *parent)
 const ErrorType *
 errorType(void)
 {
+    assert(errorContext.error.errorType != NULL);
+
     return errorContext.error.errorType;
 }
 
@@ -176,6 +195,8 @@ errorCode(void)
 const char *
 errorFileName(void)
 {
+    assert(errorContext.error.fileName != NULL);
+
     return errorContext.error.fileName;
 }
 
@@ -183,6 +204,8 @@ errorFileName(void)
 const char *
 errorFunctionName(void)
 {
+    assert(errorContext.error.functionName != NULL);
+
     return errorContext.error.functionName;
 }
 
@@ -190,6 +213,8 @@ errorFunctionName(void)
 int
 errorFileLine(void)
 {
+    assert(errorContext.error.fileLine != 0);
+
     return errorContext.error.fileLine;
 }
 
@@ -197,6 +222,8 @@ errorFileLine(void)
 const char *
 errorMessage(void)
 {
+    assert(errorContext.error.message != NULL);
+
     return errorContext.error.message;
 }
 
@@ -211,6 +238,8 @@ errorName(void)
 const char *
 errorStackTrace(void)
 {
+    assert(errorContext.error.stackTrace != NULL);
+
     return errorContext.error.stackTrace;
 }
 
@@ -284,6 +313,8 @@ errorInternalTry(const char *fileName, const char *functionName, int fileLine)
 void
 errorInternalPropagate(void)
 {
+    assert(errorContext.error.errorType != NULL);
+
     // Mark the error as uncaught
     errorContext.tryList[errorContext.tryTotal].uncaught = true;
 
@@ -312,8 +343,15 @@ errorInternalProcess(bool catch)
     // Else if just entering error state clean up the stack
     else if (errorContext.tryList[errorContext.tryTotal].state == errorStateTry)
     {
-        stackTraceClean(errorTryDepth());
-        memContextClean(errorTryDepth());
+        for (unsigned int handlerIdx = 0; handlerIdx < errorContext.handlerTotal; handlerIdx++)
+            errorContext.handlerList[handlerIdx](errorTryDepth());
+    }
+
+    // Any catch blocks have been processed and none of them called RETHROW() so clear the error
+    if (errorContext.tryList[errorContext.tryTotal].state == errorStateCatch &&
+        !errorContext.tryList[errorContext.tryTotal].uncaught)
+    {
+        errorContext.error = (Error){0};
     }
 
     // Increment the state
@@ -343,7 +381,9 @@ errorInternalProcess(bool catch)
 
 /**********************************************************************************************************************************/
 void
-errorInternalThrow(const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *message)
+errorInternalThrow(
+    const ErrorType *const errorType, const char *const fileName, const char *const functionName, const int fileLine,
+    const char *const message, const char *const stackTrace)
 {
     // Setup error data
     errorContext.error.errorType = errorType;
@@ -357,8 +397,15 @@ errorInternalThrow(const ErrorType *errorType, const char *fileName, const char
 
     errorContext.error.message = (const char *)messageBuffer;
 
-    // Generate the stack trace for the error
-    if (stackTraceToZ(
+    // If a stack trace was provided
+    if (stackTrace != NULL)
+    {
+        strncpy(stackTraceBuffer, stackTrace, sizeof(stackTraceBuffer) - 1);
+        messageBuffer[sizeof(stackTraceBuffer) - 1] = '\0';
+    }
+    // Else generate the stack trace for the error
+    else if (
+        stackTraceToZ(
             stackTraceBuffer, sizeof(stackTraceBuffer), fileName, functionName, (unsigned int)fileLine) >= sizeof(stackTraceBuffer))
     {
         // Indicate that the stack trace was truncated
@@ -380,7 +427,7 @@ errorInternalThrowFmt(
     vsnprintf(messageBufferTemp, ERROR_MESSAGE_BUFFER_SIZE - 1, format, argument);
     va_end(argument);
 
-    errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp);
+    errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp, NULL);
 }
 
 /**********************************************************************************************************************************/
@@ -397,7 +444,7 @@ errorInternalThrowSys(
     else
         snprintf(messageBufferTemp, ERROR_MESSAGE_BUFFER_SIZE - 1, "%s: [%d] %s", message, errNo, strerror(errNo));
 
-    errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp);
+    errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp, NULL);
 }
 
 #ifdef DEBUG_COVERAGE
@@ -425,7 +472,7 @@ errorInternalThrowSysFmt(
     if (errNo != 0)
         snprintf(messageBufferTemp + messageSize, ERROR_MESSAGE_BUFFER_SIZE - 1 - messageSize, ": [%d] %s", errNo, strerror(errNo));
 
-    errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp);
+    errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp, NULL);
 }
 
 #ifdef DEBUG_COVERAGE
@@ -449,7 +496,7 @@ errorInternalThrowOnSysFmt(
                 messageBufferTemp + messageSize, ERROR_MESSAGE_BUFFER_SIZE - 1 - messageSize, ": [%d] %s", errNo, strerror(errNo));
         }
 
-        errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp);
+        errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp, NULL);
     }
 }
 #endif
