@@ -1,27 +1,21 @@
 /***********************************************************************************************************************************
-Http Header
+HTTP Header
 ***********************************************************************************************************************************/
 #include "build.auto.h"
 
 #include "common/debug.h"
 #include "common/io/http/header.h"
-#include "common/memContext.h"
 #include "common/type/keyValue.h"
-#include "common/type/object.h"
 
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
 struct HttpHeader
 {
-    MemContext *memContext;                                         // Mem context
     const StringList *redactList;                                   // List of headers to redact during logging
     KeyValue *kv;                                                   // KeyValue store
 };
 
-OBJECT_DEFINE_MOVE(HTTP_HEADER);
-OBJECT_DEFINE_FREE(HTTP_HEADER);
-
 /**********************************************************************************************************************************/
 HttpHeader *
 httpHeaderNew(const StringList *redactList)
@@ -30,19 +24,18 @@ httpHeaderNew(const StringList *redactList)
 
     HttpHeader *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("HttpHeader")
+    OBJ_NEW_BEGIN(HttpHeader)
     {
         // Allocate state and set context
-        this = memNew(sizeof(HttpHeader));
+        this = OBJ_NEW_ALLOC();
 
         *this = (HttpHeader)
         {
-            .memContext = MEM_CONTEXT_NEW(),
             .redactList = strLstDup(redactList),
             .kv = kvNew(),
         };
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_TEST_RETURN(this);
 }
@@ -60,19 +53,18 @@ httpHeaderDup(const HttpHeader *header, const StringList *redactList)
 
     if (header != NULL)
     {
-        MEM_CONTEXT_NEW_BEGIN("HttpHeader")
+        OBJ_NEW_BEGIN(HttpHeader)
         {
             // Allocate state and set context
-            this = memNew(sizeof(HttpHeader));
+            this = OBJ_NEW_ALLOC();
 
             *this = (HttpHeader)
             {
-                .memContext = MEM_CONTEXT_NEW(),
                 .redactList = redactList == NULL ? strLstDup(header->redactList) : strLstDup(redactList),
                 .kv = kvDup(header->kv),
             };
         }
-        MEM_CONTEXT_NEW_END();
+        OBJ_NEW_END();
     }
 
     FUNCTION_TEST_RETURN(this);
@@ -103,9 +95,9 @@ httpHeaderAdd(HttpHeader *this, const String *key, const String *value)
     {
         MEM_CONTEXT_TEMP_BEGIN()
         {
-            String *valueAppend = strDup(varStr(valueVar));
-            strCat(valueAppend, ", ");
-            strCat(valueAppend, strPtr(value));
+            String *const valueAppend = strCat(strNew(), varStr(valueVar));
+            strCatZ(valueAppend, ", ");
+            strCatZ(valueAppend, strZ(value));
 
             kvPut(this->kv, keyVar, VARSTR(valueAppend));
         }
@@ -185,7 +177,7 @@ httpHeaderRedact(const HttpHeader *this, const String *key)
 String *
 httpHeaderToLog(const HttpHeader *this)
 {
-    String *result = strNew("{");
+    String *result = strCatZ(strNew(), "{");
     const StringList *keyList = httpHeaderList(this);
 
     for (unsigned int keyIdx = 0; keyIdx < strLstSize(keyList); keyIdx++)
@@ -193,15 +185,15 @@ httpHeaderToLog(const HttpHeader *this)
         const String *key = strLstGet(keyList, keyIdx);
 
         if (strSize(result) != 1)
-            strCat(result, ", ");
+            strCatZ(result, ", ");
 
         if (httpHeaderRedact(this, key))
-            strCatFmt(result, "%s: <redacted>", strPtr(key));
+            strCatFmt(result, "%s: <redacted>", strZ(key));
         else
-            strCatFmt(result, "%s: '%s'", strPtr(key), strPtr(httpHeaderGet(this, key)));
+            strCatFmt(result, "%s: '%s'", strZ(key), strZ(httpHeaderGet(this, key)));
     }
 
-    strCat(result, "}");
+    strCatZ(result, "}");
 
     return result;
 }
