@@ -8,7 +8,6 @@ HTTP Request
 #include "common/io/http/request.h"
 #include "common/log.h"
 #include "common/stat.h"
-#include "common/type/object.h"
 #include "common/wait.h"
 #include "version.h"
 
@@ -27,6 +26,9 @@ STRING_EXTERN(HTTP_VERB_PUT_STR,                                    HTTP_VERB_PU
 STRING_EXTERN(HTTP_HEADER_AUTHORIZATION_STR,                        HTTP_HEADER_AUTHORIZATION);
 STRING_EXTERN(HTTP_HEADER_CONTENT_LENGTH_STR,                       HTTP_HEADER_CONTENT_LENGTH);
 STRING_EXTERN(HTTP_HEADER_CONTENT_MD5_STR,                          HTTP_HEADER_CONTENT_MD5);
+STRING_EXTERN(HTTP_HEADER_CONTENT_RANGE_STR,                        HTTP_HEADER_CONTENT_RANGE);
+STRING_EXTERN(HTTP_HEADER_CONTENT_TYPE_STR,                         HTTP_HEADER_CONTENT_TYPE);
+STRING_EXTERN(HTTP_HEADER_CONTENT_TYPE_APP_FORM_URL_STR,            HTTP_HEADER_CONTENT_TYPE_APP_FORM_URL);
 STRING_EXTERN(HTTP_HEADER_ETAG_STR,                                 HTTP_HEADER_ETAG);
 STRING_EXTERN(HTTP_HEADER_DATE_STR,                                 HTTP_HEADER_DATE);
 STRING_EXTERN(HTTP_HEADER_HOST_STR,                                 HTTP_HEADER_HOST);
@@ -41,25 +43,13 @@ Object type
 ***********************************************************************************************************************************/
 struct HttpRequest
 {
-    MemContext *memContext;                                         // Mem context
+    HttpRequestPub pub;                                             // Publicly accessible variables
     HttpClient *client;                                             // HTTP client
-    const String *verb;                                             // HTTP verb (GET, POST, etc.)
-    const String *uri;                                              // HTTP URI
-    const HttpQuery *query;                                         // HTTP query
-    const HttpHeader *header;                                       // HTTP headers
     const Buffer *content;                                          // HTTP content
 
     HttpSession *session;                                           // Session for async requests
 };
 
-OBJECT_DEFINE_MOVE(HTTP_REQUEST);
-OBJECT_DEFINE_FREE(HTTP_REQUEST);
-
-OBJECT_DEFINE_GET(Verb, const, HTTP_REQUEST, const String *, verb);
-OBJECT_DEFINE_GET(Uri, const, HTTP_REQUEST, const String *, uri);
-OBJECT_DEFINE_GET(Query, const, HTTP_REQUEST, const HttpQuery *, query);
-OBJECT_DEFINE_GET(Header, const, HTTP_REQUEST, const HttpHeader *, header);
-
 /***********************************************************************************************************************************
 Process the request
 ***********************************************************************************************************************************/
@@ -106,19 +96,22 @@ httpRequestProcess(HttpRequest *this, bool waitForResponse, bool contentCache)
 
                         // Format the request and user agent
                         String *requestStr =
-                            strNewFmt(
+                            strCatFmt(
+                                strNew(),
                                 "%s %s%s%s " HTTP_VERSION CRLF_Z HTTP_HEADER_USER_AGENT ":" PROJECT_NAME "/" PROJECT_VERSION CRLF_Z,
-                                strZ(this->verb), strZ(httpUriEncode(this->uri, true)), this->query == NULL ? "" : "?",
-                                this->query == NULL ? "" : strZ(httpQueryRenderP(this->query)));
+                                strZ(httpRequestVerb(this)), strZ(httpRequestPath(this)), httpRequestQuery(this) == NULL ? "" : "?",
+                                httpRequestQuery(this) == NULL ? "" : strZ(httpQueryRenderP(httpRequestQuery(this))));
 
                         // Add headers
-                        const StringList *headerList = httpHeaderList(this->header);
+                        const StringList *headerList = httpHeaderList(httpRequestHeader(this));
 
                         for (unsigned int headerIdx = 0; headerIdx < strLstSize(headerList); headerIdx++)
                         {
                             const String *headerKey = strLstGet(headerList, headerIdx);
 
-                            strCatFmt(requestStr, "%s:%s" CRLF_Z, strZ(headerKey), strZ(httpHeaderGet(this->header, headerKey)));
+                            strCatFmt(
+                                requestStr, "%s:%s" CRLF_Z, strZ(headerKey),
+                                strZ(httpHeaderGet(httpRequestHeader(this), headerKey)));
                         }
 
                         // Add blank line to end of headers and write the request as a buffer so secrets do not show up in logs
@@ -134,13 +127,13 @@ httpRequestProcess(HttpRequest *this, bool waitForResponse, bool contentCache)
 
                         // If not waiting for the response then move the session to the object context
                         if (!waitForResponse)
-                            this->session = httpSessionMove(session, this->memContext);
+                            this->session = httpSessionMove(session, objMemContext(this));
                     }
 
                     // Wait for response
                     if (waitForResponse)
                     {
-                        result = httpResponseNew(session, this->verb, contentCache);
+                        result = httpResponseNew(session, httpRequestVerb(this), contentCache);
 
                         // Retry when response code is 5xx.  These errors generally represent a server error for a request that
                         // looks valid. There are a few errors that might be permanently fatal but they are rare and it seems best
@@ -181,34 +174,36 @@ httpRequestProcess(HttpRequest *this, bool waitForResponse, bool contentCache)
 
 /**********************************************************************************************************************************/
 HttpRequest *
-httpRequestNew(HttpClient *client, const String *verb, const String *uri, HttpRequestNewParam param)
+httpRequestNew(HttpClient *client, const String *verb, const String *path, HttpRequestNewParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug)
         FUNCTION_LOG_PARAM(HTTP_CLIENT, client);
         FUNCTION_LOG_PARAM(STRING, verb);
-        FUNCTION_LOG_PARAM(STRING, uri);
+        FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(HTTP_QUERY, param.query);
         FUNCTION_LOG_PARAM(HTTP_HEADER, param.header);
         FUNCTION_LOG_PARAM(BUFFER, param.content);
     FUNCTION_LOG_END();
 
     ASSERT(verb != NULL);
-    ASSERT(uri != NULL);
+    ASSERT(path != NULL);
 
     HttpRequest *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("HttpRequest")
+    OBJ_NEW_BEGIN(HttpRequest)
     {
-        this = memNew(sizeof(HttpRequest));
+        this = OBJ_NEW_ALLOC();
 
         *this = (HttpRequest)
         {
-            .memContext = MEM_CONTEXT_NEW(),
+            .pub =
+            {
+                .verb = strDup(verb),
+                .path = strDup(path),
+                .query = httpQueryDupP(param.query),
+                .header = param.header == NULL ? httpHeaderNew(NULL) : httpHeaderDup(param.header, NULL),
+            },
             .client = client,
-            .verb = strDup(verb),
-            .uri = strDup(uri),
-            .query = httpQueryDupP(param.query),
-            .header = param.header == NULL ? httpHeaderNew(NULL) : httpHeaderDup(param.header, NULL),
             .content = param.content == NULL ? NULL : bufDup(param.content),
         };
 
@@ -216,7 +211,7 @@ httpRequestNew(HttpClient *client, const String *verb, const String *uri, HttpRe
         httpRequestProcess(this, false, false);
         statInc(HTTP_STAT_REQUEST_STR);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(HTTP_REQUEST, this);
 }
@@ -248,22 +243,22 @@ httpRequestError(const HttpRequest *this, HttpResponse *response)
     ASSERT(response != NULL);
 
     // Error code
-    String *error = strNewFmt("HTTP request failed with %u", httpResponseCode(response));
+    String *error = strCatFmt(strNew(), "HTTP request failed with %u", httpResponseCode(response));
 
     // Add reason when present
     if (strSize(httpResponseReason(response)) > 0)
         strCatFmt(error, " (%s)", strZ(httpResponseReason(response)));
 
-    // Output uri/query
-    strCatZ(error, ":\n*** URI/Query ***:");
+    // Output path/query
+    strCatZ(error, ":\n*** Path/Query ***:");
 
-    strCatFmt(error, "\n%s", strZ(httpUriEncode(this->uri, true)));
+    strCatFmt(error, "\n%s", strZ(httpRequestPath(this)));
 
-    if (this->query != NULL)
-        strCatFmt(error, "?%s", strZ(httpQueryRenderP(this->query, .redact = true)));
+    if (httpRequestQuery(this) != NULL)
+        strCatFmt(error, "?%s", strZ(httpQueryRenderP(httpRequestQuery(this), .redact = true)));
 
     // Output request headers
-    const StringList *requestHeaderList = httpHeaderList(this->header);
+    const StringList *requestHeaderList = httpHeaderList(httpRequestHeader(this));
 
     if (!strLstEmpty(requestHeaderList))
     {
@@ -275,7 +270,7 @@ httpRequestError(const HttpRequest *this, HttpResponse *response)
 
             strCatFmt(
                 error, "\n%s: %s", strZ(key),
-                httpHeaderRedact(this->header, key) ? "<redacted>" : strZ(httpHeaderGet(this->header, key)));
+                httpHeaderRedact(httpRequestHeader(this), key) ? "<redacted>" : strZ(httpHeaderGet(httpRequestHeader(this), key)));
         }
     }
 
@@ -309,7 +304,7 @@ String *
 httpRequestToLog(const HttpRequest *this)
 {
     return strNewFmt(
-        "{verb: %s, uri: %s, query: %s, header: %s, contentSize: %zu}", strZ(this->verb), strZ(this->uri),
-        this->query == NULL ? "null" : strZ(httpQueryToLog(this->query)), strZ(httpHeaderToLog(this->header)),
-        this->content == NULL ? 0 : bufUsed(this->content));
+        "{verb: %s, path: %s, query: %s, header: %s, contentSize: %zu}", strZ(httpRequestVerb(this)), strZ(httpRequestPath(this)),
+        httpRequestQuery(this) == NULL ? "null" : strZ(httpQueryToLog(httpRequestQuery(this))),
+        strZ(httpHeaderToLog(httpRequestHeader(this))), this->content == NULL ? 0 : bufUsed(this->content));
 }
