@@ -3,144 +3,77 @@ TLS Client
 ***********************************************************************************************************************************/
 #include "build.auto.h"
 
-#include <string.h>
 #include <strings.h>
 
-#include <openssl/x509v3.h>
-
 #include "common/crypto/common.h"
 #include "common/debug.h"
 #include "common/log.h"
+#include "common/io/client.h"
 #include "common/io/io.h"
 #include "common/io/tls/client.h"
-#include "common/io/tls/session.intern.h"
-#include "common/memContext.h"
+#include "common/io/tls/common.h"
+#include "common/io/tls/session.h"
+#include "common/stat.h"
 #include "common/type/object.h"
 #include "common/wait.h"
 
 /***********************************************************************************************************************************
-Statistics
+Statistics constants
 ***********************************************************************************************************************************/
-static TlsClientStat tlsClientStatLocal;
+STRING_EXTERN(TLS_STAT_CLIENT_STR,                                  TLS_STAT_CLIENT);
+STRING_EXTERN(TLS_STAT_RETRY_STR,                                   TLS_STAT_RETRY);
+STRING_EXTERN(TLS_STAT_SESSION_STR,                                 TLS_STAT_SESSION);
 
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
-struct TlsClient
+typedef struct TlsClient
 {
-    MemContext *memContext;                                         // Mem context
-    TimeMSec timeout;                                               // Timeout for any i/o operation (connect, read, etc.)
+    const String *host;                                             // Host to use for peer verification
+    TimeMSec timeoutConnect;                                        // Timeout for connection
+    TimeMSec timeoutSession;                                        // Timeout passed to session
     bool verifyPeer;                                                // Should the peer (server) certificate be verified?
-    SocketClient *socketClient;                                     // Socket client
+    IoClient *ioClient;                                             // Underlying client (usually a SocketClient)
 
     SSL_CTX *context;                                               // TLS context
-};
+} TlsClient;
 
-OBJECT_DEFINE_FREE(TLS_CLIENT);
+/***********************************************************************************************************************************
+Macros for function logging
+***********************************************************************************************************************************/
+static String *
+tlsClientToLog(const THIS_VOID)
+{
+    THIS(const TlsClient);
+
+    return strNewFmt(
+        "{ioClient: %s, timeoutConnect: %" PRIu64 ", timeoutSession: %" PRIu64 ", verifyPeer: %s}",
+        objMemContextFreeing(this) ? NULL_Z : strZ(ioClientToLog(this->ioClient)), this->timeoutConnect, this->timeoutSession,
+        cvtBoolToConstZ(this->verifyPeer));
+}
+
+#define FUNCTION_LOG_TLS_CLIENT_TYPE                                                                                               \
+    TlsClient *
+#define FUNCTION_LOG_TLS_CLIENT_FORMAT(value, buffer, bufferSize)                                                                  \
+    FUNCTION_LOG_STRING_OBJECT_FORMAT(value, tlsClientToLog, buffer, bufferSize)
 
 /***********************************************************************************************************************************
 Free connection
 ***********************************************************************************************************************************/
-OBJECT_DEFINE_FREE_RESOURCE_BEGIN(TLS_CLIENT, LOG, logLevelTrace)
+static void
+tlsClientFreeResource(THIS_VOID)
 {
-    SSL_CTX_free(this->context);
-}
-OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
+    THIS(TlsClient);
 
-/**********************************************************************************************************************************/
-TlsClient *
-tlsClientNew(SocketClient *socket, TimeMSec timeout, bool verifyPeer, const String *caFile, const String *caPath)
-{
-    FUNCTION_LOG_BEGIN(logLevelDebug)
-        FUNCTION_LOG_PARAM(SOCKET_CLIENT, socket);
-        FUNCTION_LOG_PARAM(TIME_MSEC, timeout);
-        FUNCTION_LOG_PARAM(BOOL, verifyPeer);
-        FUNCTION_LOG_PARAM(STRING, caFile);
-        FUNCTION_LOG_PARAM(STRING, caPath);
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
     FUNCTION_LOG_END();
 
-    ASSERT(socket != NULL);
+    ASSERT(this != NULL);
 
-    TlsClient *this = NULL;
+    SSL_CTX_free(this->context);
 
-    MEM_CONTEXT_NEW_BEGIN("TlsClient")
-    {
-        this = memNew(sizeof(TlsClient));
-
-        *this = (TlsClient)
-        {
-            .memContext = MEM_CONTEXT_NEW(),
-            .socketClient = sckClientMove(socket, MEM_CONTEXT_NEW()),
-            .timeout = timeout,
-            .verifyPeer = verifyPeer,
-        };
-
-        // Setup TLS context
-        // -------------------------------------------------------------------------------------------------------------------------
-        cryptoInit();
-
-        // Select the TLS method to use.  To maintain compatibility with older versions of OpenSSL we need to use an SSL method,
-        // but SSL versions will be excluded in SSL_CTX_set_options().
-        const SSL_METHOD *method = SSLv23_method();
-        cryptoError(method == NULL, "unable to load TLS method");
-
-        // Create the TLS context
-        this->context = SSL_CTX_new(method);
-        cryptoError(this->context == NULL, "unable to create TLS context");
-
-        memContextCallbackSet(this->memContext, tlsClientFreeResource, this);
-
-        // Exclude SSL versions to only allow TLS and also disable compression
-        SSL_CTX_set_options(this->context, (long)(SSL_OP_ALL | SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_COMPRESSION));
-
-        // Disable auto-retry to prevent SSL_read() from hanging
-        SSL_CTX_clear_mode(this->context, SSL_MODE_AUTO_RETRY);
-
-        // Set location of CA certificates if the server certificate will be verified
-        // -------------------------------------------------------------------------------------------------------------------------
-        if (this->verifyPeer)
-        {
-            // If the user specified a location
-            if (caFile != NULL || caPath != NULL)                                                                   // {vm_covered}
-            {
-                cryptoError(                                                                                        // {vm_covered}
-                    SSL_CTX_load_verify_locations(this->context, strPtr(caFile), strPtr(caPath)) != 1,              // {vm_covered}
-                    "unable to set user-defined CA certificate location");                                          // {vm_covered}
-            }
-            // Else use the defaults
-            else
-                cryptoError(SSL_CTX_set_default_verify_paths(this->context) != 1, "unable to set default CA certificate location");
-        }
-
-        tlsClientStatLocal.object++;
-    }
-    MEM_CONTEXT_NEW_END();
-
-    FUNCTION_LOG_RETURN(TLS_CLIENT, this);
-}
-
-/***********************************************************************************************************************************
-Convert an ASN1 string used in certificates to a String
-***********************************************************************************************************************************/
-static String *
-asn1ToStr(ASN1_STRING *nameAsn1)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM_P(VOID, nameAsn1);
-    FUNCTION_TEST_END();
-
-    // The name should not be null
-    if (nameAsn1 == NULL)                                                                                           // {vm_covered}
-        THROW(CryptoError, "TLS certificate name entry is missing");
-
-    FUNCTION_TEST_RETURN(                                                                                           // {vm_covered}
-        strNewN(
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-            (const char *)ASN1_STRING_data(nameAsn1),
-#else
-            (const char *)ASN1_STRING_get0_data(nameAsn1),
-#endif
-            (size_t)ASN1_STRING_length(nameAsn1)));
+    FUNCTION_LOG_RETURN_VOID();
 }
 
 /***********************************************************************************************************************************
@@ -159,14 +92,13 @@ tlsClientHostVerifyName(const String *host, const String *name)
     ASSERT(host != NULL);
     ASSERT(name != NULL);
 
-    // Reject embedded nulls in certificate common or alternative name to prevent attacks like CVE-2009-4034
-    if (strlen(strPtr(name)) != strSize(name))
-        THROW(CryptoError, "TLS certificate name contains embedded null");
+    // Check for NULLs in the name
+    tlsCertNameVerify(name);
 
     bool result = false;
 
     // Try an exact match
-    if (strcasecmp(strPtr(name), strPtr(host)) == 0)                                                                // {vm_covered}
+    if (strcasecmp(strZ(name), strZ(host)) == 0)                                                                    // {vm_covered}
     {
         result = true;                                                                                              // {vm_covered}
     }
@@ -180,9 +112,9 @@ tlsClientHostVerifyName(const String *host, const String *name)
     //
     // This is roughly in line with RFC2818, but contrary to what most browsers appear to be implementing (point 3 being the
     // difference)
-    else if (strPtr(name)[0] == '*' && strPtr(name)[1] == '.' && strSize(name) > 2 &&                               // {vm_covered}
+    else if (strZ(name)[0] == '*' && strZ(name)[1] == '.' && strSize(name) > 2 &&                                   // {vm_covered}
              strSize(name) < strSize(host) &&                                                                       // {vm_covered}
-             strcasecmp(strPtr(name) + 1, strPtr(host) + strSize(host) - strSize(name) + 1) == 0 &&                 // {vm_covered}
+             strcasecmp(strZ(name) + 1, strZ(host) + strSize(host) - strSize(name) + 1) == 0 &&                     // {vm_covered}
              strChr(host, '.') >= (int)(strSize(host) - strSize(name)))                                             // {vm_covered}
     {
         result = true;                                                                                              // {vm_covered}
@@ -227,7 +159,7 @@ tlsClientHostVerify(const String *host, X509 *certificate)
                 altNameFound = true;                                                                                // {vm_covered}
 
                 if (name->type == GEN_DNS)                                                                          // {vm_covered}
-                    result = tlsClientHostVerifyName(host, asn1ToStr(name->d.dNSName));                             // {vm_covered}
+                    result = tlsClientHostVerifyName(host, tlsAsn1ToStr(name->d.dNSName));                          // {vm_covered}
 
                 if (result != false)                                                                                // {vm_covered}
                     break;                                                                                          // {vm_covered}
@@ -239,17 +171,7 @@ tlsClientHostVerify(const String *host, X509 *certificate)
         // If no subject alternative name was found then check the common name. Per RFC 2818 and RFC 6125, if the subjectAltName
         // extension of type dNSName is present the CN must be ignored.
         if (!altNameFound)                                                                                          // {vm_covered}
-        {
-            X509_NAME *subjectName = X509_get_subject_name(certificate);                                            // {vm_covered}
-            CHECK(subjectName != NULL);                                                                             // {vm_covered}
-
-            int commonNameIndex = X509_NAME_get_index_by_NID(subjectName, NID_commonName, -1);                      // {vm_covered}
-            CHECK(commonNameIndex >= 0);                                                                            // {vm_covered}
-
-            result = tlsClientHostVerifyName(                                                                       // {vm_covered}
-                host,                                                                                               // {vm_covered}
-                asn1ToStr(X509_NAME_ENTRY_get_data(X509_NAME_get_entry(subjectName, commonNameIndex))));            // {vm_covered}
-        }
+            result = tlsClientHostVerifyName(host, tlsCertCommonName(certificate));                                 // {vm_covered}
     }
     MEM_CONTEXT_TEMP_END();                                                                                         // {vm_covered}
 
@@ -257,46 +179,110 @@ tlsClientHostVerify(const String *host, X509 *certificate)
 }
 
 /***********************************************************************************************************************************
-Open connection if this is a new client or if the connection was closed by the server
+Authenticate server
+
+Adapted from PostgreSQL open_client_SSL() in src/interfaces/libpq/fe-secure-openssl.c.
 ***********************************************************************************************************************************/
-TlsSession *
-tlsClientOpen(TlsClient *this)
+static bool
+tlsClientAuth(const TlsClient *const this, SSL *const tlsSession)
 {
-    FUNCTION_LOG_BEGIN(logLevelTrace)
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+        FUNCTION_LOG_PARAM_P(VOID, tlsSession);
+    FUNCTION_LOG_END();
+
+    bool result = false;
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // Verify that the certificate presented by the server is valid
+        if (this->verifyPeer)                                                                                       // {vm_covered}
+        {
+            // Verify that the chain of trust leads to a valid CA
+            long int verifyResult = SSL_get_verify_result(tlsSession);                                              // {vm_covered}
+
+            if (verifyResult != X509_V_OK)                                                                          // {vm_covered}
+            {
+                THROW_FMT(                                                                                          // {vm_covered}
+                    CryptoError, "unable to verify certificate presented by '%s': [%ld] %s",                        // {vm_covered}
+                    strZ(ioClientName(this->ioClient)), verifyResult,                                               // {vm_covered}
+                    X509_verify_cert_error_string(verifyResult));                                                   // {vm_covered}
+            }
+
+            // Verify that the hostname appears in the certificate
+            X509 *certificate = SSL_get_peer_certificate(tlsSession);                                               // {vm_covered}
+            bool nameResult = tlsClientHostVerify(this->host, certificate);                                         // {vm_covered}
+            X509_free(certificate);                                                                                 // {vm_covered}
+
+            if (!nameResult)                                                                                        // {vm_covered}
+            {
+                THROW_FMT(                                                                                          // {vm_covered}
+                    CryptoError,                                                                                    // {vm_covered}
+                    "unable to find hostname '%s' in certificate common name or subject alternative names",         // {vm_covered}
+                    strZ(this->host));                                                                              // {vm_covered}
+            }
+        }
+
+        result = true;
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(BOOL, result);
+}
+
+/***********************************************************************************************************************************
+Open TLS session on a socket
+***********************************************************************************************************************************/
+static IoSession *
+tlsClientOpen(THIS_VOID)
+{
+    THIS(TlsClient);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(TLS_CLIENT, this);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
 
-    TlsSession *result = NULL;
-    SSL *session = NULL;
+    IoSession *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
         bool retry;
-        Wait *wait = waitNew(this->timeout);
+        Wait *wait = waitNew(this->timeoutConnect);
+        SSL *tlsSession = NULL;
 
         do
         {
             // Assume there will be no retry
             retry = false;
 
+            // Create the TLS session
+            tlsSession = SSL_new(this->context);
+            cryptoError(tlsSession == NULL, "unable to create TLS session");
+
+            // Set server host name used for validation
+            cryptoError(SSL_set_tlsext_host_name(tlsSession, strZ(this->host)) != 1, "unable to set TLS host name");
+
+            // Open TLS session
             TRY_BEGIN()
             {
-                // Open the socket session first since this is mostly likely to fail
-                SocketSession *socketSession = sckClientOpen(this->socketClient);
+                // Open the underlying session first since this is mostly likely to fail
+                IoSession *ioSession = NULL;
 
-                // Create internal TLS session. If there is a failure before the TlsSession object is created there may be a leak
-                // of the TLS session but this is likely to result in program termination so it doesn't seem worth coding for.
-                cryptoError((session = SSL_new(this->context)) == NULL, "unable to create TLS session");
+                TRY_BEGIN()
+                {
+                    ioSession = ioClientOpen(this->ioClient);
+                }
+                CATCH_ANY()
+                {
+                    SSL_free(tlsSession);
+                    RETHROW();
+                }
+                TRY_END();
 
-                // Set server host name used for validation
-                cryptoError(
-                    SSL_set_tlsext_host_name(session, strPtr(sckClientHost(this->socketClient))) != 1,
-                    "unable to set TLS host name");
-
-                // Create the TLS session
-                result = tlsSessionNew(session, socketSession, this->timeout);
+                // Open session
+                result = tlsSessionNew(tlsSession, ioSession, this->timeoutSession);
             }
             CATCH_ANY()
             {
@@ -308,7 +294,7 @@ tlsClientOpen(TlsClient *this)
                     LOG_DEBUG_FMT("retry %s: %s", errorTypeName(errorType()), errorMessage());
                     retry = true;
 
-                    tlsClientStatLocal.retry++;
+                    statInc(TLS_STAT_RETRY_STR);
                 }
                 else
                     RETHROW();
@@ -317,57 +303,118 @@ tlsClientOpen(TlsClient *this)
         }
         while (retry);
 
-        tlsSessionMove(result, memContextPrior());
+        // Authenticate TLS session
+        ASSERT(result != NULL);
+        ioSessionAuthenticatedSet(result, tlsClientAuth(this, tlsSession));
+
+        // Move session
+        ioSessionMove(result, memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
-    tlsClientStatLocal.session++;
+    statInc(TLS_STAT_SESSION_STR);
 
-    // Verify that the certificate presented by the server is valid
-    if (this->verifyPeer)                                                                                           // {vm_covered}
-    {
-        // Verify that the chain of trust leads to a valid CA
-        long int verifyResult = SSL_get_verify_result(session);                                                     // {vm_covered}
-
-        if (verifyResult != X509_V_OK)                                                                              // {vm_covered}
-        {
-            THROW_FMT(                                                                                              // {vm_covered}
-                CryptoError, "unable to verify certificate presented by '%s:%u': [%ld] %s",                         // {vm_covered}
-                strPtr(sckClientHost(this->socketClient)), sckClientPort(this->socketClient), verifyResult,         // {vm_covered}
-                X509_verify_cert_error_string(verifyResult));                                                       // {vm_covered}
-        }
-
-        // Verify that the hostname appears in the certificate
-        X509 *certificate = SSL_get_peer_certificate(session);                                                      // {vm_covered}
-        bool nameResult = tlsClientHostVerify(sckClientHost(this->socketClient), certificate);                      // {vm_covered}
-        X509_free(certificate);                                                                                     // {vm_covered}
-
-        if (!nameResult)                                                                                            // {vm_covered}
-        {
-            THROW_FMT(                                                                                              // {vm_covered}
-                CryptoError,                                                                                        // {vm_covered}
-                "unable to find hostname '%s' in certificate common name or subject alternative names",             // {vm_covered}
-                strPtr(sckClientHost(this->socketClient)));                                                         // {vm_covered}
-        }
-    }
-
-    FUNCTION_LOG_RETURN(TLS_SESSION, result);
+    FUNCTION_LOG_RETURN(IO_SESSION, result);
 }
 
 /**********************************************************************************************************************************/
-String *
-tlsClientStatStr(void)
+static const String *
+tlsClientName(THIS_VOID)
 {
-    FUNCTION_TEST_VOID();
+    THIS(TlsClient);
 
-    String *result = NULL;
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(TLS_CLIENT, this);
+    FUNCTION_TEST_END();
 
-    if (tlsClientStatLocal.object > 0)
-    {
-        result = strNewFmt(
-            "tls statistics: objects %" PRIu64 ", sessions %" PRIu64 ", retries %" PRIu64, tlsClientStatLocal.object,
-            tlsClientStatLocal.session, tlsClientStatLocal.retry);
-    }
+    ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(ioClientName(this->ioClient));
+}
+
+/***********************************************************************************************************************************
+Initialize TLS session with all required security features
+
+Adapted from PostgreSQL initialize_SSL() in src/interfaces/libpq/fe-secure-openssl.c.
+***********************************************************************************************************************************/
+static const IoClientInterface tlsClientInterface =
+{
+    .type = IO_CLIENT_TLS_TYPE,
+    .name = tlsClientName,
+    .open = tlsClientOpen,
+    .toLog = tlsClientToLog,
+};
+
+IoClient *
+tlsClientNew(
+    IoClient *const ioClient, const String *const host, const TimeMSec timeoutConnect, const TimeMSec timeoutSession,
+    const bool verifyPeer, const String *const caFile, const String *const caPath, const String *const certFile,
+    const String *const keyFile)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(IO_CLIENT, ioClient);
+        FUNCTION_LOG_PARAM(STRING, host);
+        FUNCTION_LOG_PARAM(TIME_MSEC, timeoutConnect);
+        FUNCTION_LOG_PARAM(TIME_MSEC, timeoutSession);
+        FUNCTION_LOG_PARAM(BOOL, verifyPeer);
+        FUNCTION_LOG_PARAM(STRING, caFile);
+        FUNCTION_LOG_PARAM(STRING, caPath);
+        FUNCTION_LOG_PARAM(STRING, certFile);
+        FUNCTION_LOG_PARAM(STRING, keyFile);
+    FUNCTION_LOG_END();
+
+    ASSERT(ioClient != NULL);
+
+    IoClient *this = NULL;
+
+    OBJ_NEW_BEGIN(TlsClient)
+    {
+        TlsClient *driver = OBJ_NEW_ALLOC();
+
+        *driver = (TlsClient)
+        {
+            .ioClient = ioClientMove(ioClient, MEM_CONTEXT_NEW()),
+            .host = strDup(host),
+            .timeoutConnect = timeoutConnect,
+            .timeoutSession = timeoutSession,
+            .verifyPeer = verifyPeer,
+            .context = tlsContext(),
+        };
+
+        // Set callback to free context
+        memContextCallbackSet(objMemContext(driver), tlsClientFreeResource, driver);
+
+        // Enable safe compatibility options
+        SSL_CTX_set_options(driver->context, SSL_OP_ALL);
+
+        // Set location of CA certificates if the server certificate will be verified
+        if (driver->verifyPeer)
+        {
+            // If the user specified a location
+            if (caFile != NULL || caPath != NULL)                                                                   // {vm_covered}
+            {
+                cryptoError(                                                                                        // {vm_covered}
+                    SSL_CTX_load_verify_locations(driver->context, strZNull(caFile), strZNull(caPath)) != 1,        // {vm_covered}
+                    "unable to set user-defined CA certificate location");                                          // {vm_covered}
+            }
+            // Else use the defaults
+            else
+            {
+                cryptoError(
+                    SSL_CTX_set_default_verify_paths(driver->context) != 1, "unable to set default CA certificate location");
+            }
+        }
+
+        // Load certificate and key, if specified
+        tlsCertKeyLoad(driver->context, certFile, keyFile);
+
+        // Increment stat
+        statInc(TLS_STAT_CLIENT_STR);
+
+        // Create client interface
+        this = ioClientNew(driver, &tlsClientInterface);
+    }
+    OBJ_NEW_END();
+
+    FUNCTION_LOG_RETURN(IO_CLIENT, this);
 }
