@@ -26,17 +26,20 @@ static LogLevel logLevelStdErr = logLevelError;
 static LogLevel logLevelFile = logLevelOff;
 static LogLevel logLevelAny = logLevelError;
 
-// Log file handles
-static int logHandleStdOut = STDOUT_FILENO;
-static int logHandleStdErr = STDERR_FILENO;
-DEBUG_UNIT_EXTERN int logHandleFile = -1;
+// Log file descriptors
+static int logFdStdOut = STDOUT_FILENO;
+static int logFdStdErr = STDERR_FILENO;
+static int logFdFile = -1;
 
 // Has the log file banner been written yet?
-DEBUG_UNIT_EXTERN bool logFileBanner = false;
+static bool logFileBanner = false;
 
 // Is the timestamp printed in the log?
 static bool logTimestamp = false;
 
+// Default process id if none is specified
+static unsigned int logProcessId = 0;
+
 // Size of the process id field
 static int logProcessSize = 2;
 
@@ -59,9 +62,7 @@ Log buffer -- used to format log header and message
 ***********************************************************************************************************************************/
 static char logBuffer[LOG_BUFFER_SIZE];
 
-/***********************************************************************************************************************************
-Convert log level to string and vice versa
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 #define LOG_LEVEL_TOTAL                                             (LOG_LEVEL_MAX + 1)
 
 static const char *const logLevelList[LOG_LEVEL_TOTAL] =
@@ -112,12 +113,8 @@ logLevelStr(LogLevel logLevel)
     FUNCTION_TEST_RETURN(logLevelList[logLevel]);
 }
 
-/***********************************************************************************************************************************
-Check if a log level will be logged to any output
-
-This is useful for log messages that are expensive to generate and should be skipped if they will be discarded.
-***********************************************************************************************************************************/
-DEBUG_UNIT_EXTERN void
+/**********************************************************************************************************************************/
+static void
 logAnySet(void)
 {
     FUNCTION_TEST_VOID();
@@ -127,7 +124,7 @@ logAnySet(void)
     if (logLevelStdErr > logLevelAny)
         logLevelAny = logLevelStdErr;
 
-    if (logLevelFile > logLevelAny && logHandleFile != -1)
+    if (logLevelFile > logLevelAny && logFdFile != -1)
         logLevelAny = logLevelFile;
 
     FUNCTION_TEST_RETURN_VOID();
@@ -145,19 +142,18 @@ logAny(LogLevel logLevel)
     FUNCTION_TEST_RETURN(logLevel <= logLevelAny);
 }
 
-/***********************************************************************************************************************************
-Initialize the log system
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 void
 logInit(
     LogLevel logLevelStdOutParam, LogLevel logLevelStdErrParam, LogLevel logLevelFileParam, bool logTimestampParam,
-    unsigned int logProcessMax, bool dryRunParam)
+    unsigned int processId, unsigned int logProcessMax, bool dryRunParam)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(ENUM, logLevelStdOutParam);
         FUNCTION_TEST_PARAM(ENUM, logLevelStdErrParam);
         FUNCTION_TEST_PARAM(ENUM, logLevelFileParam);
         FUNCTION_TEST_PARAM(BOOL, logTimestampParam);
+        FUNCTION_TEST_PARAM(UINT, processId);
         FUNCTION_TEST_PARAM(UINT, logProcessMax);
         FUNCTION_TEST_PARAM(BOOL, dryRunParam);
     FUNCTION_TEST_END();
@@ -165,12 +161,14 @@ logInit(
     ASSERT(logLevelStdOutParam <= LOG_LEVEL_MAX);
     ASSERT(logLevelStdErrParam <= LOG_LEVEL_MAX);
     ASSERT(logLevelFileParam <= LOG_LEVEL_MAX);
+    ASSERT(processId <= 999);
     ASSERT(logProcessMax <= 999);
 
     logLevelStdOut = logLevelStdOutParam;
     logLevelStdErr = logLevelStdErrParam;
     logLevelFile = logLevelFileParam;
     logTimestamp = logTimestampParam;
+    logProcessId = processId;
     logProcessSize = logProcessMax > 99 ? 3 : 2;
     logDryRun = dryRunParam;
 
@@ -187,11 +185,11 @@ logFileClose(void)
 {
     FUNCTION_TEST_VOID();
 
-    // Close the file handle if it is open
-    if (logHandleFile != -1)
+    // Close the file descriptor if it is open
+    if (logFdFile != -1)
     {
-        close(logHandleFile);
-        logHandleFile = -1;
+        close(logFdFile);
+        logFdFile = -1;
     }
 
     logAnySet();
@@ -199,11 +197,7 @@ logFileClose(void)
     FUNCTION_TEST_RETURN_VOID();
 }
 
-/***********************************************************************************************************************************
-Set the log file
-
-Returns true if file logging is off or the log file was successfully opened, false if the log file could not be opened.
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 bool
 logFileSet(const char *logFile)
 {
@@ -222,9 +216,9 @@ logFileSet(const char *logFile)
     if (logLevelFile != logLevelOff)
     {
         // Open the file and handle errors
-        logHandleFile = open(logFile, O_CREAT | O_APPEND | O_WRONLY, 0640);
+        logFdFile = open(logFile, O_CREAT | O_APPEND | O_WRONLY, 0640);
 
-        if (logHandleFile == -1)
+        if (logFdFile == -1)
         {
             int errNo = errno;
             LOG_WARN_FMT(
@@ -243,16 +237,14 @@ logFileSet(const char *logFile)
     FUNCTION_TEST_RETURN(result);
 }
 
-/***********************************************************************************************************************************
-Close the log system
-***********************************************************************************************************************************/
+/**********************************************************************************************************************************/
 void
 logClose(void)
 {
     FUNCTION_TEST_VOID();
 
     // Disable all logging
-    logInit(logLevelOff, logLevelOff, logLevelOff, false, 1, false);
+    logInit(logLevelOff, logLevelOff, logLevelOff, false, 0, 1, false);
 
     // Close the log file if it is open
     logFileClose();
@@ -284,21 +276,21 @@ logRange(LogLevel logLevel, LogLevel logRangeMin, LogLevel logRangeMax)
 Internal write function that handles errors
 ***********************************************************************************************************************************/
 static void
-logWrite(int handle, const char *message, size_t messageSize, const char *errorDetail)
+logWrite(int fd, const char *message, size_t messageSize, const char *errorDetail)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INT, handle);
+        FUNCTION_TEST_PARAM(INT, fd);
         FUNCTION_TEST_PARAM(STRINGZ, message);
         FUNCTION_TEST_PARAM(SIZE, messageSize);
         FUNCTION_TEST_PARAM(STRINGZ, errorDetail);
     FUNCTION_TEST_END();
 
-    ASSERT(handle != -1);
+    ASSERT(fd != -1);
     ASSERT(message != NULL);
     ASSERT(messageSize != 0);
     ASSERT(errorDetail != NULL);
 
-    if ((size_t)write(handle, message, messageSize) != messageSize)
+    if ((size_t)write(fd, message, messageSize) != messageSize)
         THROW_SYS_ERROR_FMT(FileWriteError, "unable to write %s", errorDetail);
 
     FUNCTION_TEST_RETURN_VOID();
@@ -308,19 +300,19 @@ logWrite(int handle, const char *message, size_t messageSize, const char *errorD
 Write out log message and indent subsequent lines
 ***********************************************************************************************************************************/
 static void
-logWriteIndent(int handle, const char *message, size_t indentSize, const char *errorDetail)
+logWriteIndent(int fd, const char *message, size_t indentSize, const char *errorDetail)
 {
     // Indent buffer -- used to write out indent space without having to loop
     static const char indentBuffer[] = "                                                                                          ";
 
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INT, handle);
+        FUNCTION_TEST_PARAM(INT, fd);
         FUNCTION_TEST_PARAM(STRINGZ, message);
         FUNCTION_TEST_PARAM(SIZE, indentSize);
         FUNCTION_TEST_PARAM(STRINGZ, errorDetail);
     FUNCTION_TEST_END();
 
-    ASSERT(handle != -1);
+    ASSERT(fd != -1);
     ASSERT(message != NULL);
     ASSERT(indentSize > 0 && indentSize < sizeof(indentBuffer));
     ASSERT(errorDetail != NULL);
@@ -332,11 +324,11 @@ logWriteIndent(int handle, const char *message, size_t indentSize, const char *e
     while (linefeedPtr != NULL)
     {
         if (!first)
-            logWrite(handle, indentBuffer, indentSize, errorDetail);
+            logWrite(fd, indentBuffer, indentSize, errorDetail);
         else
             first = false;
 
-        logWrite(handle, message, (size_t)(linefeedPtr - message + 1), errorDetail);
+        logWrite(fd, message, (size_t)(linefeedPtr - message + 1), errorDetail);
         message += (size_t)(linefeedPtr - message + 1);
 
         linefeedPtr = strchr(message, '\n');
@@ -379,19 +371,21 @@ logPre(LogLevel logLevel, unsigned int processId, const char *fileName, const ch
     // Add time
     if (logTimestamp)
     {
+        struct tm timePart;
         TimeMSec logTimeMSec = timeMSec();
         time_t logTimeSec = (time_t)(logTimeMSec / MSEC_PER_SEC);
 
         result.bufferPos += strftime(
-            logBuffer + result.bufferPos, sizeof(logBuffer) - result.bufferPos, "%Y-%m-%d %H:%M:%S", localtime(&logTimeSec));
+            logBuffer + result.bufferPos, sizeof(logBuffer) - result.bufferPos, "%Y-%m-%d %H:%M:%S",
+            localtime_r(&logTimeSec, &timePart));
         result.bufferPos += (size_t)snprintf(
             logBuffer + result.bufferPos, sizeof(logBuffer) - result.bufferPos, ".%03d ", (int)(logTimeMSec % 1000));
     }
 
     // Add process and aligned log level
     result.bufferPos += (size_t)snprintf(
-        logBuffer + result.bufferPos, sizeof(logBuffer) - result.bufferPos, "P%0*u %*s: ", logProcessSize, processId, 6,
-        logLevelStr(logLevel));
+        logBuffer + result.bufferPos, sizeof(logBuffer) - result.bufferPos, "P%0*u %*s: ", logProcessSize,
+        processId == (unsigned int)-1 ? logProcessId : processId, 6, logLevelStr(logLevel));
 
     // When writing to stderr the timestamp, process, and log level alignment will be skipped
     result.logBufferStdErr = logBuffer + result.bufferPos - strlen(logLevelStr(logLevel)) - 2;
@@ -455,33 +449,33 @@ logPost(LogPreResult *logData, LogLevel logLevel, LogLevel logRangeMin, LogLevel
         if (logRange(logLevelStdErr, logRangeMin, logRangeMax))
         {
             logWriteIndent(
-                logHandleStdErr, logData->logBufferStdErr, logData->indentSize - (size_t)(logData->logBufferStdErr - logBuffer),
+                logFdStdErr, logData->logBufferStdErr, logData->indentSize - (size_t)(logData->logBufferStdErr - logBuffer),
                 "log to stderr");
         }
     }
     else if (logLevel <= logLevelStdOut && logRange(logLevelStdOut, logRangeMin, logRangeMax))
-        logWriteIndent(logHandleStdOut, logBuffer, logData->indentSize, "log to stdout");
+        logWriteIndent(logFdStdOut, logBuffer, logData->indentSize, "log to stdout");
 
     // Log to file
-    if (logLevel <= logLevelFile && logHandleFile != -1 && logRange(logLevelFile, logRangeMin, logRangeMax))
+    if (logLevel <= logLevelFile && logFdFile != -1 && logRange(logLevelFile, logRangeMin, logRangeMax))
     {
         // If the banner has not been written
         if (!logFileBanner)
         {
             // Add a blank line if the file already has content
-            if (lseek(logHandleFile, 0, SEEK_END) > 0)
-                logWrite(logHandleFile, "\n", 1, "banner spacing to file");
+            if (lseek(logFdFile, 0, SEEK_END) > 0)
+                logWrite(logFdFile, "\n", 1, "banner spacing to file");
 
             // Write process start banner
             const char *banner = "-------------------PROCESS START-------------------\n";
 
-            logWrite(logHandleFile, banner, strlen(banner), "banner to file");
+            logWrite(logFdFile, banner, strlen(banner), "banner to file");
 
             // Mark banner as written
             logFileBanner = true;
         }
 
-        logWriteIndent(logHandleFile, logBuffer, logData->indentSize, "log to file");
+        logWriteIndent(logFdFile, logBuffer, logData->indentSize, "log to file");
     }
 
     FUNCTION_TEST_RETURN_VOID();
@@ -518,7 +512,6 @@ logInternal(
     FUNCTION_TEST_RETURN_VOID();
 }
 
-/**********************************************************************************************************************************/
 void
 logInternalFmt(
     LogLevel logLevel, LogLevel logRangeMin, LogLevel logRangeMax, unsigned int processId, const char *fileName,
