@@ -19,6 +19,7 @@ Configuration Load
 #include "config/config.intern.h"
 #include "config/load.h"
 #include "config/parse.h"
+#include "info/infoBackup.h"
 #include "storage/cifs/storage.h"
 #include "storage/posix/storage.h"
 #include "storage/helper.h"
@@ -68,16 +69,16 @@ cfgLoadUpdateOption(void)
 {
     FUNCTION_LOG_VOID(logLevelTrace);
 
-    // Make sure repo option is set for the default command role when it is not internal and more than one repo is configured or the
-    // first configured repo is not key 1. Filter out any commands where this does not apply.
-    if (!cfgCommandHelp() && cfgCommand() != cfgCmdInfo && cfgOptionValid(cfgOptRepo) && !cfgOptionTest(cfgOptRepo) &&
-        (cfgOptionGroupIdxTotal(cfgOptGrpRepo) > 1 || cfgOptionGroupIdxToKey(cfgOptGrpRepo, 0) != 1))
+    // Make sure the repo option is set for the stanza-delete command when more than one repo is configured or the first configured
+    // repo is not key 1.
+    if (!cfgCommandHelp() && cfgOptionValid(cfgOptRepo) && cfgCommand() == cfgCmdStanzaDelete &&
+        !cfgOptionTest(cfgOptRepo) && (cfgOptionGroupIdxTotal(cfgOptGrpRepo) > 1 || cfgOptionGroupIdxToKey(cfgOptGrpRepo, 0) != 1))
     {
         THROW_FMT(
             OptionRequiredError,
             "%s command requires option: " CFGOPT_REPO "\n"
             "HINT: this command requires a specific repository to operate on",
-            cfgCommandName(cfgCommand()));
+            cfgCommandName());
     }
 
     // If there is more than one repo configured
@@ -86,9 +87,9 @@ cfgLoadUpdateOption(void)
         for (unsigned int optionIdx = 0; optionIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); optionIdx++)
         {
             // If the repo is local and either posix or cifs
-            if (!(cfgOptionIdxTest(cfgOptRepoHost, optionIdx)) &&
-                (strEq(cfgOptionIdxStr(cfgOptRepoType, optionIdx), STORAGE_POSIX_TYPE_STR) ||
-                strEq(cfgOptionIdxStr(cfgOptRepoType, optionIdx), STORAGE_CIFS_TYPE_STR)))
+            if (!cfgOptionIdxTest(cfgOptRepoHost, optionIdx) &&
+                (cfgOptionIdxStrId(cfgOptRepoType, optionIdx) == STORAGE_POSIX_TYPE ||
+                 cfgOptionIdxStrId(cfgOptRepoType, optionIdx) == STORAGE_CIFS_TYPE))
             {
                 // Ensure a local repo does not have the same path as another local repo of the same type
                 for (unsigned int repoIdx = 0; repoIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); repoIdx++)
@@ -101,7 +102,7 @@ cfgLoadUpdateOption(void)
                             OptionInvalidValueError,
                             "local repo%u and repo%u paths are both '%s' but must be different",
                             cfgOptionGroupIdxToKey(cfgOptGrpRepo, optionIdx), cfgOptionGroupIdxToKey(cfgOptGrpRepo, repoIdx),
-                            strZ(cfgOptionIdxStr(cfgOptRepoPath, repoIdx)));
+                            strZ(cfgOptionIdxDisplay(cfgOptRepoPath, repoIdx)));
                     }
                 }
             }
@@ -145,8 +146,8 @@ cfgLoadUpdateOption(void)
                 OptionInvalidValueError,
                 "'%s' is not valid for '" CFGOPT_PROTOCOL_TIMEOUT "' option\nHINT '" CFGOPT_PROTOCOL_TIMEOUT "' option (%s)"
                     " should be greater than '" CFGOPT_DB_TIMEOUT "' option (%s).",
-                strZ(varStrForce(cfgOption(cfgOptProtocolTimeout))),  strZ(varStrForce(cfgOption(cfgOptProtocolTimeout))),
-                strZ(varStrForce(cfgOption(cfgOptDbTimeout))));
+                strZ(cfgOptionDisplay(cfgOptProtocolTimeout)),  strZ(cfgOptionDisplay(cfgOptProtocolTimeout)),
+                strZ(cfgOptionDisplay(cfgOptDbTimeout)));
         }
     }
 
@@ -186,7 +187,7 @@ cfgLoadUpdateOption(void)
                     "option '%s' is not set for '%s=%s', the repository may run out of space"
                     "\nHINT: to retain full backups indefinitely (without warning), set option '%s' to the maximum.",
                     cfgOptionIdxName(cfgOptRepoRetentionFull, optionIdx), cfgOptionIdxName(cfgOptRepoRetentionFullType, optionIdx),
-                    strZ(cfgOptionIdxStr(cfgOptRepoRetentionFullType, optionIdx)),
+                    strZ(cfgOptionIdxDisplay(cfgOptRepoRetentionFullType, optionIdx)),
                     cfgOptionIdxName(cfgOptRepoRetentionFull, optionIdx));
             }
         }
@@ -198,52 +199,57 @@ cfgLoadUpdateOption(void)
         // For each possible repo, check and adjust the settings as appropriate
         for (unsigned int optionIdx = 0; optionIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); optionIdx++)
         {
-            const String *archiveRetentionType = cfgOptionIdxStr(cfgOptRepoRetentionArchiveType, optionIdx);
+            const BackupType archiveRetentionType = (BackupType)cfgOptionIdxStrId(cfgOptRepoRetentionArchiveType, optionIdx);
 
             const String *msgArchiveOff = strNewFmt(
                 "WAL segments will not be expired: option '%s=%s' but", cfgOptionIdxName(cfgOptRepoRetentionArchiveType, optionIdx),
-                strZ(archiveRetentionType));
+                strZ(strIdToStr(archiveRetentionType)));
 
             // If the archive retention is not explicitly set then determine what it should be defaulted to
             if (!cfgOptionIdxTest(cfgOptRepoRetentionArchive, optionIdx))
             {
                 // If repo-retention-archive-type is default (full), then if repo-retention-full is set, set the
                 // repo-retention-archive to this value when retention-full-type is 'count', else ignore archiving. If
-                // retention-full-type is 'time' then the the expire command will default the archive retention accordingly.
-                if (strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_FULL))
+                // retention-full-type is 'time' then the expire command will default the archive retention accordingly.
+                switch (archiveRetentionType)
                 {
-                    if (strEqZ(
-                            cfgOptionIdxStr(cfgOptRepoRetentionFullType, optionIdx),
-                            CFGOPTVAL_TMP_REPO_RETENTION_FULL_TYPE_COUNT) &&
-                        cfgOptionIdxTest(cfgOptRepoRetentionFull, optionIdx))
+                    case backupTypeFull:
                     {
-                        cfgOptionIdxSet(cfgOptRepoRetentionArchive, optionIdx, cfgSourceDefault,
-                            VARUINT(cfgOptionIdxUInt(cfgOptRepoRetentionFull, optionIdx)));
-                    }
-                }
-                else if (strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_DIFF))
-                {
-                    // if repo-retention-diff is set then user must have set it
-                    if (cfgOptionIdxTest(cfgOptRepoRetentionDiff, optionIdx))
-                    {
-                        cfgOptionIdxSet(cfgOptRepoRetentionArchive, optionIdx, cfgSourceDefault,
-                            VARUINT(cfgOptionIdxUInt(cfgOptRepoRetentionDiff, optionIdx)));
-                    }
-                    else
-                    {
-                        LOG_WARN_FMT(
-                            "%s neither option '%s' nor option '%s' is set", strZ(msgArchiveOff),
-                            cfgOptionIdxName(cfgOptRepoRetentionArchive, optionIdx),
-                            cfgOptionIdxName(cfgOptRepoRetentionDiff, optionIdx));
-                    }
-                }
-                else
-                {
-                    CHECK(strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_INCR));
+                        if (cfgOptionIdxStrId(cfgOptRepoRetentionFullType, optionIdx) ==
+                                CFGOPTVAL_REPO_RETENTION_FULL_TYPE_COUNT &&
+                            cfgOptionIdxTest(cfgOptRepoRetentionFull, optionIdx))
+                        {
+                            cfgOptionIdxSet(cfgOptRepoRetentionArchive, optionIdx, cfgSourceDefault,
+                                VARINT64(cfgOptionIdxInt64(cfgOptRepoRetentionFull, optionIdx)));
+                        }
 
-                    LOG_WARN_FMT(
-                        "%s option '%s' is not set", strZ(msgArchiveOff),
-                        cfgOptionIdxName(cfgOptRepoRetentionArchive, optionIdx));
+                        break;
+                    }
+
+                    case backupTypeDiff:
+                    {
+                        // if repo-retention-diff is set then user must have set it
+                        if (cfgOptionIdxTest(cfgOptRepoRetentionDiff, optionIdx))
+                        {
+                            cfgOptionIdxSet(cfgOptRepoRetentionArchive, optionIdx, cfgSourceDefault,
+                                VARINT64(cfgOptionIdxInt64(cfgOptRepoRetentionDiff, optionIdx)));
+                        }
+                        else
+                        {
+                            LOG_WARN_FMT(
+                                "%s neither option '%s' nor option '%s' is set", strZ(msgArchiveOff),
+                                cfgOptionIdxName(cfgOptRepoRetentionArchive, optionIdx),
+                                cfgOptionIdxName(cfgOptRepoRetentionDiff, optionIdx));
+                        }
+
+                        break;
+                    }
+
+                    case backupTypeIncr:
+                        LOG_WARN_FMT(
+                            "%s option '%s' is not set", strZ(msgArchiveOff),
+                            cfgOptionIdxName(cfgOptRepoRetentionArchive, optionIdx));
+                        break;
                 }
             }
             else
@@ -251,14 +257,13 @@ cfgLoadUpdateOption(void)
                 // If repo-retention-archive is set then check repo-retention-archive-type and issue a warning if the
                 // corresponding setting is UNDEF since UNDEF means backups will not be expired but they should be in the
                 // practice of setting this value even though expiring the archive itself is OK and will be performed.
-                if ((strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_DIFF)) &&
-                    (!cfgOptionIdxTest(cfgOptRepoRetentionDiff, optionIdx)))
+                if (archiveRetentionType == backupTypeDiff && !cfgOptionIdxTest(cfgOptRepoRetentionDiff, optionIdx))
                 {
-                    LOG_WARN_FMT("option '%s' is not set for '%s=%s'\n"
+                    LOG_WARN_FMT(
+                        "option '%s' is not set for '%s=" CFGOPTVAL_TYPE_DIFF_Z "'\n"
                         "HINT: to retain differential backups indefinitely (without warning), set option '%s' to the maximum.",
                         cfgOptionIdxName(cfgOptRepoRetentionDiff, optionIdx),
                         cfgOptionIdxName(cfgOptRepoRetentionArchiveType, optionIdx),
-                        CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_DIFF,
                         cfgOptionIdxName(cfgOptRepoRetentionDiff, optionIdx));
                 }
             }
@@ -268,7 +273,7 @@ cfgLoadUpdateOption(void)
     // For each possible repo, error if an S3 bucket name contains dots
     for (unsigned int repoIdx = 0; repoIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); repoIdx++)
     {
-        if (cfgOptionIdxTest(cfgOptRepoS3Bucket, repoIdx) && cfgOptionIdxBool(cfgOptRepoS3VerifyTls, repoIdx) &&
+        if (cfgOptionIdxTest(cfgOptRepoS3Bucket, repoIdx) && cfgOptionIdxBool(cfgOptRepoStorageVerifyTls, repoIdx) &&
             strChr(cfgOptionIdxStr(cfgOptRepoS3Bucket, repoIdx), '.') != -1)
         {
             THROW_FMT(
@@ -277,7 +282,35 @@ cfgLoadUpdateOption(void)
                     "\nHINT: RFC-2818 forbids dots in wildcard matches."
                     "\nHINT: TLS/SSL verification cannot proceed with this bucket name."
                     "\nHINT: remove dots from the bucket name.",
-                strZ(cfgOptionIdxStr(cfgOptRepoS3Bucket, repoIdx)), cfgOptionIdxName(cfgOptRepoS3Bucket, repoIdx));
+                strZ(cfgOptionIdxDisplay(cfgOptRepoS3Bucket, repoIdx)), cfgOptionIdxName(cfgOptRepoS3Bucket, repoIdx));
+        }
+    }
+
+    // Set pg-host-port/repo-host-port default when pg-host-type/repo-host-type is tls. ??? This should be handled in the parser but
+    // it requires a default that depends on another option value and that is not currently possible.
+    #define HOST_PORT_TLS                                           8432
+
+    if (cfgOptionValid(cfgOptRepoHostPort))
+    {
+        for (unsigned int repoIdx = 0; repoIdx < cfgOptionGroupIdxTotal(cfgOptGrpRepo); repoIdx++)
+        {
+            if (cfgOptionIdxStrId(cfgOptRepoHostType, repoIdx) == CFGOPTVAL_REPO_HOST_TYPE_TLS &&
+                cfgOptionIdxSource(cfgOptRepoHostPort, repoIdx) == cfgSourceDefault)
+            {
+                cfgOptionIdxSet(cfgOptRepoHostPort, repoIdx, cfgSourceDefault, VARINT64(HOST_PORT_TLS));
+            }
+        }
+    }
+
+    if (cfgOptionValid(cfgOptPgHostPort))
+    {
+        for (unsigned int pgIdx = 0; pgIdx < cfgOptionGroupIdxTotal(cfgOptGrpPg); pgIdx++)
+        {
+            if (cfgOptionIdxStrId(cfgOptPgHostType, pgIdx) == CFGOPTVAL_PG_HOST_TYPE_TLS &&
+                cfgOptionIdxSource(cfgOptPgHostPort, pgIdx) == cfgSourceDefault)
+            {
+                cfgOptionIdxSet(cfgOptPgHostPort, pgIdx, cfgSourceDefault, VARINT64(HOST_PORT_TLS));
+            }
         }
     }
 
@@ -313,7 +346,7 @@ cfgLoadUpdateOption(void)
     {
         cfgOptionSet(
             cfgOptCompressLevel, cfgSourceDefault,
-            VARINT(compressLevelDefault(compressTypeEnum(cfgOptionStr(cfgOptCompressType)))));
+            VARINT64(compressLevelDefault(compressTypeEnum(cfgOptionStr(cfgOptCompressType)))));
     }
 
     FUNCTION_LOG_RETURN_VOID();
@@ -328,9 +361,10 @@ cfgLoadLogFile(void)
         MEM_CONTEXT_TEMP_BEGIN()
         {
             // Construct log filename prefix
-            String *logFile = strNewFmt(
+            String *logFile = strCatFmt(
+                strNew(),
                 "%s/%s-%s", strZ(cfgOptionStr(cfgOptLogPath)),
-                cfgOptionTest(cfgOptStanza) ? strZ(cfgOptionStr(cfgOptStanza)): "all", cfgCommandName(cfgCommand()));
+                cfgOptionTest(cfgOptStanza) ? strZ(cfgOptionStr(cfgOptStanza)): "all", cfgCommandName());
 
             // ??? Append async for local/remote archive async commands.  It would be good to find a more generic way to do this in
             // case the async role is added to more commands.
@@ -340,9 +374,9 @@ cfgLoadLogFile(void)
                     strCatFmt(logFile, "-async");
             }
 
-            // Add command role if it is not default
-            if (cfgCommandRole() != cfgCmdRoleDefault)
-                strCatFmt(logFile, "-%s", strZ(cfgCommandRoleStr(cfgCommandRole())));
+            // Add command role if it is not main
+            if (cfgCommandRole() != cfgCmdRoleMain)
+                strCatFmt(logFile, "-%s", strZ(cfgParseCommandRoleStr(cfgCommandRole())));
 
             // Add process id if local or remote role
             if (cfgCommandRole() == cfgCmdRoleLocal || cfgCommandRole() == cfgCmdRoleRemote)
@@ -373,13 +407,6 @@ cfgLoad(unsigned int argListSize, const char *argList[])
         // Parse config from command line and config file
         configParse(storageLocal(), argListSize, argList, true);
 
-        // Check that only repo1 is configured. This is temporary until the multi-repo support is finalized.
-        if (cfgCommandRole() == cfgCmdRoleDefault && cfgOptionGroupValid(cfgOptGrpRepo) &&
-            (cfgOptionGroupIdxTotal(cfgOptGrpRepo) > 1 || cfgOptionGroupIdxToKey(cfgOptGrpRepo, 0) != 1))
-        {
-            THROW_FMT(OptionInvalidValueError, "only repo1 may be configured");
-        }
-
         // Initialize dry-run mode for storage when valid for the current command
         storageHelperDryRunInit(cfgOptionValid(cfgOptDryRun) && cfgOptionBool(cfgOptDryRun));
 
