@@ -7,57 +7,46 @@ Configuration Protocol Handler
 #include "common/io/io.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/type/json.h"
 #include "config/config.intern.h"
 #include "config/parse.h"
 #include "config/protocol.h"
 
-/***********************************************************************************************************************************
-Constants
-***********************************************************************************************************************************/
-STRING_EXTERN(PROTOCOL_COMMAND_CONFIG_OPTION_STR,                   PROTOCOL_COMMAND_CONFIG_OPTION);
-
 /**********************************************************************************************************************************/
-bool
-configProtocol(const String *command, const VariantList *paramList, ProtocolServer *server)
+void
+configOptionProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(STRING, command);
-        FUNCTION_LOG_PARAM(VARIANT_LIST, paramList);
+        FUNCTION_LOG_PARAM(PACK_READ, param);
         FUNCTION_LOG_PARAM(PROTOCOL_SERVER, server);
     FUNCTION_LOG_END();
 
-    ASSERT(command != NULL);
-
-    // Attempt to satisfy the request -- we may get requests that are meant for other handlers
-    bool found = true;
+    ASSERT(param != NULL);
+    ASSERT(server != NULL);
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        if (strEq(command, PROTOCOL_COMMAND_CONFIG_OPTION_STR))
+        VariantList *optionList = varLstNew();
+
+        while (pckReadNext(param))
         {
-            VariantList *optionList = varLstNew();
+            CfgParseOptionResult option = cfgParseOptionP(pckReadStrP(param));
+            CHECK(option.found);
 
-            for (unsigned int optionIdx = 0; optionIdx < varLstSize(paramList); optionIdx++)
-            {
-                CfgParseOptionResult option = cfgParseOption(varStr(varLstGet(paramList, optionIdx)));
-                CHECK(option.found);
-
-                varLstAdd(optionList, varDup(cfgOptionIdx(option.id, cfgOptionKeyToIdx(option.id, option.keyIdx + 1))));
-            }
-
-            protocolServerResponse(server, varNewVarLst(optionList));
+            varLstAdd(optionList, varDup(cfgOptionIdxVar(option.id, cfgOptionKeyToIdx(option.id, option.keyIdx + 1))));
         }
-        else
-            found = false;
+
+        protocolServerDataPut(server, pckWriteStrP(protocolPackNew(), jsonFromVar(varNewVarLst(optionList))));
+        protocolServerDataEndPut(server);
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(BOOL, found);
+    FUNCTION_LOG_RETURN_VOID();
 }
 
 /**********************************************************************************************************************************/
 VariantList *
-configProtocolOption(ProtocolClient *client, const VariantList *paramList)
+configOptionRemote(ProtocolClient *client, const VariantList *paramList)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(PROTOCOL_CLIENT, client);
@@ -68,14 +57,15 @@ configProtocolOption(ProtocolClient *client, const VariantList *paramList)
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_CONFIG_OPTION_STR);
+        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_CONFIG_OPTION);
+        PackWrite *const param = protocolCommandParam(command);
 
         for (unsigned int paramIdx = 0; paramIdx < varLstSize(paramList); paramIdx++)
-            protocolCommandParamAdd(command, varLstGet(paramList, paramIdx));
+            pckWriteStrP(param, varStr(varLstGet(paramList, paramIdx)));
 
         MEM_CONTEXT_PRIOR_BEGIN()
         {
-            result = varVarLst(protocolClientExecute(client, command, true));
+            result = varVarLst(jsonToVar(pckReadStrP(protocolClientExecute(client, command, true))));
         }
         MEM_CONTEXT_PRIOR_END();
     }
