@@ -5,8 +5,7 @@ Database Client
 
 #include "common/debug.h"
 #include "common/log.h"
-#include "common/memContext.h"
-#include "common/type/object.h"
+#include "common/type/json.h"
 #include "common/wait.h"
 #include "db/db.h"
 #include "db/protocol.h"
@@ -25,32 +24,34 @@ Object type
 ***********************************************************************************************************************************/
 struct Db
 {
-    MemContext *memContext;
+    DbPub pub;                                                      // Publicly accessible variables
     PgClient *client;                                               // Local PostgreSQL client
     ProtocolClient *remoteClient;                                   // Protocol client for remote db queries
     unsigned int remoteIdx;                                         // Index provided by the remote on open for subsequent calls
     const String *applicationName;                                  // Used to identify this connection in PostgreSQL
-
-    unsigned int pgVersion;                                         // Version as reported by the database
-    const String *pgDataPath;                                       // Data directory reported by the database
-    const String *archiveMode;                                      // The archive_mode reported by the database
-    const String *archiveCommand;                                   // The archive_command reported by the database
 };
 
-OBJECT_DEFINE_MOVE(DB);
-OBJECT_DEFINE_FREE(DB);
-
 /***********************************************************************************************************************************
 Close protocol connection.  No need to close a locally created PgClient since it has its own destructor.
 ***********************************************************************************************************************************/
-OBJECT_DEFINE_FREE_RESOURCE_BEGIN(DB, LOG, logLevelTrace)
+static void
+dbFreeResource(THIS_VOID)
 {
-    ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_CLOSE_STR);
-    protocolCommandParamAdd(command, VARUINT(this->remoteIdx));
+    THIS(Db);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(DB, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_CLOSE);
+    pckWriteU32P(protocolCommandParam(command), this->remoteIdx);
 
     protocolClientExecute(this->remoteClient, command, false);
+
+    FUNCTION_LOG_RETURN_VOID();
 }
-OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
 
 /**********************************************************************************************************************************/
 Db *
@@ -67,20 +68,23 @@ dbNew(PgClient *client, ProtocolClient *remoteClient, const String *applicationN
 
     Db *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("Db")
+    OBJ_NEW_BEGIN(Db)
     {
-        this = memNew(sizeof(Db));
+        this = OBJ_NEW_ALLOC();
 
         *this = (Db)
         {
-            .memContext = memContextCurrent(),
+            .pub =
+            {
+                .memContext = memContextCurrent(),
+            },
             .remoteClient = remoteClient,
             .applicationName = strDup(applicationName),
         };
 
-        this->client = pgClientMove(client, this->memContext);
+        this->client = pgClientMove(client, this->pub.memContext);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(DB, this);
 }
@@ -104,11 +108,13 @@ dbQuery(Db *this, const String *query)
     // Query remotely
     if (this->remoteClient != NULL)
     {
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_QUERY_STR);
-        protocolCommandParamAdd(command, VARUINT(this->remoteIdx));
-        protocolCommandParamAdd(command, VARSTR(query));
+        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_QUERY);
+        PackWrite *const param = protocolCommandParam(command);
 
-        result = varVarLst(protocolClientExecute(this->remoteClient, command, true));
+        pckWriteU32P(param, this->remoteIdx);
+        pckWriteStrP(param, query);
+
+        result = varVarLst(jsonToVar(pckReadStrP(protocolClientExecute(this->remoteClient, command, true))));
     }
     // Else locally
     else
@@ -194,11 +200,11 @@ dbOpen(Db *this)
         // Open the connection
         if (this->remoteClient != NULL)
         {
-            ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_OPEN_STR);
-            this->remoteIdx = varUIntForce(protocolClientExecute(this->remoteClient, command, true));
+            ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_DB_OPEN);
+            this->remoteIdx = pckReadU32P(protocolClientExecute(this->remoteClient, command, true));
 
             // Set a callback to notify the remote when a connection is closed
-            memContextCallbackSet(this->memContext, dbFreeResource, this);
+            memContextCallbackSet(this->pub.memContext, dbFreeResource, this);
         }
         else
             pgClientOpen(this->client);
@@ -234,25 +240,25 @@ dbOpen(Db *this)
 
         // Strip the minor version off since we don't need it.  In the future it might be a good idea to warn users when they are
         // running an old minor version.
-        this->pgVersion = varUIntForce(varLstGet(row, 0)) / 100 * 100;
+        this->pub.pgVersion = varUIntForce(varLstGet(row, 0)) / 100 * 100;
 
         // Store the data directory that PostgreSQL is running in, the archive mode, and archive command. These can be compared to
         // the configured pgBackRest directory, and archive settings checked for validity, when validating the configuration.
-        MEM_CONTEXT_BEGIN(this->memContext)
+        MEM_CONTEXT_BEGIN(this->pub.memContext)
         {
-            this->pgDataPath = strDup(varStr(varLstGet(row, 1)));
-            this->archiveMode = strDup(varStr(varLstGet(row, 2)));
-            this->archiveCommand = strDup(varStr(varLstGet(row, 3)));
+            this->pub.pgDataPath = strDup(varStr(varLstGet(row, 1)));
+            this->pub.archiveMode = strDup(varStr(varLstGet(row, 2)));
+            this->pub.archiveCommand = strDup(varStr(varLstGet(row, 3)));
         }
         MEM_CONTEXT_END();
 
         // Set application name to help identify the backup session
-        if (this->pgVersion >= PG_VERSION_APPLICATION_NAME)
+        if (dbPgVersion(this) >= PG_VERSION_APPLICATION_NAME)
             dbExec(this, strNewFmt("set application_name = '%s'", strZ(this->applicationName)));
 
         // There is no need to have parallelism enabled in a backup session. In particular, 9.6 marks pg_stop_backup() as
         // parallel-safe but an error will be thrown if pg_stop_backup() is run in a worker.
-        if (this->pgVersion >= PG_VERSION_PARALLEL_QUERY)
+        if (dbPgVersion(this) >= PG_VERSION_PARALLEL_QUERY)
             dbExec(this, STRDEF("set max_parallel_workers_per_gather = 0"));
     }
     MEM_CONTEXT_TEMP_END();
@@ -271,7 +277,8 @@ dbBackupStartQuery(unsigned int pgVersion, bool startFast)
     FUNCTION_TEST_END();
 
     // Build query to return start lsn and WAL segment name
-    String *result = strNewFmt(
+    String *result = strCatFmt(
+        strNew(),
         "select lsn::text as lsn,\n"
         "       pg_catalog.pg_%sfile_name(lsn)::text as wal_segment_name\n"
         "  from pg_catalog.pg_start_backup('" PROJECT_NAME " backup started at ' || current_timestamp",
@@ -296,11 +303,6 @@ dbBackupStartQuery(unsigned int pgVersion, bool startFast)
     FUNCTION_TEST_RETURN(result);
 }
 
-#define FUNCTION_LOG_DB_BACKUP_START_RESULT_TYPE                                                                                   \
-    DbBackupStartResult
-#define FUNCTION_LOG_DB_BACKUP_START_RESULT_FORMAT(value, buffer, bufferSize)                                                      \
-    objToLog(&value, "DbBackupStartResult", buffer, bufferSize)
-
 DbBackupStartResult
 dbBackupStart(Db *this, bool startFast, bool stopAuto)
 {
@@ -359,7 +361,7 @@ dbBackupStart(Db *this, bool startFast, bool stopAuto)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(DB_BACKUP_START_RESULT, result);
+    FUNCTION_LOG_RETURN_STRUCT(result);
 }
 /**********************************************************************************************************************************/
 // Helper to build stop backup query
@@ -371,7 +373,8 @@ dbBackupStopQuery(unsigned int pgVersion)
     FUNCTION_TEST_END();
 
     // Build query to return start lsn and WAL segment name
-    String *result = strNewFmt(
+    String *result = strCatFmt(
+        strNew(),
         "select lsn::text as lsn,\n"
         "       pg_catalog.pg_%sfile_name(lsn)::text as wal_segment_name",
         strZ(pgWalName(pgVersion)));
@@ -409,11 +412,6 @@ dbBackupStopQuery(unsigned int pgVersion)
     FUNCTION_TEST_RETURN(result);
 }
 
-#define FUNCTION_LOG_DB_BACKUP_STOP_RESULT_TYPE                                                                                   \
-    DbBackupStopResult
-#define FUNCTION_LOG_DB_BACKUP_STOP_RESULT_FORMAT(value, buffer, bufferSize)                                                      \
-    objToLog(&value, "DbBackupStopResult", buffer, bufferSize)
-
 DbBackupStopResult
 dbBackupStop(Db *this)
 {
@@ -452,7 +450,7 @@ dbBackupStop(Db *this)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(DB_BACKUP_STOP_RESULT, result);
+    FUNCTION_LOG_RETURN_STRUCT(result);
 }
 
 /**********************************************************************************************************************************/
@@ -467,7 +465,7 @@ dbIsStandby(Db *this)
 
     bool result = false;
 
-    if (this->pgVersion >= PG_VERSION_HOT_STANDBY)
+    if (dbPgVersion(this) >= PG_VERSION_HOT_STANDBY)
     {
         result = varBool(dbQueryColumn(this, STRDEF("select pg_catalog.pg_is_in_recovery()")));
     }
@@ -514,7 +512,8 @@ dbReplayWait(Db *this, const String *targetLsn, TimeMSec timeout)
         do
         {
             // Build the query
-            String *query = strNewFmt(
+            String *query = strCatFmt(
+                strNew(),
                 "select replayLsn::text,\n"
                 "       (replayLsn > '%s')::bool as targetReached",
                 strZ(targetLsn));
@@ -539,7 +538,7 @@ dbReplayWait(Db *this, const String *targetLsn, TimeMSec timeout)
             replayLsn = varStr(varLstGet(row, 0));
 
             // Error when replayLsn is null which indicates that this is not a standby.  This should have been sorted out before we
-            // connected but it's possible that the standy was promoted in the meantime.
+            // connected but it's possible that the standby was promoted in the meantime.
             if (replayLsn == NULL)
             {
                 THROW_FMT(
@@ -563,37 +562,47 @@ dbReplayWait(Db *this, const String *targetLsn, TimeMSec timeout)
         if (!targetReached)
         {
             THROW_FMT(
-                ArchiveTimeoutError, "timeout before standby replayed to %s - only reached %s", strZ(targetLsn),
-                strZ(replayLsn));
+                ArchiveTimeoutError,
+                "timeout before standby replayed to %s - only reached %s\n"
+                "HINT: is replication running and current on the standby?\n"
+                "HINT: disable the 'backup-standby' option to backup directly from the primary.",
+                strZ(targetLsn), strZ(replayLsn));
         }
 
         // Perform a checkpoint
         dbExec(this, STRDEF("checkpoint"));
 
-        // On PostgreSQL >= 9.6 the checkpoint location can be verified
-        //
-        // ??? We have seen one instance where this check failed.  Is there any chance that the replayed position could be ahead of
-        // the checkpoint recorded in pg_control?  It seems possible since it would be safer if the checkpoint in pg_control was
-        // behind rather than ahead, so add a loop to keep checking until the checkpoint has been recorded or timeout.
+        // On PostgreSQL >= 9.6 the checkpoint location can be verified so loop until lsn has been reached or timeout
         if (dbPgVersion(this) >= PG_VERSION_96)
         {
-            // Build the query
-            const String *query = strNewFmt(
-                "select (checkpoint_%s > '%s')::bool as targetReached,\n"
-                "       checkpoint_%s::text as checkpointLsn\n"
-                "  from pg_catalog.pg_control_checkpoint()",
-                lsnName, strZ(targetLsn), lsnName);
+            wait = waitNew(timeout);
+            targetReached = false;
+            const String *checkpointLsn = NULL;
 
-            // Execute query
-            VariantList *row = dbQueryRow(this, query);
+            do
+            {
+                // Build the query
+                const String *query = strNewFmt(
+                    "select (checkpoint_%s > '%s')::bool as targetReached,\n"
+                    "       checkpoint_%s::text as checkpointLsn\n"
+                    "  from pg_catalog.pg_control_checkpoint()",
+                    lsnName, strZ(targetLsn), lsnName);
 
-            // Verify target was reached
-            if (!varBool(varLstGet(row, 0)))
+                // Execute the query and get checkpointLsn
+                VariantList *row = dbQueryRow(this, query);
+                targetReached = varBool(varLstGet(row, 0));
+                checkpointLsn = varStr(varLstGet(row, 1));
+
+                protocolKeepAlive();
+            }
+            while (!targetReached && waitMore(wait));
+
+            // Error if a timeout occurred before the target lsn was reached
+            if (!targetReached)
             {
                 THROW_FMT(
-                    ArchiveTimeoutError,
-                    "the checkpoint lsn %s is less than the target lsn %s even though the replay lsn is %s",
-                    strZ(varStr(varLstGet(row, 1))), strZ(targetLsn), strZ(replayLsn));
+                    ArchiveTimeoutError, "timeout before standby checkpoint lsn reached %s - only reached %s", strZ(targetLsn),
+                    strZ(checkpointLsn));
             }
         }
     }
@@ -641,11 +650,11 @@ dbWalSwitch(Db *this)
     {
         // Create a restore point to ensure current WAL will be archived.  For versions < 9.1 activity will need to be generated by
         // the user if there have been no writes since the last WAL switch.
-        if (this->pgVersion >= PG_VERSION_RESTORE_POINT)
+        if (dbPgVersion(this) >= PG_VERSION_RESTORE_POINT)
             dbQueryColumn(this, STRDEF("select pg_catalog.pg_create_restore_point('" PROJECT_NAME " Archive Check')::text"));
 
         // Request a WAL segment switch
-        const char *walName = strZ(pgWalName(this->pgVersion));
+        const char *walName = strZ(pgWalName(dbPgVersion(this)));
         const String *walFileName = varStr(
             dbQueryColumn(this, strNewFmt("select pg_catalog.pg_%sfile_name(pg_catalog.pg_switch_%s())::text", walName, walName)));
 
@@ -661,58 +670,6 @@ dbWalSwitch(Db *this)
     FUNCTION_LOG_RETURN(STRING, result);
 }
 
-/**********************************************************************************************************************************/
-const String *
-dbPgDataPath(const Db *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(DB, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->pgDataPath);
-}
-
-/**********************************************************************************************************************************/
-unsigned int
-dbPgVersion(const Db *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(DB, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->pgVersion);
-}
-
-/**********************************************************************************************************************************/
-const String *
-dbArchiveMode(const Db *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(DB, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->archiveMode);
-}
-
-/**********************************************************************************************************************************/
-const String *
-dbArchiveCommand(const Db *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(DB, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->archiveCommand);
-}
-
 /**********************************************************************************************************************************/
 String *
 dbToLog(const Db *this)
