@@ -10,11 +10,9 @@ Info Handler
 #include "common/type/convert.h"
 #include "common/crypto/hash.h"
 #include "common/debug.h"
-#include "common/encode.h"
-#include "common/io/filter/filter.intern.h"
+#include "common/io/filter/filter.h"
 #include "common/ini.h"
 #include "common/log.h"
-#include "common/memContext.h"
 #include "common/type/json.h"
 #include "common/type/object.h"
 #include "info/info.h"
@@ -39,9 +37,8 @@ Object types
 ***********************************************************************************************************************************/
 struct Info
 {
+    InfoPub pub;                                                    // Publicly accessible variables
     MemContext *memContext;                                         // Mem context
-    const String *backrestVersion;                                  // pgBackRest version
-    const String *cipherPass;                                       // Cipher passphrase if set
 };
 
 struct InfoSave
@@ -117,7 +114,7 @@ infoNewInternal(void)
 {
     FUNCTION_TEST_VOID();
 
-    Info *this = memNew(sizeof(Info));
+    Info *this = OBJ_NEW_ALLOC();
 
     *this = (Info)
     {
@@ -137,15 +134,15 @@ infoNew(const String *cipherPass)
 
     Info *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("Info")
+    OBJ_NEW_BEGIN(Info)
     {
         this = infoNewInternal();
 
         // Cipher used to encrypt/decrypt subsequent dependent files. Value may be NULL.
         infoCipherPassSet(this, cipherPass);
-        this->backrestVersion = STRDEF(PROJECT_VERSION);
+        this->pub.backrestVersion = STRDEF(PROJECT_VERSION);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(INFO, this);
 }
@@ -220,7 +217,7 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
         {
             MEM_CONTEXT_BEGIN(loadData->info->memContext)
             {
-                loadData->info->backrestVersion = strDup(varStr(valueVar));
+                loadData->info->pub.backrestVersion = strDup(varStr(valueVar));
             }
             MEM_CONTEXT_END();
         }
@@ -244,7 +241,7 @@ infoLoadCallback(void *data, const String *section, const String *key, const Str
         {
             MEM_CONTEXT_BEGIN(loadData->info->memContext)
             {
-                loadData->info->cipherPass = strDup(varStr(valueVar));
+                loadData->info->pub.cipherPass = strDup(varStr(valueVar));
             }
             MEM_CONTEXT_END();
         }
@@ -272,7 +269,7 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
 
     Info *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("Info")
+    OBJ_NEW_BEGIN(Info)
     {
         this = infoNewInternal();
 
@@ -303,7 +300,7 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
             INFO_CHECKSUM_END(data.checksumActual);
 
             // Verify the checksum
-            const String *checksumActual = varStr(ioFilterResult(data.checksumActual));
+            const String *checksumActual = pckReadStrP(pckReadNew(ioFilterResult(data.checksumActual)));
 
             if (data.checksumExpected == NULL)
                 THROW_FMT(ChecksumError, "invalid checksum, actual '%s' but no checksum found", strZ(checksumActual));
@@ -316,7 +313,7 @@ infoNewLoad(IoRead *read, InfoLoadNewCallback *callbackFunction, void *callbackD
         }
         MEM_CONTEXT_TEMP_END();
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(INFO, this);
 }
@@ -351,6 +348,9 @@ infoSaveValue(InfoSave *infoSaveData, const String *section, const String *key,
     ASSERT(section != NULL);
     ASSERT(key != NULL);
     ASSERT(jsonValue != NULL);
+    ASSERT(strSize(jsonValue) != 0);
+    // The JSON value must not be an array because this may be confused with a section in the ini file
+    ASSERT(strZ(jsonValue)[0] != '[');
 
     // Save section
     if (infoSaveData->sectionLast == NULL || !strEq(section, infoSaveData->sectionLast))
@@ -422,10 +422,10 @@ infoSave(Info *this, IoWrite *write, InfoSaveCallback *callbackFunction, void *c
         infoSaveValue(&data, INFO_SECTION_BACKREST_STR, INFO_KEY_VERSION_STR, jsonFromStr(STRDEF(PROJECT_VERSION)));
 
         // Add cipher passphrase if defined
-        if (this->cipherPass != NULL)
+        if (infoCipherPass(this) != NULL)
         {
             callbackFunction(callbackData, INFO_SECTION_CIPHER_STR, &data);
-            infoSaveValue(&data, INFO_SECTION_CIPHER_STR, INFO_KEY_CIPHER_PASS_STR, jsonFromStr(this->cipherPass));
+            infoSaveValue(&data, INFO_SECTION_CIPHER_STR, INFO_KEY_CIPHER_PASS_STR, jsonFromStr(infoCipherPass(this)));
         }
 
         // Flush out any additional sections
@@ -435,7 +435,7 @@ infoSave(Info *this, IoWrite *write, InfoSaveCallback *callbackFunction, void *c
         INFO_CHECKSUM_END(data.checksum);
 
         ioWrite(data.write, BUFSTRDEF("\n[" INFO_SECTION_BACKREST "]\n" INFO_KEY_CHECKSUM "="));
-        ioWriteLine(data.write, BUFSTR(jsonFromVar(ioFilterResult(data.checksum))));
+        ioWriteLine(data.write, BUFSTR(jsonFromStr(pckReadStrP(pckReadNew(ioFilterResult(data.checksum))))));
 
         // Close the file
         ioWriteClose(data.write);
@@ -448,18 +448,6 @@ infoSave(Info *this, IoWrite *write, InfoSaveCallback *callbackFunction, void *c
 /***********************************************************************************************************************************
 Getters/Setters
 ***********************************************************************************************************************************/
-const String *
-infoCipherPass(const Info *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INFO, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->cipherPass);
-}
-
 void
 infoCipherPassSet(Info *this, const String *cipherPass)
 {
@@ -472,25 +460,13 @@ infoCipherPassSet(Info *this, const String *cipherPass)
 
     MEM_CONTEXT_BEGIN(this->memContext)
     {
-        this->cipherPass = strDup(cipherPass);
+        this->pub.cipherPass = strDup(cipherPass);
     }
     MEM_CONTEXT_END();
 
     FUNCTION_TEST_RETURN_VOID();
 }
 
-const String *
-infoBackrestVersion(const Info *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INFO, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->backrestVersion);
-}
-
 /**********************************************************************************************************************************/
 void
 infoLoad(const String *error, InfoLoadCallback *callbackFunction, void *callbackData)
@@ -521,8 +497,8 @@ infoLoad(const String *error, InfoLoadCallback *callbackFunction, void *callback
                 loaded = callbackFunction(callbackData, try);
                 done = true;
 
-                // There must be at least one attempt to the load file
-                ASSERT(loaded || try > 0);
+                // There must be at least one attempt to load the file
+                CHECK(loaded || try > 0);
             }
             CATCH_ANY()
             {
@@ -530,7 +506,7 @@ infoLoad(const String *error, InfoLoadCallback *callbackFunction, void *callback
                 if (loadErrorType == NULL)
                 {
                     loadErrorType = errorType();
-                    loadErrorMessage = strNewFmt("%s:", strZ(error));
+                    loadErrorMessage = strCatFmt(strNew(), "%s:", strZ(error));
                 }
                 // Else if the error type is different
                 else if (loadErrorType != errorType())
