@@ -4,14 +4,24 @@ Backup Info Handler
 #ifndef INFO_INFOBACKUP_H
 #define INFO_INFOBACKUP_H
 
+#include "common/type/stringId.h"
+
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
-#define INFO_BACKUP_TYPE                                            InfoBackup
-#define INFO_BACKUP_PREFIX                                          infoBackup
-
 typedef struct InfoBackup InfoBackup;
 
+/***********************************************************************************************************************************
+Backup type enum
+***********************************************************************************************************************************/
+typedef enum
+{
+    backupTypeFull = STRID5("full", 0x632a60),
+    backupTypeDiff = STRID5("diff", 0x319240),
+    backupTypeIncr = STRID5("incr", 0x90dc90),
+} BackupType;
+
+#include "common/type/object.h"
 #include "common/type/string.h"
 #include "common/type/stringList.h"
 #include "info/infoPg.h"
@@ -47,7 +57,8 @@ typedef struct InfoBackupData
     StringList *backupReference;
     time_t backupTimestampStart;
     time_t backupTimestampStop;
-    const String *backupType;
+    BackupType backupType;
+    const Variant *backupError;                                     // Were errors detected during the backup?
     bool optionArchiveCheck;
     bool optionArchiveCopy;
     bool optionBackupStandby;
@@ -60,7 +71,54 @@ typedef struct InfoBackupData
 /***********************************************************************************************************************************
 Constructors
 ***********************************************************************************************************************************/
-InfoBackup *infoBackupNew(unsigned int pgVersion, uint64_t pgSystemId, const String *cipherPassSub);
+InfoBackup *infoBackupNew(unsigned int pgVersion, uint64_t pgSystemId, unsigned int pgCatalogVersion, const String *cipherPassSub);
+
+// Create new object and load contents from IoRead
+InfoBackup *infoBackupNewLoad(IoRead *read);
+
+/***********************************************************************************************************************************
+Getters/Setters
+***********************************************************************************************************************************/
+typedef struct InfoBackupPub
+{
+    MemContext *memContext;                                         // Mem context
+    InfoPg *infoPg;                                                 // Contents of the DB data
+    List *backup;                                                   // List of current backups and their associated data
+} InfoBackupPub;
+
+// PostgreSQL info
+__attribute__((always_inline)) static inline InfoPg *
+infoBackupPg(const InfoBackup *const this)
+{
+    return THIS_PUB(InfoBackup)->infoPg;
+}
+
+InfoBackup *infoBackupPgSet(InfoBackup *this, unsigned int pgVersion, uint64_t pgSystemId, unsigned int pgCatalogVersion);
+
+// Cipher passphrase
+__attribute__((always_inline)) static inline const String *
+infoBackupCipherPass(const InfoBackup *const this)
+{
+    return infoPgCipherPass(infoBackupPg(this));
+}
+
+// Return a structure of the backup data from a specific index
+InfoBackupData infoBackupData(const InfoBackup *this, unsigned int backupDataIdx);
+
+// Return a pointer to a structure from the current backup data given a label, else NULL
+__attribute__((always_inline)) static inline InfoBackupData *
+infoBackupDataByLabel(const InfoBackup *const this, const String *const backupLabel)
+{
+    ASSERT_INLINE(backupLabel != NULL);
+    return lstFind(THIS_PUB(InfoBackup)->backup, &backupLabel);
+}
+
+// Get total current backups
+__attribute__((always_inline)) static inline unsigned int
+infoBackupDataTotal(const InfoBackup *const this)
+{
+    return lstSize(THIS_PUB(InfoBackup)->backup);
+}
 
 /***********************************************************************************************************************************
 Functions
@@ -71,35 +129,27 @@ void infoBackupDataAdd(const InfoBackup *this, const Manifest *manifest);
 // Delete backup from the current backup list
 void infoBackupDataDelete(const InfoBackup *this, const String *backupDeleteLabel);
 
-/***********************************************************************************************************************************
-Getters/Setters
-***********************************************************************************************************************************/
-// Return a list of current backup labels, applying a regex expression if provided
-StringList *infoBackupDataLabelList(const InfoBackup *this, const String *expression);
-
-// PostgreSQL info
-InfoPg *infoBackupPg(const InfoBackup *this);
-InfoBackup *infoBackupPgSet(InfoBackup *this, unsigned int pgVersion, uint64_t pgSystemId);
-
-// Return a structure of the backup data from a specific index
-InfoBackupData infoBackupData(const InfoBackup *this, unsigned int backupDataIdx);
-
-// Return a pointer to a structure from the current backup data given a label, else NULL
-InfoBackupData *infoBackupDataByLabel(const InfoBackup *this, const String *backupLabel);
-
 // Given a backup label, get the dependency list
 StringList *infoBackupDataDependentList(const InfoBackup *this, const String *backupLabel);
 
-// Get total current backups
-unsigned int infoBackupDataTotal(const InfoBackup *this);
+// Return a list of current backup labels, applying a regex expression if provided
+StringList *infoBackupDataLabelList(const InfoBackup *this, const String *expression);
 
-// Cipher passphrase
-const String *infoBackupCipherPass(const InfoBackup *this);
+// Move to a new parent mem context
+__attribute__((always_inline)) static inline InfoBackup *
+infoBackupMove(InfoBackup *const this, MemContext *const parentNew)
+{
+    return objMove(this, parentNew);
+}
 
 /***********************************************************************************************************************************
 Destructor
 ***********************************************************************************************************************************/
-void infoBackupFree(InfoBackup *this);
+__attribute__((always_inline)) static inline void
+infoBackupFree(InfoBackup *const this)
+{
+    objFree(this);
+}
 
 /***********************************************************************************************************************************
 Helper functions
