@@ -13,7 +13,6 @@ PostgreSQL Info Handler
 #include "common/debug.h"
 #include "common/ini.h"
 #include "common/log.h"
-#include "common/memContext.h"
 #include "common/type/json.h"
 #include "common/type/list.h"
 #include "common/type/object.h"
@@ -41,10 +40,9 @@ Object type
 ***********************************************************************************************************************************/
 struct InfoPg
 {
+    InfoPgPub pub;                                                  // Publicly accessible variables
     MemContext *memContext;                                         // Mem context
-    Info *info;                                                     // Info contents
     InfoPgType type;                                                // Type of info file being loaded
-    List *history;                                                  // A list of InfoPgData
     unsigned int historyCurrent;                                    // Index of the current history item
 };
 
@@ -55,16 +53,19 @@ static InfoPg *
 infoPgNewInternal(InfoPgType type)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(ENUM, type);
+        FUNCTION_TEST_PARAM(STRING_ID, type);
     FUNCTION_TEST_END();
 
-    InfoPg *this = memNew(sizeof(InfoPg));
+    InfoPg *this = OBJ_NEW_ALLOC();
 
     *this = (InfoPg)
     {
+        .pub =
+        {
+            .history = lstNewP(sizeof(InfoPgData)),
+        },
         .memContext = memContextCurrent(),
         .type = type,
-        .history = lstNewP(sizeof(InfoPgData)),
     };
 
     FUNCTION_TEST_RETURN(this);
@@ -75,18 +76,18 @@ InfoPg *
 infoPgNew(InfoPgType type, const String *cipherPassSub)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(ENUM, type);
+        FUNCTION_LOG_PARAM(STRING_ID, type);
         FUNCTION_TEST_PARAM(STRING, cipherPassSub);
     FUNCTION_LOG_END();
 
     InfoPg *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("InfoPg")
+    OBJ_NEW_BEGIN(InfoPg)
     {
         this = infoPgNewInternal(type);
-        this->info = infoNew(cipherPassSub);
+        this->pub.info = infoNew(cipherPassSub);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(INFO_PG, this);
 }
@@ -142,7 +143,7 @@ infoPgLoadCallback(void *data, const String *section, const String *key, const V
         };
 
         // Insert at beginning of list so the history is reverse ordered
-        lstInsert(loadData->infoPg->history, 0, &infoPgData);
+        lstInsert(loadData->infoPg->pub.history, 0, &infoPgData);
     }
     // Callback if set
     else if (loadData->callbackFunction != NULL)
@@ -156,7 +157,7 @@ infoPgNewLoad(IoRead *read, InfoPgType type, InfoLoadNewCallback *callbackFuncti
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(IO_READ, read);
-        FUNCTION_LOG_PARAM(ENUM, type);
+        FUNCTION_LOG_PARAM(STRING_ID, type);
         FUNCTION_LOG_PARAM(FUNCTIONP, callbackFunction);
         FUNCTION_LOG_PARAM_P(VOID, callbackData);
     FUNCTION_LOG_END();
@@ -167,7 +168,7 @@ infoPgNewLoad(IoRead *read, InfoPgType type, InfoLoadNewCallback *callbackFuncti
 
     InfoPg *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("InfoPg")
+    OBJ_NEW_BEGIN(InfoPg)
     {
         this = infoPgNewInternal(type);
 
@@ -182,25 +183,25 @@ infoPgNewLoad(IoRead *read, InfoPgType type, InfoLoadNewCallback *callbackFuncti
             .infoPg = this,
         };
 
-        this->info = infoNewLoad(read, infoPgLoadCallback, &loadData);
+        this->pub.info = infoNewLoad(read, infoPgLoadCallback, &loadData);
 
         // History must include at least one item or the file is corrupt
-        CHECK(lstSize(this->history) > 0);
+        CHECK(!lstEmpty(this->pub.history));
 
         // If the current id was not found then the file is corrupt
         CHECK(loadData.currentId > 0);
 
         // Find the current history item
-        for (unsigned int historyIdx = 0; historyIdx < lstSize(this->history); historyIdx++)
+        for (unsigned int historyIdx = 0; historyIdx < lstSize(this->pub.history); historyIdx++)
         {
-            if (((InfoPgData *)lstGet(this->history, historyIdx))->id == loadData.currentId)
+            if (((InfoPgData *)lstGet(this->pub.history, historyIdx))->id == loadData.currentId)
                 this->historyCurrent = historyIdx;
         }
 
         // If the current id did not match the history list then the file is corrupt
         CHECK(this->historyCurrent != UINT_MAX);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(INFO_PG, this);
 }
@@ -217,7 +218,7 @@ infoPgAdd(InfoPg *this, const InfoPgData *infoPgData)
     ASSERT(this != NULL);
     ASSERT(infoPgData != NULL);
 
-    lstInsert(this->history, 0, infoPgData);
+    lstInsert(this->pub.history, 0, infoPgData);
     this->historyCurrent = 0;
 
     FUNCTION_LOG_RETURN_VOID();
@@ -230,7 +231,7 @@ infoPgSet(
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(INFO_PG, this);
-        FUNCTION_LOG_PARAM(ENUM, type);
+        FUNCTION_LOG_PARAM(STRING_ID, type);
         FUNCTION_LOG_PARAM(UINT, pgVersion);
         FUNCTION_LOG_PARAM(UINT64, pgSystemId);
         FUNCTION_LOG_PARAM(UINT, pgCatalogVersion);
@@ -345,7 +346,7 @@ infoPgSaveCallback(void *data, const String *sectionNext, InfoSave *infoSaveData
     if (saveData->callbackFunction != NULL)
         saveData->callbackFunction(saveData->callbackData, sectionNext, infoSaveData);
 
-    FUNCTION_TEST_RETURN_VOID()
+    FUNCTION_TEST_RETURN_VOID();
 }
 
 void
@@ -394,19 +395,6 @@ infoPgArchiveId(const InfoPg *this, unsigned int pgDataIdx)
     FUNCTION_LOG_RETURN(STRING, strNewFmt("%s-%u", strZ(pgVersionToStr(pgData.version)), pgData.id));
 }
 
-/**********************************************************************************************************************************/
-const String *
-infoPgCipherPass(const InfoPg *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INFO_PG, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(infoCipherPass(this->info));
-}
-
 /**********************************************************************************************************************************/
 InfoPgData
 infoPgData(const InfoPg *this, unsigned int pgDataIdx)
@@ -418,7 +406,7 @@ infoPgData(const InfoPg *this, unsigned int pgDataIdx)
 
     ASSERT(this != NULL);
 
-    FUNCTION_LOG_RETURN(INFO_PG_DATA, *(InfoPgData *)lstGet(this->history, pgDataIdx));
+    FUNCTION_LOG_RETURN(INFO_PG_DATA, *(InfoPgData *)lstGet(this->pub.history, pgDataIdx));
 }
 
 /**********************************************************************************************************************************/
@@ -447,32 +435,6 @@ infoPgDataCurrentId(const InfoPg *this)
     FUNCTION_LOG_RETURN(UINT, this->historyCurrent);
 }
 
-/**********************************************************************************************************************************/
-Info *
-infoPgInfo(const InfoPg *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INFO_PG, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->info);
-}
-
-/**********************************************************************************************************************************/
-unsigned int
-infoPgDataTotal(const InfoPg *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(INFO_PG, this);
-    FUNCTION_LOG_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_LOG_RETURN(UINT, lstSize(this->history));
-}
-
 /**********************************************************************************************************************************/
 unsigned int
 infoPgCurrentDataId(const InfoPg *this)
