@@ -7,9 +7,7 @@ Postgres Client
 
 #include "common/debug.h"
 #include "common/log.h"
-#include "common/memContext.h"
 #include "common/type/list.h"
-#include "common/type/object.h"
 #include "common/wait.h"
 #include "postgres/client.h"
 
@@ -18,7 +16,6 @@ Object type
 ***********************************************************************************************************************************/
 struct PgClient
 {
-    MemContext *memContext;
     const String *host;
     unsigned int port;
     const String *database;
@@ -28,17 +25,24 @@ struct PgClient
     PGconn *connection;
 };
 
-OBJECT_DEFINE_MOVE(PG_CLIENT);
-OBJECT_DEFINE_FREE(PG_CLIENT);
-
 /***********************************************************************************************************************************
 Close protocol connection
 ***********************************************************************************************************************************/
-OBJECT_DEFINE_FREE_RESOURCE_BEGIN(PG_CLIENT, LOG, logLevelTrace)
+static void
+pgClientFreeResource(THIS_VOID)
 {
+    THIS(PgClient);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(PG_CLIENT, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
     PQfinish(this->connection);
+
+    FUNCTION_LOG_RETURN_VOID();
 }
-OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
 
 /**********************************************************************************************************************************/
 PgClient *
@@ -57,13 +61,12 @@ pgClientNew(const String *host, const unsigned int port, const String *database,
 
     PgClient *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("PgClient")
+    OBJ_NEW_BEGIN(PgClient)
     {
-        this = memNew(sizeof(PgClient));
+        this = OBJ_NEW_ALLOC();
 
         *this = (PgClient)
         {
-            .memContext = MEM_CONTEXT_NEW(),
             .host = strDup(host),
             .port = port,
             .database = strDup(database),
@@ -71,7 +74,7 @@ pgClientNew(const String *host, const unsigned int port, const String *database,
             .queryTimeout = queryTimeout,
         };
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(PG_CLIENT, this);
 }
@@ -98,7 +101,7 @@ pgClientEscape(const String *string)
 
     ASSERT(string != NULL);
 
-    String *result = strNew("'");
+    String *result = strCatZ(strNew(), "'");
 
     // Iterate all characters in the string
     for (unsigned stringIdx = 0; stringIdx < strSize(string); stringIdx++)
@@ -131,7 +134,7 @@ pgClientOpen(PgClient *this)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Base connection string
-        String *connInfo = strNewFmt("dbname=%s port=%u", strZ(pgClientEscape(this->database)), this->port);
+        String *connInfo = strCatFmt(strNew(), "dbname=%s port=%u", strZ(pgClientEscape(this->database)), this->port);
 
         // Add user if specified
         if (this->user != NULL)
@@ -145,14 +148,14 @@ pgClientOpen(PgClient *this)
         this->connection = PQconnectdb(strZ(connInfo));
 
         // Set a callback to shutdown the connection
-        memContextCallbackSet(this->memContext, pgClientFreeResource, this);
+        memContextCallbackSet(objMemContext(this), pgClientFreeResource, this);
 
         // Handle errors
         if (PQstatus(this->connection) != CONNECTION_OK)
         {
             THROW_FMT(
                 DbConnectError, "unable to connect to '%s': %s", strZ(connInfo),
-                strZ(strTrim(strNew(PQerrorMessage(this->connection)))));
+                strZ(strTrim(strNewZ(PQerrorMessage(this->connection)))));
         }
 
         // Set notice and warning processor
@@ -185,7 +188,7 @@ pgClientQuery(PgClient *this, const String *query)
         {
             THROW_FMT(
                 DbQueryError, "unable to send query '%s': %s", strZ(query),
-                strZ(strTrim(strNew(PQerrorMessage(this->connection)))));
+                strZ(strTrim(strNewZ(PQerrorMessage(this->connection)))));
         }
 
         // Wait for a result
@@ -203,14 +206,17 @@ pgClientQuery(PgClient *this, const String *query)
         if (busy)
         {
             PGcancel *cancel = PQgetCancel(this->connection);
-            CHECK(cancel != NULL);
+
+            // If cancel is NULL then more than likely the server process crashed or disconnected
+            if (cancel == NULL)
+                THROW_FMT(DbQueryError, "unable to cancel query '%s': connection was lost", strZ(query));
 
             TRY_BEGIN()
             {
                 char error[256];
 
                 if (!PQcancel(cancel, error, sizeof(error)))
-                    THROW_FMT(DbQueryError, "unable to cancel query '%s': %s", strZ(query), strZ(strTrim(strNew(error))));
+                    THROW_FMT(DbQueryError, "unable to cancel query '%s': %s", strZ(query), strZ(strTrim(strNewZ(error))));
             }
             FINALLY()
             {
@@ -229,7 +235,7 @@ pgClientQuery(PgClient *this, const String *query)
                 THROW_FMT(DbQueryError, "query '%s' timed out after %" PRIu64 "ms", strZ(query), this->queryTimeout);
 
             // If this was a command that returned no results then we are done
-            int resultStatus = PQresultStatus(pgResult);
+            ExecStatusType resultStatus = PQresultStatus(pgResult);
 
             if (resultStatus != PGRES_COMMAND_OK)
             {
@@ -238,7 +244,7 @@ pgClientQuery(PgClient *this, const String *query)
                 {
                     THROW_FMT(
                         DbQueryError, "unable to execute query '%s': %s", strZ(query),
-                        strZ(strTrim(strNew(PQresultErrorMessage(pgResult)))));
+                        strZ(strTrim(strNewZ(PQresultErrorMessage(pgResult)))));
                 }
 
                 // Fetch row and column values
@@ -277,36 +283,28 @@ pgClientQuery(PgClient *this, const String *query)
                                 {
                                     // Boolean type
                                     case 16:                            // bool
-                                    {
                                         varLstAdd(resultRow, varNewBool(varBoolForce(varNewStrZ(value))));
                                         break;
-                                    }
 
                                     // Text/char types
                                     case 18:                            // char
                                     case 19:                            // name
                                     case 25:                            // text
-                                    {
                                         varLstAdd(resultRow, varNewStrZ(value));
                                         break;
-                                    }
 
                                     // Integer types
                                     case 20:                            // int8
                                     case 21:                            // int2
                                     case 23:                            // int4
                                     case 26:                            // oid
-                                    {
                                         varLstAdd(resultRow, varNewInt64(cvtZToInt64(value)));
                                         break;
-                                    }
 
                                     default:
-                                    {
                                         THROW_FMT(
                                             FormatError, "unable to parse type %u in column %d for query '%s'",
                                             columnType[columnIdx], columnIdx, strZ(query));
-                                    }
                                 }
                             }
                         }
@@ -346,7 +344,7 @@ pgClientClose(PgClient *this)
 
     if (this->connection != NULL)
     {
-        memContextCallbackClear(this->memContext);
+        memContextCallbackClear(objMemContext(this));
         PQfinish(this->connection);
         this->connection = NULL;
     }
