@@ -9,19 +9,16 @@ Storage Helper
 #include "common/io/io.h"
 #include "common/memContext.h"
 #include "common/regExp.h"
-#include "config/define.h"
 #include "config/config.h"
 #include "protocol/helper.h"
-#include "storage/azure/storage.h"
-#include "storage/cifs/storage.h"
 #include "storage/posix/storage.h"
 #include "storage/remote/storage.h"
-#include "storage/s3/storage.h"
 #include "storage/helper.h"
 
 /***********************************************************************************************************************************
 Storage path constants
 ***********************************************************************************************************************************/
+STRING_EXTERN(STORAGE_SPOOL_ARCHIVE_STR,                            STORAGE_SPOOL_ARCHIVE);
 STRING_EXTERN(STORAGE_SPOOL_ARCHIVE_IN_STR,                         STORAGE_SPOOL_ARCHIVE_IN);
 STRING_EXTERN(STORAGE_SPOOL_ARCHIVE_OUT_STR,                        STORAGE_SPOOL_ARCHIVE_OUT);
 
@@ -40,16 +37,18 @@ Error message when writable storage is requested in dry-run mode
 /***********************************************************************************************************************************
 Local variables
 ***********************************************************************************************************************************/
-static struct StorageHelper
+static struct StorageHelperLocal
 {
     MemContext *memContext;                                         // Mem context for storage helper
 
+    const StorageHelper *helperList;                                // List of helpers to create storage
+
     Storage *storageLocal;                                          // Local read-only storage
     Storage *storageLocalWrite;                                     // Local write storage
     Storage **storagePg;                                            // PostgreSQL read-only storage
     Storage **storagePgWrite;                                       // PostgreSQL write storage
-    Storage *storageRepo;                                           // Repository read-only storage
-    Storage *storageRepoWrite;                                      // Repository write storage
+    Storage **storageRepo;                                          // Repository read-only storage
+    Storage **storageRepoWrite;                                     // Repository write storage
     Storage *storageSpool;                                          // Spool read-only storage
     Storage *storageSpoolWrite;                                     // Spool write storage
 
@@ -64,7 +63,7 @@ static struct StorageHelper
 Create the storage helper memory context
 ***********************************************************************************************************************************/
 static void
-storageHelperInit(void)
+storageHelperContextInit(void)
 {
     FUNCTION_TEST_VOID();
 
@@ -84,6 +83,18 @@ storageHelperInit(void)
     FUNCTION_TEST_RETURN_VOID();
 }
 
+/**********************************************************************************************************************************/
+void storageHelperInit(const StorageHelper *const helperList)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM_P(VOID, helperList);
+    FUNCTION_TEST_END();
+
+    storageHelper.helperList = helperList;
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
 /**********************************************************************************************************************************/
 void
 storageHelperDryRunInit(bool dryRun)
@@ -129,7 +140,7 @@ storageLocal(void)
 
     if (storageHelper.storageLocal == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
@@ -148,7 +159,7 @@ storageLocalWrite(void)
 
     if (storageHelper.storageLocalWrite == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
@@ -164,26 +175,26 @@ storageLocalWrite(void)
 Get pg storage for the specified host id
 ***********************************************************************************************************************************/
 static Storage *
-storagePgGet(unsigned int hostId, bool write)
+storagePgGet(unsigned int pgIdx, bool write)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, hostId);
+        FUNCTION_TEST_PARAM(UINT, pgIdx);
         FUNCTION_TEST_PARAM(BOOL, write);
     FUNCTION_TEST_END();
 
     Storage *result = NULL;
 
     // Use remote storage
-    if (!pgIsLocal(hostId))
+    if (!pgIsLocal(pgIdx))
     {
         result = storageRemoteNew(
             STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, NULL,
-            protocolRemoteGet(protocolStorageTypePg, hostId), cfgOptionUInt(cfgOptCompressLevelNetwork));
+            protocolRemoteGet(protocolStorageTypePg, pgIdx), cfgOptionUInt(cfgOptCompressLevelNetwork));
     }
     // Use Posix storage
     else
     {
-        result = storagePosixNewP(cfgOptionStr(cfgOptPgPath + hostId - 1), .write = write);
+        result = storagePosixNewP(cfgOptionIdxStr(cfgOptPgPath, pgIdx), .write = write);
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -191,69 +202,69 @@ storagePgGet(unsigned int hostId, bool write)
 
 /**********************************************************************************************************************************/
 const Storage *
-storagePgId(unsigned int hostId)
+storagePgIdx(unsigned int pgIdx)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, hostId);
+        FUNCTION_TEST_PARAM(UINT, pgIdx);
     FUNCTION_TEST_END();
 
-    if (storageHelper.storagePg == NULL || storageHelper.storagePg[hostId - 1] == NULL)
+    if (storageHelper.storagePg == NULL || storageHelper.storagePg[pgIdx] == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
             if (storageHelper.storagePg == NULL)
-                storageHelper.storagePg = memNewPtrArray(cfgDefOptionIndexTotal(cfgDefOptPgPath));
+                storageHelper.storagePg = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpPg));
 
-            storageHelper.storagePg[hostId - 1] = storagePgGet(hostId, false);
+            storageHelper.storagePg[pgIdx] = storagePgGet(pgIdx, false);
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storagePg[hostId - 1]);
+    FUNCTION_TEST_RETURN(storageHelper.storagePg[pgIdx]);
 }
 
 const Storage *
 storagePg(void)
 {
     FUNCTION_TEST_VOID();
-    FUNCTION_TEST_RETURN(storagePgId(cfgOptionTest(cfgOptHostId) ? cfgOptionUInt(cfgOptHostId) : 1));
+    FUNCTION_TEST_RETURN(storagePgIdx(cfgOptionGroupIdxDefault(cfgOptGrpPg)));
 }
 
 const Storage *
-storagePgIdWrite(unsigned int hostId)
+storagePgIdxWrite(unsigned int pgIdx)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, hostId);
+        FUNCTION_TEST_PARAM(UINT, pgIdx);
     FUNCTION_TEST_END();
 
     // Writes not allowed in dry-run mode
     if (!storageHelper.dryRunInit || storageHelper.dryRun)
         THROW(AssertError, WRITABLE_WHILE_DRYRUN);
 
-    if (storageHelper.storagePgWrite == NULL || storageHelper.storagePgWrite[hostId - 1] == NULL)
+    if (storageHelper.storagePgWrite == NULL || storageHelper.storagePgWrite[pgIdx] == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
             if (storageHelper.storagePgWrite == NULL)
-                storageHelper.storagePgWrite = memNewPtrArray(cfgDefOptionIndexTotal(cfgDefOptPgPath));
+                storageHelper.storagePgWrite = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpPg));
 
-            storageHelper.storagePgWrite[hostId - 1] = storagePgGet(hostId, true);
+            storageHelper.storagePgWrite[pgIdx] = storagePgGet(pgIdx, true);
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storagePgWrite[hostId - 1]);
+    FUNCTION_TEST_RETURN(storageHelper.storagePgWrite[pgIdx]);
 }
 
 const Storage *
 storagePgWrite(void)
 {
     FUNCTION_TEST_VOID();
-    FUNCTION_TEST_RETURN(storagePgIdWrite(cfgOptionTest(cfgOptHostId) ? cfgOptionUInt(cfgOptHostId) : 1));
+    FUNCTION_TEST_RETURN(storagePgIdxWrite(cfgOptionGroupIdxDefault(cfgOptGrpPg)));
 }
 
 /***********************************************************************************************************************************
@@ -295,9 +306,9 @@ storageRepoPathExpression(const String *expression, const String *path)
     {
         // Construct the base path
         if (storageHelper.stanza != NULL)
-            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s", strZ(storageHelper.stanza));
+            result = strCatFmt(strNew(), STORAGE_PATH_ARCHIVE "/%s", strZ(storageHelper.stanza));
         else
-            result = strNew(STORAGE_PATH_ARCHIVE);
+            result = strCatZ(strNew(), STORAGE_PATH_ARCHIVE);
 
         // If a subpath should be appended, determine if it is WAL path, else just append the subpath
         if (path != NULL)
@@ -315,9 +326,9 @@ storageRepoPathExpression(const String *expression, const String *path)
     {
         // Construct the base path
         if (storageHelper.stanza != NULL)
-            result = strNewFmt(STORAGE_PATH_BACKUP "/%s", strZ(storageHelper.stanza));
+            result = strCatFmt(strNew(), STORAGE_PATH_BACKUP "/%s", strZ(storageHelper.stanza));
         else
-            result = strNew(STORAGE_PATH_BACKUP);
+            result = strCatZ(strNew(), STORAGE_PATH_BACKUP);
 
         // Append subpath if provided
         if (path != NULL)
@@ -333,103 +344,99 @@ storageRepoPathExpression(const String *expression, const String *path)
 Get the repo storage
 ***********************************************************************************************************************************/
 static Storage *
-storageRepoGet(const String *type, bool write)
+storageRepoGet(unsigned int repoIdx, bool write)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STRING, type);
+        FUNCTION_TEST_PARAM(UINT, repoIdx);
         FUNCTION_TEST_PARAM(BOOL, write);
     FUNCTION_TEST_END();
 
-    ASSERT(type != NULL);
-
     Storage *result = NULL;
 
     // Use remote storage
-    if (!repoIsLocal())
+    if (!repoIsLocal(repoIdx))
     {
         result = storageRemoteNew(
             STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression,
-            protocolRemoteGet(protocolStorageTypeRepo, 1), cfgOptionUInt(cfgOptCompressLevelNetwork));
-    }
-    // Use Azure storage
-    else if (strEqZ(type, STORAGE_AZURE_TYPE))
-    {
-        result = storageAzureNew(
-            cfgOptionStr(cfgOptRepoPath), write, storageRepoPathExpression, cfgOptionStr(cfgOptRepoAzureContainer),
-            cfgOptionStr(cfgOptRepoAzureAccount),
-            strEqZ(cfgOptionStr(cfgOptRepoAzureKeyType), STORAGE_AZURE_KEY_TYPE_SHARED) ?
-                storageAzureKeyTypeShared : storageAzureKeyTypeSas,
-            cfgOptionStr(cfgOptRepoAzureKey), STORAGE_AZURE_BLOCKSIZE_MIN, cfgOptionStrNull(cfgOptRepoAzureHost),
-            cfgOptionUInt(cfgOptRepoAzurePort), ioTimeoutMs(), cfgOptionBool(cfgOptRepoAzureVerifyTls),
-            cfgOptionStrNull(cfgOptRepoAzureCaFile), cfgOptionStrNull(cfgOptRepoAzureCaPath));
-    }
-    // Use CIFS storage
-    else if (strEqZ(type, STORAGE_CIFS_TYPE))
-    {
-        result = storageCifsNew(
-            cfgOptionStr(cfgOptRepoPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression);
-    }
-    // Use Posix storage
-    else if (strEqZ(type, STORAGE_POSIX_TYPE))
-    {
-        result = storagePosixNewP(
-            cfgOptionStr(cfgOptRepoPath), .write = write, .pathExpressionFunction = storageRepoPathExpression);
-    }
-    // Use S3 storage
-    else if (strEqZ(type, STORAGE_S3_TYPE))
-    {
-        // Set the default port
-        unsigned int port = cfgOptionUInt(cfgOptRepoS3Port);
-
-        // Extract port from the endpoint and host if it is present
-        const String *endPoint = cfgOptionHostPort(cfgOptRepoS3Endpoint, &port);
-        const String *host = cfgOptionHostPort(cfgOptRepoS3Host, &port);
-
-        // If the port option was set explicitly then use it in preference to appended ports
-        if (cfgOptionSource(cfgOptRepoS3Port) != cfgSourceDefault)
-            port = cfgOptionUInt(cfgOptRepoS3Port);
-
-        result = storageS3New(
-            cfgOptionStr(cfgOptRepoPath), write, storageRepoPathExpression, cfgOptionStr(cfgOptRepoS3Bucket), endPoint,
-            strEqZ(cfgOptionStr(cfgOptRepoS3UriStyle), STORAGE_S3_URI_STYLE_HOST) ? storageS3UriStyleHost : storageS3UriStylePath,
-            cfgOptionStr(cfgOptRepoS3Region),
-            strEqZ(cfgOptionStr(cfgOptRepoS3KeyType), STORAGE_S3_KEY_TYPE_SHARED) ? storageS3KeyTypeShared : storageS3KeyTypeAuto,
-            cfgOptionStrNull(cfgOptRepoS3Key), cfgOptionStrNull(cfgOptRepoS3KeySecret), cfgOptionStrNull(cfgOptRepoS3Token),
-            cfgOptionStrNull(cfgOptRepoS3Role), STORAGE_S3_PARTSIZE_MIN, host, port, ioTimeoutMs(),
-            cfgOptionBool(cfgOptRepoS3VerifyTls), cfgOptionStrNull(cfgOptRepoS3CaFile), cfgOptionStrNull(cfgOptRepoS3CaPath));
+            protocolRemoteGet(protocolStorageTypeRepo, repoIdx), cfgOptionUInt(cfgOptCompressLevelNetwork));
     }
+    // Use local storage
     else
-        THROW_FMT(AssertError, "invalid storage type '%s'", strZ(type));
+    {
+        // Search for the helper
+        const StringId type = cfgOptionIdxStrId(cfgOptRepoType, repoIdx);
+
+        if (storageHelper.helperList != NULL)
+        {
+            for (const StorageHelper *helper = storageHelper.helperList; helper->type != 0; helper++)
+            {
+                if (helper->type == type)
+                {
+                    result = helper->helper(repoIdx, write, storageRepoPathExpression);
+                    break;
+                }
+            }
+        }
+
+        // If no helper was found it try Posix
+        if (result == NULL)
+        {
+            CHECK(type == STORAGE_POSIX_TYPE);
+
+            result = storagePosixNewP(
+                cfgOptionIdxStr(cfgOptRepoPath, repoIdx), .write = write, .pathExpressionFunction = storageRepoPathExpression);
+        }
+    }
 
     FUNCTION_TEST_RETURN(result);
 }
 
 /**********************************************************************************************************************************/
 const Storage *
-storageRepo(void)
+storageRepoIdx(unsigned int repoIdx)
 {
-    FUNCTION_TEST_VOID();
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, repoIdx);
+    FUNCTION_TEST_END();
 
     if (storageHelper.storageRepo == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
         storageHelperStanzaInit(false);
         storageHelperRepoInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageRepo = storageRepoGet(cfgOptionStr(cfgOptRepoType), false);
+            storageHelper.storageRepo = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpRepo));
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storageRepo);
+    if (storageHelper.storageRepo[repoIdx] == NULL)
+    {
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storageRepo[repoIdx] = storageRepoGet(repoIdx, false);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storageRepo[repoIdx]);
 }
 
 const Storage *
-storageRepoWrite(void)
+storageRepo(void)
 {
     FUNCTION_TEST_VOID();
+    FUNCTION_TEST_RETURN(storageRepoIdx(cfgOptionGroupIdxDefault(cfgOptGrpRepo)));
+}
+
+const Storage *
+storageRepoIdxWrite(unsigned int repoIdx)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, repoIdx);
+    FUNCTION_TEST_END();
 
     // Writes not allowed in dry-run mode
     if (!storageHelper.dryRunInit || storageHelper.dryRun)
@@ -437,18 +444,34 @@ storageRepoWrite(void)
 
     if (storageHelper.storageRepoWrite == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
         storageHelperStanzaInit(false);
         storageHelperRepoInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageRepoWrite = storageRepoGet(cfgOptionStr(cfgOptRepoType), true);
+            storageHelper.storageRepoWrite = memNewPtrArray(cfgOptionGroupIdxTotal(cfgOptGrpRepo));
         }
         MEM_CONTEXT_END();
     }
 
-    FUNCTION_TEST_RETURN(storageHelper.storageRepoWrite);
+    if (storageHelper.storageRepoWrite[repoIdx] == NULL)
+    {
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storageRepoWrite[repoIdx] = storageRepoGet(repoIdx, true);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storageRepoWrite[repoIdx]);
+}
+
+const Storage *
+storageRepoWrite(void)
+{
+    FUNCTION_TEST_VOID();
+    FUNCTION_TEST_RETURN(storageRepoIdxWrite(cfgOptionGroupIdxDefault(cfgOptGrpRepo)));
 }
 
 /***********************************************************************************************************************************
@@ -467,7 +490,14 @@ storageSpoolPathExpression(const String *expression, const String *path)
 
     String *result = NULL;
 
-    if (strEqZ(expression, STORAGE_SPOOL_ARCHIVE_IN))
+    if (strEqZ(expression, STORAGE_SPOOL_ARCHIVE))
+    {
+        if (path == NULL)
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s", strZ(storageHelper.stanza));
+        else
+            result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/%s", strZ(storageHelper.stanza), strZ(path));
+    }
+    else if (strEqZ(expression, STORAGE_SPOOL_ARCHIVE_IN))
     {
         if (path == NULL)
             result = strNewFmt(STORAGE_PATH_ARCHIVE "/%s/in", strZ(storageHelper.stanza));
@@ -495,7 +525,7 @@ storageSpool(void)
 
     if (storageHelper.storageSpool == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
         storageHelperStanzaInit(true);
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
@@ -520,7 +550,7 @@ storageSpoolWrite(void)
 
     if (storageHelper.storageSpoolWrite == NULL)
     {
-        storageHelperInit();
+        storageHelperContextInit();
         storageHelperStanzaInit(true);
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
@@ -543,7 +573,7 @@ storageHelperFree(void)
     if (storageHelper.memContext != NULL)
         memContextFree(storageHelper.memContext);
 
-    storageHelper = (struct StorageHelper){.memContext = NULL};
+    storageHelper = (struct StorageHelperLocal){.memContext = NULL, .helperList = storageHelper.helperList};
 
     FUNCTION_TEST_RETURN_VOID();
 }
