@@ -11,21 +11,17 @@ Storage Interface
 #include "common/type/list.h"
 #include "common/log.h"
 #include "common/memContext.h"
-#include "common/type/object.h"
 #include "common/regExp.h"
 #include "common/wait.h"
-#include "storage/storage.intern.h"
+#include "storage/storage.h"
 
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
 struct Storage
 {
+    StoragePub pub;                                                 // Publicly accessible variables
     MemContext *memContext;
-    void *driver;
-    StorageInterface interface;
-    const String *type;
-
     const String *path;
     mode_t modeFile;
     mode_t modePath;
@@ -33,16 +29,14 @@ struct Storage
     StoragePathExpressionCallback *pathExpressionFunction;
 };
 
-OBJECT_DEFINE_FREE(STORAGE);
-
 /**********************************************************************************************************************************/
 Storage *
 storageNew(
-    const String *type, const String *path, mode_t modeFile, mode_t modePath, bool write,
+    StringId type, const String *path, mode_t modeFile, mode_t modePath, bool write,
     StoragePathExpressionCallback pathExpressionFunction, void *driver, StorageInterface interface)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(STRING, type);
+        FUNCTION_LOG_PARAM(STRING_ID, type);
         FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(MODE, modeFile);
         FUNCTION_LOG_PARAM(MODE, modePath);
@@ -52,8 +46,8 @@ storageNew(
         FUNCTION_LOG_PARAM(STORAGE_INTERFACE, interface);
     FUNCTION_LOG_END();
 
-    ASSERT(type != NULL);
-    ASSERT(strSize(path) >= 1 && strPtr(path)[0] == '/');
+    ASSERT(type != 0);
+    ASSERT(strSize(path) >= 1 && strZ(path)[0] == '/');
     ASSERT(driver != NULL);
     ASSERT(interface.info != NULL);
     ASSERT(interface.infoList != NULL);
@@ -66,10 +60,13 @@ storageNew(
 
     *this = (Storage)
     {
+        .pub =
+        {
+            .type = type,
+            .driver = driver,
+            .interface = interface,
+        },
         .memContext = memContextCurrent(),
-        .driver = driver,
-        .interface = interface,
-        .type = type,
         .path = strDup(path),
         .modeFile = modeFile,
         .modePath = modePath,
@@ -197,10 +194,7 @@ storageGet(StorageRead *file, StorageGetParam param)
 
                 // If an exact read make sure the size is as expected
                 if (bufUsed(result) != param.exactSize)
-                {
-                    THROW_FMT(
-                        FileReadError, "unable to read %zu byte(s) from '%s'", param.exactSize, strPtr(storageReadName(file)));
-                }
+                    THROW_FMT(FileReadError, "unable to read %zu byte(s) from '%s'", param.exactSize, strZ(storageReadName(file)));
             }
             // Else read entire file
             else
@@ -245,7 +239,7 @@ storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
-    ASSERT(this->interface.info != NULL);
+    ASSERT(this->pub.interface.info != NULL);
 
     StorageInfo result = {0};
 
@@ -258,12 +252,19 @@ storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
         if (param.level == storageInfoLevelDefault)
             param.level = storageFeature(this, storageFeatureInfoDetail) ? storageInfoLevelDetail : storageInfoLevelBasic;
 
-        result = storageInterfaceInfoP(
-            this->driver, file, param.level, .followLink = param.followLink);
+        // If file is / then this is definitely a path so skip the call for drivers that do not support paths and do not provide
+        // additional info to return. Also, some object stores (e.g. S3) behave strangely when getting info for /.
+        if (strEq(file, FSLASH_STR) && !storageFeature(this, storageFeaturePath))
+        {
+            result = (StorageInfo){.level = param.level};
+        }
+        // Else call the driver
+        else
+            result = storageInterfaceInfoP(storageDriver(this), file, param.level, .followLink = param.followLink);
 
         // Error if the file missing and not ignoring
         if (!result.exists && !param.ignoreMissing)
-            THROW_SYS_ERROR_FMT(FileOpenError, STORAGE_ERROR_INFO_MISSING, strPtr(file));
+            THROW_SYS_ERROR_FMT(FileOpenError, STORAGE_ERROR_INFO_MISSING, strZ(file));
 
         // Dup the strings into the prior context
         MEM_CONTEXT_PRIOR_BEGIN()
@@ -338,7 +339,7 @@ storageInfoListSort(
         // If no sorting then use the callback directly
         if (sortOrder == sortOrderNone)
         {
-            result = storageInterfaceInfoListP(this->driver, path, level, callback, callbackData, .expression = expression);
+            result = storageInterfaceInfoListP(storageDriver(this), path, level, callback, callbackData, .expression = expression);
         }
         // Else sort the info before sending it to the callback
         else
@@ -351,7 +352,7 @@ storageInfoListSort(
             };
 
             result = storageInterfaceInfoListP(
-                this->driver, path, level, storageInfoListSortCallback, &data, .expression = expression);
+                storageDriver(this), path, level, storageInfoListSortCallback, &data, .expression = expression);
             lstSort(data.infoList, sortOrder);
 
             MEM_CONTEXT_TEMP_RESET_BEGIN()
@@ -410,29 +411,30 @@ storageInfoListCallback(void *data, const StorageInfo *info)
     StorageInfo infoUpdate = *info;
 
     if (listData->subPath != NULL)
-        infoUpdate.name = strNewFmt("%s/%s", strPtr(listData->subPath), strPtr(infoUpdate.name));
+        infoUpdate.name = strNewFmt("%s/%s", strZ(listData->subPath), strZ(infoUpdate.name));
 
-    // Only continue if there is no expression or the expression matches
-    if (listData->expression == NULL || regExpMatch(listData->regExp, infoUpdate.name))
+    // Is this file a match?
+    bool match = listData->expression == NULL || regExpMatch(listData->regExp, infoUpdate.name);
+
+    // Callback before checking path contents when not descending
+    if (match && listData->sortOrder != sortOrderDesc)
+        listData->callbackFunction(listData->callbackData, &infoUpdate);
+
+    // Recurse into paths
+    if (infoUpdate.type == storageTypePath && listData->recurse && !dotPath)
     {
-        if (listData->sortOrder != sortOrderDesc)
-            listData->callbackFunction(listData->callbackData, &infoUpdate);
+        StorageInfoListData data = *listData;
+        data.subPath = infoUpdate.name;
 
-        // Recurse into paths
-        if (infoUpdate.type == storageTypePath && listData->recurse && !dotPath)
-        {
-            StorageInfoListData data = *listData;
-            data.subPath = infoUpdate.name;
-
-            storageInfoListSort(
-                data.storage, strNewFmt("%s/%s", strPtr(data.path), strPtr(data.subPath)), infoUpdate.level, data.expression,
-                data.sortOrder, storageInfoListCallback, &data);
-        }
-
-        if (listData->sortOrder == sortOrderDesc)
-            listData->callbackFunction(listData->callbackData, &infoUpdate);
+        storageInfoListSort(
+            data.storage, strNewFmt("%s/%s", strZ(data.path), strZ(data.subPath)), infoUpdate.level, data.expression,
+            data.sortOrder, storageInfoListCallback, &data);
     }
 
+    // Callback after checking path contents when descending
+    if (match && listData->sortOrder == sortOrderDesc)
+        listData->callbackFunction(listData->callbackData, &infoUpdate);
+
     FUNCTION_TEST_RETURN_VOID();
 }
 
@@ -454,7 +456,7 @@ storageInfoList(
 
     ASSERT(this != NULL);
     ASSERT(callback != NULL);
-    ASSERT(this->interface.infoList != NULL);
+    ASSERT(this->pub.interface.infoList != NULL);
     ASSERT(!param.errorOnMissing || storageFeature(this, storageFeaturePath));
 
     bool result = false;
@@ -492,7 +494,7 @@ storageInfoList(
             result = storageInfoListSort(this, path, param.level, NULL, param.sortOrder, callback, callbackData);
 
         if (!result && param.errorOnMissing)
-            THROW_FMT(PathMissingError, STORAGE_ERROR_LIST_INFO_MISSING, strPtr(path));
+            THROW_FMT(PathMissingError, STORAGE_ERROR_LIST_INFO_MISSING, strZ(path));
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -569,29 +571,29 @@ storageMove(const Storage *this, StorageRead *source, StorageWrite *destination)
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
-    ASSERT(this->interface.move != NULL);
+    ASSERT(this->pub.interface.move != NULL);
     ASSERT(source != NULL);
     ASSERT(destination != NULL);
     ASSERT(!storageReadIgnoreMissing(source));
-    ASSERT(strEq(this->type, storageReadType(source)));
-    ASSERT(strEq(storageReadType(source), storageWriteType(destination)));
+    ASSERT(storageType(this) == storageReadType(source));
+    ASSERT(storageReadType(source) == storageWriteType(destination));
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // If the file can't be moved it will need to be copied
-        if (!storageInterfaceMoveP(this->driver, source, destination))
+        if (!storageInterfaceMoveP(storageDriver(this), source, destination))
         {
             // Perform the copy
             storageCopyP(source, destination);
 
             // Remove the source file
-            storageInterfaceRemoveP(this->driver, storageReadName(source));
+            storageInterfaceRemoveP(storageDriver(this), storageReadName(source));
 
             // Sync source path if the destination path was synced.  We know the source and destination paths are different because
             // the move did not succeed.  This will need updating when drivers other than Posix/CIFS are implemented because there's
             // no way to get coverage on it now.
             if (storageWriteSyncPath(destination))
-                storageInterfacePathSyncP(this->driver, strPath(storageReadName(source)));
+                storageInterfacePathSyncP(storageDriver(this), strPath(storageReadName(source)));
         }
     }
     MEM_CONTEXT_TEMP_END();
@@ -621,7 +623,7 @@ storageNewRead(const Storage *this, const String *fileExp, StorageNewReadParam p
     {
         result = storageReadMove(
             storageInterfaceNewReadP(
-                this->driver, storagePathP(this, fileExp), param.ignoreMissing, .compressible = param.compressible,
+                storageDriver(this), storagePathP(this, fileExp), param.ignoreMissing, .compressible = param.compressible,
                 .limit = param.limit),
             memContextPrior());
     }
@@ -658,7 +660,7 @@ storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam
     {
         result = storageWriteMove(
             storageInterfaceNewWriteP(
-                this->driver, storagePathP(this, fileExp), .modeFile = param.modeFile != 0 ? param.modeFile : this->modeFile,
+                storageDriver(this), storagePathP(this, fileExp), .modeFile = param.modeFile != 0 ? param.modeFile : this->modeFile,
                 .modePath = param.modePath != 0 ? param.modePath : this->modePath, .user = param.user, .group = param.group,
                 .timeModified = param.timeModified, .createPath = !param.noCreatePath, .syncFile = !param.noSyncFile,
                 .syncPath = !param.noSyncPath, .atomic = !param.noAtomic, .compressible = param.compressible),
@@ -691,15 +693,15 @@ storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
     else
     {
         // If the path expression is absolute then use it as is
-        if ((strPtr(pathExp))[0] == '/')
+        if ((strZ(pathExp))[0] == '/')
         {
             // Make sure the base storage path is contained within the path expression
             if (!strEqZ(this->path, "/"))
             {
                 if (!param.noEnforce && (!strBeginsWith(pathExp, this->path) ||
-                    !(strSize(pathExp) == strSize(this->path) || *(strPtr(pathExp) + strSize(this->path)) == '/')))
+                    !(strSize(pathExp) == strSize(this->path) || *(strZ(pathExp) + strSize(this->path)) == '/')))
                 {
-                    THROW_FMT(AssertError, "absolute path '%s' is not in base path '%s'", strPtr(pathExp), strPtr(this->path));
+                    THROW_FMT(AssertError, "absolute path '%s' is not in base path '%s'", strZ(pathExp), strZ(this->path));
                 }
             }
 
@@ -712,20 +714,20 @@ storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
             String *pathEvaluated = NULL;
 
             // Check if there is a path expression that needs to be evaluated
-            if ((strPtr(pathExp))[0] == '<')
+            if ((strZ(pathExp))[0] == '<')
             {
                 if (this->pathExpressionFunction == NULL)
-                    THROW_FMT(AssertError, "expression '%s' not valid without callback function", strPtr(pathExp));
+                    THROW_FMT(AssertError, "expression '%s' not valid without callback function", strZ(pathExp));
 
                 // Get position of the expression end
-                char *end = strchr(strPtr(pathExp), '>');
+                char *end = strchr(strZ(pathExp), '>');
 
                 // Error if end is not found
                 if (end == NULL)
-                    THROW_FMT(AssertError, "end > not found in path expression '%s'", strPtr(pathExp));
+                    THROW_FMT(AssertError, "end > not found in path expression '%s'", strZ(pathExp));
 
                 // Create a string from the expression
-                String *expression = strNewN(strPtr(pathExp), (size_t)(end - strPtr(pathExp) + 1));
+                String *expression = strNewZN(strZ(pathExp), (size_t)(end - strZ(pathExp) + 1));
 
                 // Create a string from the path if there is anything left after the expression
                 String *path = NULL;
@@ -734,13 +736,13 @@ storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
                 {
                     // Error if path separator is not found
                     if (end[1] != '/')
-                        THROW_FMT(AssertError, "'/' should separate expression and path '%s'", strPtr(pathExp));
+                        THROW_FMT(AssertError, "'/' should separate expression and path '%s'", strZ(pathExp));
 
                     // Only create path if there is something after the path separator
                     if (end[2] == 0)
-                        THROW_FMT(AssertError, "path '%s' should not end in '/'", strPtr(pathExp));
+                        THROW_FMT(AssertError, "path '%s' should not end in '/'", strZ(pathExp));
 
-                    path = strNew(end + 2);
+                    path = strNewZ(end + 2);
                 }
 
                 // Evaluate the path
@@ -748,7 +750,7 @@ storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
 
                 // Evaluated path cannot be NULL
                 if (pathEvaluated == NULL)
-                    THROW_FMT(AssertError, "evaluated path '%s' cannot be null", strPtr(pathExp));
+                    THROW_FMT(AssertError, "evaluated path '%s' cannot be null", strZ(pathExp));
 
                 // Assign evaluated path to path
                 pathExp = pathEvaluated;
@@ -759,9 +761,9 @@ storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
             }
 
             if (strEqZ(this->path, "/"))
-                result = strNewFmt("/%s", strPtr(pathExp));
+                result = strNewFmt("/%s", strZ(pathExp));
             else
-                result = strNewFmt("%s/%s", strPtr(this->path), strPtr(pathExp));
+                result = strNewFmt("%s/%s", strZ(this->path), strZ(pathExp));
 
             strFree(pathEvaluated);
         }
@@ -783,7 +785,7 @@ storagePathCreate(const Storage *this, const String *pathExp, StoragePathCreateP
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
-    ASSERT(this->interface.pathCreate != NULL && storageFeature(this, storageFeaturePath));
+    ASSERT(this->pub.interface.pathCreate != NULL && storageFeature(this, storageFeaturePath));
     ASSERT(this->write);
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -793,7 +795,7 @@ storagePathCreate(const Storage *this, const String *pathExp, StoragePathCreateP
 
         // Call driver function
         storageInterfacePathCreateP(
-            this->driver, path, param.errorOnExists, param.noParentCreate, param.mode != 0 ? param.mode : this->modePath);
+            storageDriver(this), path, param.errorOnExists, param.noParentCreate, param.mode != 0 ? param.mode : this->modePath);
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -841,9 +843,9 @@ storagePathRemove(const Storage *this, const String *pathExp, StoragePathRemoveP
         String *path = storagePathP(this, pathExp);
 
         // Call driver function
-        if (!storageInterfacePathRemoveP(this->driver, path, param.recurse) && param.errorOnMissing)
+        if (!storageInterfacePathRemoveP(storageDriver(this), path, param.recurse) && param.errorOnMissing)
         {
-            THROW_FMT(PathRemoveError, STORAGE_ERROR_PATH_REMOVE_MISSING, strPtr(path));
+            THROW_FMT(PathRemoveError, STORAGE_ERROR_PATH_REMOVE_MISSING, strZ(path));
         }
     }
     MEM_CONTEXT_TEMP_END();
@@ -863,11 +865,11 @@ void storagePathSync(const Storage *this, const String *pathExp)
     ASSERT(this->write);
 
     // Not all storage requires path sync so just do nothing if the function is not implemented
-    if (this->interface.pathSync != NULL)
+    if (this->pub.interface.pathSync != NULL)
     {
         MEM_CONTEXT_TEMP_BEGIN()
         {
-            storageInterfacePathSyncP(this->driver, storagePathP(this, pathExp));
+            storageInterfacePathSyncP(storageDriver(this), storagePathP(this, pathExp));
         }
         MEM_CONTEXT_TEMP_END();
     }
@@ -912,70 +914,18 @@ storageRemove(const Storage *this, const String *fileExp, StorageRemoveParam par
         String *file = storagePathP(this, fileExp);
 
         // Call driver function
-        storageInterfaceRemoveP(this->driver, file, .errorOnMissing = param.errorOnMissing);
+        storageInterfaceRemoveP(storageDriver(this), file, .errorOnMissing = param.errorOnMissing);
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN_VOID();
 }
 
-/**********************************************************************************************************************************/
-void *
-storageDriver(const Storage *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STORAGE, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->driver);
-}
-
-/**********************************************************************************************************************************/
-bool
-storageFeature(const Storage *this, StorageFeature feature)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STORAGE, this);
-        FUNCTION_TEST_PARAM(ENUM, feature);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->interface.feature >> feature & 1);
-}
-
-/**********************************************************************************************************************************/
-StorageInterface
-storageInterface(const Storage *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STORAGE, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->interface);
-}
-
-/**********************************************************************************************************************************/
-const String *
-storageType(const Storage *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STORAGE, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->type);
-}
-
 /**********************************************************************************************************************************/
 String *
 storageToLog(const Storage *this)
 {
     return strNewFmt(
-        "{type: %s, path: %s, write: %s}", strPtr(this->type), strPtr(strToLog(this->path)), cvtBoolToConstZ(this->write));
+        "{type: %s, path: %s, write: %s}", strZ(strIdToStr(storageType(this))), strZ(strToLog(this->path)),
+        cvtBoolToConstZ(this->write));
 }
