@@ -14,6 +14,16 @@ Standard binary name
 ***********************************************************************************************************************************/
 #define PROJECT_BIN                                                 "pgbackrest"
 
+/***********************************************************************************************************************************
+Config file name. The path will vary based on configuration.
+***********************************************************************************************************************************/
+#define PROJECT_CONFIG_FILE                                         PROJECT_BIN ".conf"
+
+/***********************************************************************************************************************************
+Config include path name. The parent path will vary based on configuration.
+***********************************************************************************************************************************/
+#define PROJECT_CONFIG_INCLUDE_PATH                                 "conf.d"
+
 /***********************************************************************************************************************************
 Format Number -- defines format for info and manifest files as well as on-disk structure.  If this number changes then the
 repository will be invalid unless migration functions are written.
@@ -21,8 +31,8 @@ repository will be invalid unless migration functions are written.
 #define REPOSITORY_FORMAT                                           5
 
 /***********************************************************************************************************************************
-Software version.  Currently this value is maintained in Version.pm and updated by test.pl.
+Software version
 ***********************************************************************************************************************************/
-#define PROJECT_VERSION                                             "2.30"
+#define PROJECT_VERSION                                             "2.36"
 
 #endif
