@@ -1,19 +1,46 @@
 /***********************************************************************************************************************************
 Test Help Command
 ***********************************************************************************************************************************/
+#include "config/load.h"
 #include "config/parse.h"
+#include "storage/helper.h"
 #include "storage/posix/storage.h"
-#include "storage/storage.h"
 #include "version.h"
 
+#include "common/harnessConfig.h"
+#include "common/harnessStorage.h"
+
+/***********************************************************************************************************************************
+Configuration load with just enough funcionality to test help
+***********************************************************************************************************************************/
+static void
+testCfgLoad(const StringList *const argList)
+{
+    FUNCTION_HARNESS_BEGIN();
+        FUNCTION_HARNESS_PARAM(STRING_LIST, argList);
+    FUNCTION_HARNESS_END();
+
+    // Parse config
+    configParse(storageLocal(), strLstSize(argList), strLstPtr(argList), false);
+
+    // Apply special option rules
+    cfgLoadUpdateOption();
+
+    FUNCTION_HARNESS_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
 
+    // Create help data
+    const BldCfg bldCfg = bldCfgParse(storagePosixNewP(HRN_PATH_REPO_STR));
+    const Buffer *const helpData = bldHlpRenderHelpAutoCCmp(bldCfg, bldHlpParse(storagePosixNewP(HRN_PATH_REPO_STR), bldCfg));
+
     // Program name a version are used multiple times
     const char *helpVersion = PROJECT_NAME " " PROJECT_VERSION;
 
@@ -32,6 +59,8 @@ testRun(void)
         "    expire          Expire backups that exceed retention.\n"
         "    help            Get help.\n"
         "    info            Retrieve information about backups.\n"
+        "    repo-get        Get a file from a repository.\n"
+        "    repo-ls         List files in a repository.\n"
         "    restore         Restore a database cluster.\n"
         "    stanza-create   Create the required stanza data.\n"
         "    stanza-delete   Delete a stanza.\n"
@@ -43,37 +72,62 @@ testRun(void)
         "Use 'pgbackrest help [command]' for more information.\n",
         helpVersion));
 
+    // *****************************************************************************************************************************
+    if (testBegin("helpRenderSplitSize()"))
+    {
+        TEST_RESULT_STRLST_Z(helpRenderSplitSize(STRDEF("abc def"), " ", 3), "abc\ndef\n", "two items");
+        TEST_RESULT_STRLST_Z(helpRenderSplitSize(STRDEF("abc def"), " ", 4), "abc\ndef\n", "two items, size > each item");
+        TEST_RESULT_STRLST_Z(helpRenderSplitSize(STRDEF("abc def ghi"), " ", 4), "abc\ndef\nghi\n", "three items");
+        TEST_RESULT_STRLST_Z(helpRenderSplitSize(STRDEF("abc def ghi"), " ", 8), "abc def\nghi\n", "three items, size > first two");
+        TEST_RESULT_STRLST_Z(helpRenderSplitSize(STRDEF("abc def "), " ", 4), "abc\ndef \n", "two items, size includes space");
+        TEST_RESULT_STRLST_Z(helpRenderSplitSize(STRDEF(""), " ", 1), "\n", "empty list");
+
+        TEST_RESULT_STRLST_Z(
+            helpRenderSplitSize(STRDEF("this is a short sentence"), " ", 10),
+            "this is a\n"
+            "short\n"
+            "sentence\n",
+            "variable word sizes");
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("helpRenderText()"))
     {
-        TEST_RESULT_STR_Z(helpRenderText(strNew("this is a short sentence"), 0, false, 80), "this is a short sentence", "one line");
+        TEST_RESULT_STR_Z(
+            helpRenderText(STRDEF("this is a short sentence"), false, 0, false, 80), "this is a short sentence", "one line");
 
         TEST_RESULT_STR_Z(
-            helpRenderText(strNew("this is a short sentence"), 4, false, 14),
+            helpRenderText(STRDEF("this is a short sentence"), false, 4, false, 14),
             "this is a\n"
             "    short\n"
             "    sentence",
             "three lines, no indent first");
 
         TEST_RESULT_STR_Z(
-            helpRenderText(strNew("This is a short paragraph.\n\nHere is another one."), 2, true, 16),
+            helpRenderText(STRDEF("This is a short paragraph.\n\nHere is another one."), true, 2, true, 16),
             "  This is a\n"
             "  short\n"
             "  paragraph.\n"
             "\n"
             "  Here is\n"
-            "  another one.",
-            "two paragraphs, indent first");
+            "  another one.\n"
+            "\n"
+            "  FOR INTERNAL\n"
+            "  USE ONLY. DO\n"
+            "  NOT USE IN\n"
+            "  PRODUCTION.",
+            "two paragraphs, indent first, internal");
     }
 
     // *****************************************************************************************************************************
     if (testBegin("helpRenderValue()"))
     {
-        TEST_RESULT_STR_Z(helpRenderValue(varNewBool(true)), "y", "boolean y");
-        TEST_RESULT_STR_Z(helpRenderValue(varNewBool(false)), "n", "boolean n");
-        TEST_RESULT_STR_Z(helpRenderValue(varNewStrZ("test-string")), "test-string", "string");
-        TEST_RESULT_STR_Z(helpRenderValue(varNewDbl(1.234)), "1.234", "double");
-        TEST_RESULT_STR_Z(helpRenderValue(varNewInt(1234)), "1234", "int");
+        TEST_RESULT_STR_Z(helpRenderValue(varNewBool(true), cfgOptTypeBoolean), "y", "boolean y");
+        TEST_RESULT_STR_Z(helpRenderValue(varNewBool(false), cfgOptTypeBoolean), "n", "boolean n");
+        TEST_RESULT_STR_Z(helpRenderValue(varNewStrZ("test-string"), cfgOptTypeString), "test-string", "string");
+        TEST_RESULT_STR_Z(helpRenderValue(varNewInt64(1234), cfgOptTypeInteger), "1234", "int");
+        TEST_RESULT_STR_Z(helpRenderValue(varNewInt64(1234000), cfgOptTypeTime), "1234", "time");
+        TEST_RESULT_STR_Z(helpRenderValue(NULL, cfgOptTypeString), NULL, "null");
     }
 
     // *****************************************************************************************************************************
@@ -82,18 +136,22 @@ testRun(void)
         StringList *argList = NULL;
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("general invocation");
+
         argList = strLstNew();
         strLstAddZ(argList, "/path/to/pgbackrest");
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "help from empty command line");
-        TEST_RESULT_STR_Z(helpRender(), generalHelp, "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help from empty command line");
+        TEST_RESULT_STR_Z(helpRender(helpData), generalHelp, "check text");
 
         argList = strLstNew();
         strLstAddZ(argList, "/path/to/pgbackrest");
         strLstAddZ(argList, "help");
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "help from help command");
-        TEST_RESULT_STR_Z(helpRender(), generalHelp, "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help from help command");
+        TEST_RESULT_STR_Z(helpRender(helpData), generalHelp, "check text");
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("version command");
+
         const char *commandHelp = strZ(strNewFmt(
             "%s%s",
             helpVersion,
@@ -107,11 +165,13 @@ testRun(void)
         strLstAddZ(argList, "/path/to/pgbackrest");
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "version");
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "help for version command");
-        TEST_RESULT_STR_Z(helpRender(), commandHelp, "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for version command");
+        TEST_RESULT_STR_Z(helpRender(helpData), commandHelp, "check text");
 
         // This test is broken up into multiple strings because C99 does not require compilers to support const strings > 4095 bytes
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("restore command");
+
         commandHelp = strZ(strNewFmt(
             "%s%s%s",
             helpVersion,
@@ -119,13 +179,26 @@ testRun(void)
             "\n"
             "Restore a database cluster.\n"
             "\n"
-            "This command is generally run manually, but there are instances where it might\n"
-            "be automated.\n"
+            "The restore command automatically defaults to selecting the latest backup from\n"
+            "the first repository where backups exist (see Quick Start - Restore a Backup).\n"
+            "The order in which the repositories are checked is dictated by the\n"
+            "pgbackrest.conf (e.g. repo1 will be checked before repo2). To select from a\n"
+            "specific repository, the --repo option can be passed (e.g. --repo=1). The --set\n"
+            "option can be passed if a backup other than the latest is desired.\n"
+            "\n"
+            "When PITR of --type=time is specified, then the target time must be specified\n"
+            "with the --target option. If a backup is not specified via the --set option,\n"
+            "then the configured repositories will be checked, in order, for a backup that\n"
+            "contains the requested time. If no backup can be found, the latest backup from\n"
+            "the first repository containing backups will be used. For other types of PITR,\n"
+            "e.g. xid, the --set option must be provided if the target is prior to the\n"
+            "latest backup. See Point-in-Time Recovery for more details and examples.\n"
             "\n"
             "Command Options:\n"
             "\n"
             "  --archive-mode                   preserve or disable archiving on restored\n"
             "                                   cluster [default=preserve]\n"
+            "  --db-exclude                     restore excluding the specified databases\n"
             "  --db-include                     restore only specified databases\n"
             "                                   [current=db1, db2]\n"
             "  --force                          force a restore [default=n]\n"
@@ -148,9 +221,9 @@ testRun(void)
             "\n"
             "General Options:\n"
             "\n"
-            "  --buffer-size                    buffer size for file operations\n"
+            "  --buffer-size                    buffer size for I/O operations\n"
             "                                   [current=32768, default=1048576]\n"
-            "  --cmd-ssh                        path to ssh client executable [default=ssh]\n"
+            "  --cmd-ssh                        SSH client command [default=ssh]\n"
             "  --compress-level-network         network compression level [default=3]\n"
             "  --config                         pgBackRest configuration file\n"
             "                                   [default=/etc/pgbackrest/pgbackrest.conf]\n"
@@ -159,7 +232,7 @@ testRun(void)
             "  --config-path                    base path of pgBackRest configuration files\n"
             "                                   [default=/etc/pgbackrest]\n"
             "  --delta                          restore or backup using checksums [default=n]\n"
-            "  --io-timeout                     i/O timeout [default=60]\n"
+            "  --io-timeout                     I/O timeout [default=60]\n"
             "  --lock-path                      path where lock files are stored\n"
             "                                   [default=/tmp/pgbackrest]\n"
             "  --neutral-umask                  use a neutral umask [default=y]\n"
@@ -184,23 +257,27 @@ testRun(void)
             "\n",
             "Repository Options:\n"
             "\n"
+            "  --repo                           set repository\n"
             "  --repo-azure-account             azure repository account\n"
-            "  --repo-azure-ca-file             azure repository TLS CA file\n"
-            "  --repo-azure-ca-path             azure repository TLS CA path\n"
             "  --repo-azure-container           azure repository container\n"
-            "  --repo-azure-host                azure repository host\n"
+            "  --repo-azure-endpoint            azure repository endpoint\n"
+            "                                   [default=blob.core.windows.net]\n"
             "  --repo-azure-key                 azure repository key\n"
             "  --repo-azure-key-type            azure repository key type [default=shared]\n"
-            "  --repo-azure-port                azure repository server port [default=443]\n"
-            "  --repo-azure-verify-tls          azure repository server certificate verify\n"
-            "                                   [default=y]\n"
+            "  --repo-azure-uri-style           azure URI Style [default=host]\n"
             "  --repo-cipher-pass               repository cipher passphrase\n"
             "                                   [current=<redacted>]\n"
             "  --repo-cipher-type               cipher used to encrypt the repository\n"
             "                                   [current=aes-256-cbc, default=none]\n"
+            "  --repo-gcs-bucket                GCS repository bucket\n"
+            "  --repo-gcs-endpoint              GCS repository endpoint\n"
+            "                                   [default=storage.googleapis.com]\n"
+            "  --repo-gcs-key                   GCS repository key\n"
+            "  --repo-gcs-key-type              GCS repository key type [default=service]\n"
             "  --repo-host                      repository host when operating remotely via\n"
             "                                   SSH [current=backup.example.net]\n"
-            "  --repo-host-cmd                  pgBackRest exe path on the repository host\n"
+            "  --repo-host-cmd                  repository host pgBackRest command\n"
+            "                                   [default=/path/to/pgbackrest]\n"
             "  --repo-host-config               pgBackRest repository host configuration\n"
             "                                   file\n"
             "                                   [default=/etc/pgbackrest/pgbackrest.conf]\n"
@@ -213,20 +290,21 @@ testRun(void)
             "                                   [default=pgbackrest]\n"
             "  --repo-path                      path where backups and archive are stored\n"
             "                                   [default=/var/lib/pgbackrest]\n"
-            "  --repo-s3-bucket                 s3 repository bucket\n"
-            "  --repo-s3-ca-file                s3 SSL CA File\n"
-            "  --repo-s3-ca-path                s3 SSL CA Path\n"
-            "  --repo-s3-endpoint               s3 repository endpoint\n"
-            "  --repo-s3-host                   s3 repository host\n"
-            "  --repo-s3-key                    s3 repository access key\n"
-            "  --repo-s3-key-secret             s3 repository secret access key\n"
-            "  --repo-s3-key-type               s3 repository key type [default=shared]\n"
-            "  --repo-s3-port                   s3 repository port [default=443]\n"
-            "  --repo-s3-region                 s3 repository region\n"
-            "  --repo-s3-role                   s3 repository role\n"
-            "  --repo-s3-token                  s3 repository security token\n"
-            "  --repo-s3-uri-style              s3 URI Style [default=host]\n"
-            "  --repo-s3-verify-tls             verify S3 server certificate [default=y]\n"
+            "  --repo-s3-bucket                 S3 repository bucket\n"
+            "  --repo-s3-endpoint               S3 repository endpoint\n"
+            "  --repo-s3-key                    S3 repository access key\n"
+            "  --repo-s3-key-secret             S3 repository secret access key\n"
+            "  --repo-s3-key-type               S3 repository key type [default=shared]\n"
+            "  --repo-s3-region                 S3 repository region\n"
+            "  --repo-s3-role                   S3 repository role\n"
+            "  --repo-s3-token                  S3 repository security token\n"
+            "  --repo-s3-uri-style              S3 URI Style [default=host]\n"
+            "  --repo-storage-ca-file           repository storage CA file\n"
+            "  --repo-storage-ca-path           repository storage CA path\n"
+            "  --repo-storage-host              repository storage host\n"
+            "  --repo-storage-port              repository storage port [default=443]\n"
+            "  --repo-storage-verify-tls        repository storage certificate verify\n"
+            "                                   [default=y]\n"
             "  --repo-type                      type of storage used for the repository\n"
             "                                   [default=posix]\n"
             "\n"
@@ -240,45 +318,57 @@ testRun(void)
         strLstAddZ(argList, "/path/to/pgbackrest");
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "restore");
-        strLstAddZ(argList, "--buffer-size=32768");
-        strLstAddZ(argList, "--repo1-cipher-type=aes-256-cbc");
-        setenv("PGBACKREST_REPO1_CIPHER_PASS", "supersecret", true);
-        strLstAddZ(argList, "--repo1-host=backup.example.net");
-        strLstAddZ(argList, "--link-map=/link1=/dest1");
-        strLstAddZ(argList, "--link-map=/link2=/dest2");
-        strLstAddZ(argList, "--db-include=db1");
-        strLstAddZ(argList, "--db-include=db2");
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "help for restore command");
-        unsetenv("PGBACKREST_REPO1_CIPHER_PASS");
-        TEST_RESULT_STR_Z(helpRender(), commandHelp, "    check text");
+        hrnCfgArgRawZ(argList, cfgOptBufferSize, "32768");
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, "aes-256-cbc");
+        hrnCfgEnvRawZ(cfgOptRepoCipherPass, TEST_CIPHER_PASS);
+        hrnCfgArgRawZ(argList, cfgOptRepoHost, "backup.example.net");
+        hrnCfgArgRawZ(argList, cfgOptLinkMap, "/link1=/dest1");
+        hrnCfgArgRawZ(argList, cfgOptLinkMap, "/link2=/dest2");
+        hrnCfgArgRawZ(argList, cfgOptDbInclude, "db1");
+        hrnCfgArgRawZ(argList, cfgOptDbInclude, "db2");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for restore command");
+        TEST_RESULT_STR_Z(helpRender(helpData), commandHelp, "check text");
+        hrnCfgEnvRemoveRaw(cfgOptRepoCipherPass);
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("errors with specified options");
+
         argList = strLstNew();
         strLstAddZ(argList, "/path/to/pgbackrest");
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "archive-push");
         strLstAddZ(argList, "buffer-size");
         strLstAddZ(argList, "buffer-size");
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "parse too many options");
-        TEST_ERROR(helpRender(), ParamInvalidError, "only one option allowed for option help");
+        TEST_RESULT_VOID(testCfgLoad(argList), "parse too many options");
+        TEST_ERROR(helpRender(helpData), ParamInvalidError, "only one option allowed for option help");
 
         argList = strLstNew();
         strLstAddZ(argList, "/path/to/pgbackrest");
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "archive-push");
         strLstAddZ(argList, BOGUS_STR);
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "parse bogus option");
-        TEST_ERROR(helpRender(), OptionInvalidError, "option 'BOGUS' is not valid for command 'archive-push'");
+        TEST_RESULT_VOID(testCfgLoad(argList), "parse bogus option");
+        TEST_ERROR(helpRender(helpData), OptionInvalidError, "option 'BOGUS' is not valid for command 'archive-push'");
+
+        argList = strLstNew();
+        strLstAddZ(argList, "/path/to/pgbackrest");
+        strLstAddZ(argList, CFGCMD_HELP);
+        strLstAddZ(argList, CFGCMD_ARCHIVE_PUSH);
+        strLstAddZ(argList, CFGOPT_PROCESS);
+        TEST_RESULT_VOID(testCfgLoad(argList), "parse option invalid for command");
+        TEST_ERROR(helpRender(helpData), OptionInvalidError, "option 'process' is not valid for command 'archive-push'");
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("default and current option value");
+
         const char *optionHelp = strZ(strNewFmt(
             "%s - 'archive-push' command - 'buffer-size' option help\n"
             "\n"
-            "Buffer size for file operations.\n"
+            "Buffer size for I/O operations.\n"
             "\n"
-            "Set the buffer size used for copy, compress, and uncompress functions. A\n"
-            "maximum of 3 buffers will be in use at a time per process. An additional\n"
-            "maximum of 256K per process may be used for zlib buffers.\n"
+            "Buffer size used for copy, compress, encrypt, and other operations. The number\n"
+            "of buffers used depends on options and each operation may use additional\n"
+            "memory, e.g. gz compression may use an additional 256KiB of memory.\n"
             "\n"
             "Size can be entered in bytes (default) or KB, MB, GB, TB, or PB where the\n"
             "multiplier is a power of 1024. For example, the case-insensitive value 32k (or\n"
@@ -293,22 +383,33 @@ testRun(void)
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "archive-push");
         strLstAddZ(argList, "buffer-size");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for archive-push command, buffer-size option");
-        TEST_RESULT_STR(helpRender(), strNewFmt("%s\ndefault: 1048576\n", optionHelp), "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for archive-push command, buffer-size option");
+        TEST_RESULT_STR(helpRender(helpData), strNewFmt("%s\ndefault: 1048576\n", optionHelp), "check text");
 
-        strLstAddZ(argList, "--buffer-size=32768");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for archive-push command, buffer-size option");
-        TEST_RESULT_STR(helpRender(), strNewFmt("%s\ncurrent: 32768\ndefault: 1048576\n", optionHelp), "    check text");
+        // Set a current value
+        hrnCfgArgRawZ(argList, cfgOptBufferSize, "32768");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for archive-push command, buffer-size option");
+        TEST_RESULT_STR(
+            helpRender(helpData), strNewFmt("%s\ncurrent: 32768\ndefault: 1048576\n", optionHelp), "check text, current value");
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("deprecated host option names");
+
+        #define HELP_OPTION                                                                                                        \
+            "%s - 'archive-push' command - 'repo-storage-host' option help\n"                                                      \
+            "\n"                                                                                                                   \
+            "Repository storage host.\n"                                                                                           \
+            "\n"                                                                                                                   \
+            "Connect to a host other than the storage (e.g. S3, Azure) endpoint. This is\n"                                        \
+            "typically used for testing.\n"                                                                                        \
+            "\n"
+
+        #define HELP_OPTION_DEPRECATED_NAMES                                                                                       \
+            "deprecated names: repo-azure-host, repo-s3-host\n"
+
         optionHelp = strZ(strNewFmt(
-            "%s - 'archive-push' command - 'repo1-s3-host' option help\n"
-            "\n"
-            "S3 repository host.\n"
-            "\n"
-            "Connect to a host other than the end point. This is typically used for testing.\n",
+            HELP_OPTION
+            HELP_OPTION_DEPRECATED_NAMES,
             helpVersion));
 
         argList = strLstNew();
@@ -316,17 +417,25 @@ testRun(void)
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "archive-push");
         strLstAddZ(argList, "repo1-s3-host");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for archive-push command, repo1-s3-host option");
-        TEST_RESULT_STR_Z(helpRender(), optionHelp, "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for archive-push command, repo1-s3-host option");
+        TEST_RESULT_STR_Z(helpRender(helpData), optionHelp, "check text");
 
-        strLstAddZ(argList, "--repo1-type=s3");
+        optionHelp = strZ(strNewFmt(
+            HELP_OPTION
+            "current: s3-host\n"
+            "\n"
+            HELP_OPTION_DEPRECATED_NAMES,
+            helpVersion));
+
+        // Set a current value for deprecated option name
+        hrnCfgArgRawZ(argList, cfgOptRepoType, "s3");
         strLstAddZ(argList, "--repo1-s3-host=s3-host");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for archive-push command, repo1-s3-host option");
-        TEST_RESULT_STR(helpRender(), strNewFmt("%s\ncurrent: s3-host\n", optionHelp), "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for archive-push command, repo1-s3-host option");
+        TEST_RESULT_STR_Z(helpRender(helpData), optionHelp, "check text, current value");
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("cipher pass redacted");
+
         optionHelp = strZ(strNewFmt(
             "%s - 'archive-push' command - 'repo-cipher-pass' option help\n"
             "\n"
@@ -340,16 +449,17 @@ testRun(void)
         argList = strLstNew();
         strLstAddZ(argList, "/path/to/pgbackrest");
         strLstAddZ(argList, "help");
-        strLstAddZ(argList, "--repo1-cipher-type=aes-256-cbc");
-        setenv("PGBACKREST_REPO1_CIPHER_PASS", "supersecret", true);
+        hrnCfgArgRawZ(argList, cfgOptRepoCipherType, "aes-256-cbc");
+        hrnCfgEnvRawZ(cfgOptRepoCipherPass, TEST_CIPHER_PASS);
         strLstAddZ(argList, "archive-push");
         strLstAddZ(argList, "repo-cipher-pass");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for archive-push command, repo1-s3-host option");
-        unsetenv("PGBACKREST_REPO1_CIPHER_PASS");
-        TEST_RESULT_STR_Z(helpRender(), optionHelp, "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for archive-push command, repo1-s3-host option");
+        TEST_RESULT_STR_Z(helpRender(helpData), optionHelp, "check text");
+        hrnCfgEnvRemoveRaw(cfgOptRepoCipherPass);
 
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("deprecated and new option name produce same results");
+
         optionHelp = strZ(strNewFmt(
             "%s - 'backup' command - 'repo-hardlink' option help\n"
             "\n"
@@ -370,12 +480,20 @@ testRun(void)
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "backup");
         strLstAddZ(argList, "repo-hardlink");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for backup command, repo-hardlink option");
-        TEST_RESULT_STR_Z(helpRender(), optionHelp, "    check text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for backup command, repo-hardlink option");
+        TEST_RESULT_STR_Z(helpRender(helpData), optionHelp, "check text");
+
+        argList = strLstNew();
+        strLstAddZ(argList, "/path/to/pgbackrest");
+        strLstAddZ(argList, "help");
+        strLstAddZ(argList, "backup");
+        strLstAddZ(argList, "hardlink");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for backup command, deprecated hardlink option");
+        TEST_RESULT_STR_Z(helpRender(helpData), optionHelp, "check text");
 
-        // Check admonition
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("check admonition");
+
         optionHelp = strZ(strNewFmt(
             "%s - 'backup' command - 'repo-retention-archive' option help\n"
             "\n"
@@ -406,33 +524,31 @@ testRun(void)
         strLstAddZ(argList, "help");
         strLstAddZ(argList, "backup");
         strLstAddZ(argList, "repo-retention-archive");
-        TEST_RESULT_VOID(
-            configParse(strLstSize(argList), strLstPtr(argList), false), "help for backup command, repo-retention-archive option");
-        TEST_RESULT_STR_Z(helpRender(), optionHelp, "    check admonition text");
+        TEST_RESULT_VOID(testCfgLoad(argList), "help for backup command, repo-retention-archive option");
+        TEST_RESULT_STR_Z(helpRender(helpData), optionHelp, "check admonition text");
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("cmdHelp()"))
+    if (testBegin("cmdHelp() to file"))
     {
         StringList *argList = strLstNew();
         strLstAddZ(argList, "/path/to/pgbackrest");
-        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "parse help from empty command line");
+        TEST_RESULT_VOID(testCfgLoad(argList), "parse help from empty command line");
 
         // Redirect stdout to a file
         int stdoutSave = dup(STDOUT_FILENO);
-        String *stdoutFile = strNewFmt("%s/stdout.help", testPath());
 
-        THROW_ON_SYS_ERROR(freopen(strZ(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
+        THROW_ON_SYS_ERROR(freopen(TEST_PATH "/stdout.help", "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
 
         // Not in a test wrapper to avoid writing to stdout
-        cmdHelp();
+        cmdHelp(helpData);
 
         // Restore normal stdout
         dup2(stdoutSave, STDOUT_FILENO);
 
-        Storage *storage = storagePosixNewP(strNew(testPath()));
-        TEST_RESULT_STR_Z(strNewBuf(storageGetP(storageNewReadP(storage, stdoutFile))), generalHelp, "    check text");
+        Storage *storage = storagePosixNewP(TEST_PATH_STR);
+        TEST_STORAGE_GET(storage, TEST_PATH "/stdout.help", generalHelp, .comment = "check text sent to file");
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
