@@ -79,18 +79,19 @@ testSuite(CompressType type, const char *decompressCmd)
     Buffer *compressed = NULL;
     Buffer *decompressed = bufNewC(simpleData, strlen(simpleData));
 
-    VariantList *compressParamList = varLstNew();
-    varLstAdd(compressParamList, varNewUInt(1));
+    PackWrite *packWrite = pckWriteNewP();
+    pckWriteI32P(packWrite, 1);
+    pckWriteEndP(packWrite);
 
     // Create default storage object for testing
-    Storage *storageTest = storagePosixNewP(strNew(testPath()), .write = true);
+    Storage *storageTest = storagePosixNewP(TEST_PATH_STR, .write = true);
 
     TEST_TITLE("simple data");
 
     TEST_ASSIGN(
         compressed,
         testCompress(
-            compressFilterVar(strNewFmt("%sCompress", strZ(compressTypeStr(type))), compressParamList), decompressed, 1024,
+            compressFilterPack(compressHelperLocal[type].compressType, pckWriteResult(packWrite)), decompressed, 1024,
             256 * 1024 * 1024),
         "simple data - compress large in/large out buffer");
 
@@ -98,7 +99,7 @@ testSuite(CompressType type, const char *decompressCmd)
     TEST_TITLE("compressed output can be decompressed with command-line tool");
 
     storagePutP(storageNewWriteP(storageTest, STRDEF("test.cmp")), compressed);
-    TEST_SYSTEM_FMT("%s {[path]}/test.cmp > {[path]}/test.out", decompressCmd);
+    HRN_SYSTEM_FMT("%s " TEST_PATH "/test.cmp > " TEST_PATH "/test.out", decompressCmd);
     TEST_RESULT_BOOL(bufEq(decompressed, storageGetP(storageNewReadP(storageTest, STRDEF("test.out")))), true, "check output");
 
     TEST_RESULT_BOOL(
@@ -116,8 +117,7 @@ testSuite(CompressType type, const char *decompressCmd)
     TEST_RESULT_BOOL(
         bufEq(
             decompressed,
-            testDecompress(
-                compressFilterVar(strNewFmt("%sDecompress", strZ(compressTypeStr(type))), NULL), compressed, 1024, 1024)),
+            testDecompress(compressFilterPack(compressHelperLocal[type].decompressType, NULL), compressed, 1024, 1024)),
         true, "simple data - decompress large in/large out buffer");
 
     TEST_RESULT_BOOL(
@@ -171,7 +171,7 @@ testSuite(CompressType type, const char *decompressCmd)
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
@@ -337,7 +337,12 @@ testRun(void)
 
         TEST_RESULT_UINT(compressTypeEnum(STRDEF("none")), compressTypeNone, "none enum");
         TEST_RESULT_UINT(compressTypeEnum(STRDEF("gz")), compressTypeGz, "gz enum");
-        TEST_ERROR(compressTypeEnum(strNew(BOGUS_STR)), AssertError, "invalid compression type 'BOGUS'");
+        TEST_ERROR(compressTypeEnum(STRDEF(BOGUS_STR)), AssertError, "invalid compression type 'BOGUS'");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("compressTypeStr()");
+
+        TEST_RESULT_STR_Z(compressTypeStr(compressTypeGz), "gz", "gz str");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("compressTypePresent()");
@@ -352,9 +357,9 @@ testRun(void)
         TEST_RESULT_UINT(compressTypeFromName(STRDEF("file.gz")), compressTypeGz, "type from name");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("compressFilterVar()");
+        TEST_TITLE("compressFilterPack()");
 
-        TEST_RESULT_PTR(compressFilterVar(STRDEF("BOGUS"), 0), NULL, "no filter match");
+        TEST_RESULT_PTR(compressFilterPack(STRID5("bogus", 0x13a9de20), NULL), NULL, "no filter match");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("compressExtStr()");
@@ -365,7 +370,7 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("compressExtCat()");
 
-        String *file = strNew("file");
+        String *file = strCatZ(strNew(), "file");
         TEST_RESULT_VOID(compressExtCat(file, compressTypeGz), "cat gz ext");
         TEST_RESULT_STR_Z(file, "file.gz", "    check gz ext");
 
@@ -383,5 +388,5 @@ testRun(void)
         TEST_RESULT_INT(compressLevelDefault(compressTypeGz), 6, "gz level=6");
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
