@@ -20,7 +20,7 @@ HTTP user agent header
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
@@ -29,16 +29,16 @@ testRun(void)
     if (testBegin("httpUriEncode() and httpUriDecode()"))
     {
         TEST_RESULT_STR(httpUriEncode(NULL, false), NULL, "null encodes to null");
-        TEST_RESULT_STR_Z(httpUriEncode(strNew("0-9_~/A Z.az"), false), "0-9_~%2FA%20Z.az", "non-path encoding");
-        TEST_RESULT_STR_Z(httpUriEncode(strNew("0-9_~/A Z.az"), true), "0-9_~/A%20Z.az", "path encoding");
+        TEST_RESULT_STR_Z(httpUriEncode(STRDEF("0-9_~/A Z.az"), false), "0-9_~%2FA%20Z.az", "non-path encoding");
+        TEST_RESULT_STR_Z(httpUriEncode(STRDEF("0-9_~/A Z.az"), true), "0-9_~/A%20Z.az", "path encoding");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("decode");
 
         TEST_RESULT_STR(httpUriDecode(NULL), NULL, "null decodes to null");
-        TEST_RESULT_STR_Z(httpUriDecode(strNew("0-9_~%2FA%20Z.az")), "0-9_~/A Z.az", "valid decode");
-        TEST_ERROR(httpUriDecode(strNew("%A")), FormatError, "invalid escape sequence length in '%A'");
-        TEST_ERROR(httpUriDecode(strNew("%XX")), FormatError, "unable to convert base 16 string 'XX' to unsigned int");
+        TEST_RESULT_STR_Z(httpUriDecode(STRDEF("0-9_~%2FA%20Z.az")), "0-9_~/A Z.az", "valid decode");
+        TEST_ERROR(httpUriDecode(STRDEF("%A")), FormatError, "invalid escape sequence length in '%A'");
+        TEST_ERROR(httpUriDecode(STRDEF("%XX")), FormatError, "unable to convert base 16 string 'XX' to unsigned int");
     }
 
     // *****************************************************************************************************************************
@@ -49,7 +49,7 @@ testRun(void)
             httpDateToTime(STRDEF("Wed,  1 Oct 2015 07:28:00 GMT")), FormatError, "unable to convert base 10 string ' 1' to int");
         TEST_RESULT_INT(httpDateToTime(STRDEF("Wed, 21 Oct 2015 07:28:00 GMT")), 1445412480, "convert HTTP date to time_t");
 
-        TEST_RESULT_STR_Z(httpDateFromTime(1592743579), "Sun, 21 Jun 2020 12:46:19 GMT", "convert time_t to HTTP date")
+        TEST_RESULT_STR_Z(httpDateFromTime(1592743579), "Sun, 21 Jun 2020 12:46:19 GMT", "convert time_t to HTTP date");
     }
 
     // *****************************************************************************************************************************
@@ -66,16 +66,16 @@ testRun(void)
         }
         MEM_CONTEXT_TEMP_END();
 
-        TEST_RESULT_PTR(httpHeaderAdd(header, strNew("key2"), strNew("value2")), header, "add header");
-        TEST_RESULT_PTR(httpHeaderPut(header, strNew("key2"), strNew("value2a")), header, "put header");
-        TEST_RESULT_PTR(httpHeaderAdd(header, strNew("key2"), strNew("value2b")), header, "add header");
+        TEST_RESULT_PTR(httpHeaderAdd(header, STRDEF("key2"), STRDEF("value2")), header, "add header");
+        TEST_RESULT_PTR(httpHeaderPut(header, STRDEF("key2"), STRDEF("value2a")), header, "put header");
+        TEST_RESULT_PTR(httpHeaderAdd(header, STRDEF("key2"), STRDEF("value2b")), header, "add header");
 
-        TEST_RESULT_PTR(httpHeaderAdd(header, strNew("key1"), strNew("value1")), header, "add header");
-        TEST_RESULT_STR_Z(strLstJoin(httpHeaderList(header), ", "), "key1, key2", "header list");
+        TEST_RESULT_PTR(httpHeaderAdd(header, STRDEF("key1"), STRDEF("value1")), header, "add header");
+        TEST_RESULT_STRLST_Z(httpHeaderList(header), "key1\nkey2\n", "header list");
 
-        TEST_RESULT_STR_Z(httpHeaderGet(header, strNew("key1")), "value1", "get value");
-        TEST_RESULT_STR_Z(httpHeaderGet(header, strNew("key2")), "value2a, value2b", "get value");
-        TEST_RESULT_STR(httpHeaderGet(header, strNew(BOGUS_STR)), NULL, "get missing value");
+        TEST_RESULT_STR_Z(httpHeaderGet(header, STRDEF("key1")), "value1", "get value");
+        TEST_RESULT_STR_Z(httpHeaderGet(header, STRDEF("key2")), "value2a, value2b", "get value");
+        TEST_RESULT_STR(httpHeaderGet(header, STRDEF(BOGUS_STR)), NULL, "get missing value");
 
         TEST_RESULT_STR_Z(httpHeaderToLog(header), "{key1: 'value1', key2: 'value2a, value2b'}", "log output");
 
@@ -87,8 +87,8 @@ testRun(void)
         strLstAddZ(redact, "secret");
 
         TEST_ASSIGN(header, httpHeaderNew(redact), "new header with redaction");
-        httpHeaderAdd(header, strNew("secret"), strNew("secret-value"));
-        httpHeaderAdd(header, strNew("public"), strNew("public-value"));
+        httpHeaderAdd(header, STRDEF("secret"), STRDEF("secret-value"));
+        httpHeaderAdd(header, STRDEF("public"), STRDEF("public-value"));
 
         TEST_RESULT_STR_Z(httpHeaderToLog(header), "{public: 'public-value', secret: <redacted>}", "log output");
 
@@ -126,20 +126,20 @@ testRun(void)
         TEST_RESULT_STR(httpQueryRenderP(NULL), NULL, "null query renders null");
         TEST_RESULT_STR(httpQueryRenderP(query), NULL, "empty query renders null");
 
-        TEST_RESULT_PTR(httpQueryAdd(query, strNew("key2"), strNew("value2")), query, "add query");
-        TEST_ERROR(httpQueryAdd(query, strNew("key2"), strNew("value2")), AssertError, "key 'key2' already exists");
-        TEST_RESULT_PTR(httpQueryPut(query, strNew("key2"), strNew("value2a")), query, "put query");
+        TEST_RESULT_PTR(httpQueryAdd(query, STRDEF("key2"), STRDEF("value2")), query, "add query");
+        TEST_ERROR(httpQueryAdd(query, STRDEF("key2"), STRDEF("value2")), AssertError, "key 'key2' already exists");
+        TEST_RESULT_PTR(httpQueryPut(query, STRDEF("key2"), STRDEF("value2a")), query, "put query");
         TEST_RESULT_STR_Z(httpQueryRenderP(query), "key2=value2a", "render one query item");
 
-        TEST_RESULT_PTR(httpQueryAdd(query, strNew("key1"), strNew("value 1?")), query, "add query");
-        TEST_RESULT_STR_Z(strLstJoin(httpQueryList(query), ", "), "key1, key2", "query list");
+        TEST_RESULT_PTR(httpQueryAdd(query, STRDEF("key1"), STRDEF("value 1?")), query, "add query");
+        TEST_RESULT_STRLST_Z(httpQueryList(query), "key1\nkey2\n", "query list");
         TEST_RESULT_STR_Z(httpQueryRenderP(query), "key1=value%201%3F&key2=value2a", "render two query items");
         TEST_RESULT_STR_Z(
             httpQueryRenderP(query, .redact = true), "key1=value%201%3F&key2=<redacted>", "render two query items with redaction");
 
-        TEST_RESULT_STR_Z(httpQueryGet(query, strNew("key1")), "value 1?", "get value");
-        TEST_RESULT_STR_Z(httpQueryGet(query, strNew("key2")), "value2a", "get value");
-        TEST_RESULT_STR(httpQueryGet(query, strNew(BOGUS_STR)), NULL, "get missing value");
+        TEST_RESULT_STR_Z(httpQueryGet(query, STRDEF("key1")), "value 1?", "get value");
+        TEST_RESULT_STR_Z(httpQueryGet(query, STRDEF("key2")), "value2a", "get value");
+        TEST_RESULT_STR(httpQueryGet(query, STRDEF(BOGUS_STR)), NULL, "get missing value");
 
         TEST_RESULT_STR_Z(httpQueryToLog(query), "{key1: 'value 1?', key2: <redacted>}", "log output");
 
@@ -173,45 +173,136 @@ testRun(void)
         TEST_RESULT_VOID(httpQueryFree(query), "free");
     }
 
+    // *****************************************************************************************************************************
+    if (testBegin("HttpUrl"))
+    {
+        HttpUrl *url = NULL;
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("invalid url");
+
+        TEST_ERROR(httpUrlNewParseP(STRDEF("ftp://" BOGUS_STR)), FormatError, "invalid URL 'ftp://BOGUS'");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("HttpProtocolTypeStr");
+
+        TEST_RESULT_STR_Z(httpProtocolTypeStr(httpProtocolTypeHttp), "http", "check http");
+        TEST_RESULT_STR_Z(httpProtocolTypeStr(httpProtocolTypeAny), NULL, "check any");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("simple http");
+
+        TEST_ASSIGN(url, httpUrlNewParseP(STRDEF("http://test"), .type = httpProtocolTypeHttp), "new");
+        TEST_RESULT_STR_Z(httpUrl(url), "http://test", "check url");
+        TEST_RESULT_STR_Z(httpUrlHost(url), "test", "check host");
+        TEST_RESULT_STR_Z(httpUrlPath(url), "/", "check path");
+        TEST_RESULT_UINT(httpUrlPort(url), 80, "check port");
+        TEST_RESULT_UINT(httpUrlProtocolType(url), httpProtocolTypeHttp, "check protocol");
+        TEST_RESULT_STR_Z(httpUrlToLog(url), "{http://test:80/}", "check log");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("host and port");
+
+        TEST_ASSIGN(url, httpUrlNewParseP(STRDEF("gcs:4443"), .type = httpProtocolTypeHttps), "new");
+        TEST_RESULT_STR_Z(httpUrl(url), "gcs:4443", "check url");
+        TEST_RESULT_STR_Z(httpUrlHost(url), "gcs", "check host");
+        TEST_RESULT_STR_Z(httpUrlPath(url), "/", "check path");
+        TEST_RESULT_UINT(httpUrlPort(url), 4443, "check port");
+        TEST_RESULT_UINT(httpUrlProtocolType(url), httpProtocolTypeHttps, "check protocol");
+        TEST_RESULT_STR_Z(httpUrlToLog(url), "{https://gcs:4443/}", "check log");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("http but expected https");
+
+        TEST_ERROR(
+            httpUrlNewParseP(STRDEF("http://test"), .type = httpProtocolTypeHttps), FormatError,
+            "expected protocol 'https' in URL 'http://test'");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("https with port and path");
+
+        TEST_ASSIGN(url, httpUrlNewParseP(STRDEF("https://test.com:445/path")), "new");
+        TEST_RESULT_STR_Z(httpUrl(url), "https://test.com:445/path", "check url");
+        TEST_RESULT_STR_Z(httpUrlHost(url), "test.com", "check host");
+        TEST_RESULT_STR_Z(httpUrlPath(url), "/path", "check path");
+        TEST_RESULT_UINT(httpUrlPort(url), 445, "check port");
+        TEST_RESULT_UINT(httpUrlProtocolType(url), httpProtocolTypeHttps, "check protocol");
+        TEST_RESULT_STR_Z(httpUrlToLog(url), "{https://test.com:445/path}", "check log");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("host only");
+
+        TEST_ASSIGN(url, httpUrlNewParseP(STRDEF("test.com"), .type = httpProtocolTypeHttps), "new");
+        TEST_RESULT_STR_Z(httpUrl(url), "test.com", "check url");
+        TEST_RESULT_STR_Z(httpUrlHost(url), "test.com", "check host");
+        TEST_RESULT_STR_Z(httpUrlPath(url), "/", "check path");
+        TEST_RESULT_UINT(httpUrlPort(url), 443, "check port");
+        TEST_RESULT_UINT(httpUrlProtocolType(url), httpProtocolTypeHttps, "check protocol");
+        TEST_RESULT_STR_Z(httpUrlToLog(url), "{https://test.com:443/}", "check log");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("IPv6");
+
+        TEST_ASSIGN(url, httpUrlNewParseP(STRDEF("http://[2001:db8::ff00:42:8329]:81"), .type = httpProtocolTypeHttp), "new");
+        TEST_RESULT_STR_Z(httpUrl(url), "http://[2001:db8::ff00:42:8329]:81", "check url");
+        TEST_RESULT_STR_Z(httpUrlHost(url), "2001:db8::ff00:42:8329", "check host");
+        TEST_RESULT_STR_Z(httpUrlPath(url), "/", "check path");
+        TEST_RESULT_UINT(httpUrlPort(url), 81, "check port");
+        TEST_RESULT_UINT(httpUrlProtocolType(url), httpProtocolTypeHttp, "check protocol");
+        TEST_RESULT_STR_Z(httpUrlToLog(url), "{http://[2001:db8::ff00:42:8329]:81/}", "check log");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("IPv6 no port");
+
+        TEST_ASSIGN(url, httpUrlNewParseP(STRDEF("http://[2001:db8::ff00:42:8329]/url"), .type = httpProtocolTypeHttp), "new");
+        TEST_RESULT_STR_Z(httpUrl(url), "http://[2001:db8::ff00:42:8329]/url", "check url");
+        TEST_RESULT_STR_Z(httpUrlHost(url), "2001:db8::ff00:42:8329", "check host");
+        TEST_RESULT_STR_Z(httpUrlPath(url), "/url", "check path");
+        TEST_RESULT_UINT(httpUrlPort(url), 80, "check port");
+        TEST_RESULT_UINT(httpUrlProtocolType(url), httpProtocolTypeHttp, "check protocol");
+        TEST_RESULT_STR_Z(httpUrlToLog(url), "{http://[2001:db8::ff00:42:8329]:80/url}", "check log");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("free");
+
+        TEST_RESULT_VOID(httpUrlFree(url), "free");
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("HttpClient"))
     {
         HttpClient *client = NULL;
 
-        TEST_ASSIGN(client, httpClientNew(sckClientNew(strNew("localhost"), hrnServerPort(0), 500), 500), "new client");
+        TEST_ASSIGN(client, httpClientNew(sckClientNew(STRDEF("localhost"), hrnServerPort(0), 500, 500), 500), "new client");
 
         TEST_ERROR_FMT(
-            httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), HostConnectError,
+            httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), HostConnectError,
             "unable to connect to 'localhost:%u': [111] Connection refused", hrnServerPort(0));
 
-        HARNESS_FORK_BEGIN()
+        HRN_FORK_BEGIN()
         {
-            HARNESS_FORK_CHILD_BEGIN(0, true)
+            HRN_FORK_CHILD_BEGIN(.prefix = "test server", .timeout = 5000)
             {
                 // Start HTTP test server
-                TEST_RESULT_VOID(
-                    hrnServerRunP(
-                        ioFdReadNew(strNew("test server read"), HARNESS_FORK_CHILD_READ(), 5000), hrnServerProtocolSocket),
-                    "http server run");
+                TEST_RESULT_VOID(hrnServerRunP(HRN_FORK_CHILD_READ(), hrnServerProtocolSocket), "http server run");
             }
-            HARNESS_FORK_CHILD_END();
+            HRN_FORK_CHILD_END();
 
-            HARNESS_FORK_PARENT_BEGIN()
+            HRN_FORK_PARENT_BEGIN()
             {
-                IoWrite *http = hrnServerScriptBegin(
-                    ioFdWriteNew(strNew("test client write"), HARNESS_FORK_PARENT_WRITE_PROCESS(0), 2000));
+                IoWrite *http = hrnServerScriptBegin(HRN_FORK_PARENT_WRITE(0));
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("create client");
 
                 ioBufferSizeSet(35);
 
-                TEST_ASSIGN(client, httpClientNew(sckClientNew(hrnServerHost(), hrnServerPort(0), 5000), 5000), "new client");
+                TEST_ASSIGN(client, httpClientNew(sckClientNew(hrnServerHost(), hrnServerPort(0), 5000, 5000), 5000), "new client");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("no output from server");
 
-                client->timeout = 0;
+                client->pub.timeout = 0;
 
                 hrnServerScriptAccept(http);
 
@@ -221,7 +312,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FileReadError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FileReadError,
                     "unexpected eof while reading line");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -235,7 +326,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "HTTP response status 'HTTP/1.0 200 OK' should be CR-terminated");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -249,7 +340,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "HTTP response 'HTTP/1.0 200' has invalid length");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -263,7 +354,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "HTTP version of response 'HTTP/1 200 OK' must be HTTP/1.1 or HTTP/1.0");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -277,7 +368,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "response status '200OK' must have a space after the status code");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -291,7 +382,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FileReadError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FileReadError,
                     "unexpected eof while reading line");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -305,7 +396,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "header 'header-value' missing colon");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -319,7 +410,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "only 'chunked' is supported for 'transfer-encoding' header");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -333,7 +424,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), FormatError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), FormatError,
                     "'transfer-encoding' and 'content-length' headers are both set");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -347,7 +438,7 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ERROR(
-                    httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), ServiceError,
+                    httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), ServiceError,
                     "[503] Slow Down");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -368,13 +459,13 @@ testRun(void)
                     http, "HTTP/1.1 200 OK\r\nkey1:0\r\n key2 : value2\r\nConnection:ack\r\ncontent-length:0\r\n\r\n");
 
                 HttpHeader *headerRequest = httpHeaderNew(NULL);
-                httpHeaderAdd(headerRequest, strNew("host"), strNew("myhost.com"));
+                httpHeaderAdd(headerRequest, STRDEF("host"), STRDEF("myhost.com"));
 
                 HttpQuery *query = httpQueryNewP();
-                httpQueryAdd(query, strNew("name"), strNew("/path/A Z.txt"));
-                httpQueryAdd(query, strNew("type"), strNew("test"));
+                httpQueryAdd(query, STRDEF("name"), STRDEF("/path/A Z.txt"));
+                httpQueryAdd(query, STRDEF("type"), STRDEF("test"));
 
-                client->timeout = 5000;
+                client->pub.timeout = 5000;
 
                 HttpRequest *request = NULL;
                 HttpResponse *response = NULL;
@@ -382,7 +473,7 @@ testRun(void)
                 MEM_CONTEXT_TEMP_BEGIN()
                 {
                     TEST_ASSIGN(
-                        request, httpRequestNewP(client, strNew("GET"), strNew("/"), .query = query, .header = headerRequest),
+                        request, httpRequestNewP(client, STRDEF("GET"), STRDEF("/"), .query = query, .header = headerRequest),
                         "request");
                     TEST_ASSIGN(response, httpRequestResponse(request, false), "request");
 
@@ -392,7 +483,7 @@ testRun(void)
                 MEM_CONTEXT_TEMP_END();
 
                 TEST_RESULT_STR_Z(httpRequestVerb(request), "GET", "check request verb");
-                TEST_RESULT_STR_Z(httpRequestUri(request), "/", "check request uri");
+                TEST_RESULT_STR_Z(httpRequestPath(request), "/", "check request path");
                 TEST_RESULT_STR_Z(
                     httpQueryRenderP(httpRequestQuery(request)), "name=%2Fpath%2FA%20Z.txt&type=test", "check request query");
                 TEST_RESULT_PTR_NE(httpRequestHeader(request), NULL, "check request headers");
@@ -417,7 +508,7 @@ testRun(void)
 
                 hrnServerScriptClose(http);
 
-                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, strNew("HEAD"), strNew("/")), true), "request");
+                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, STRDEF("HEAD"), STRDEF("/")), true), "request");
                 TEST_RESULT_UINT(httpResponseCode(response), 200, "check response code");
                 TEST_RESULT_STR_Z(httpResponseReason(response), "OK", "check response message");
                 TEST_RESULT_BOOL(httpResponseEof(response), true, "io is eof");
@@ -433,7 +524,7 @@ testRun(void)
                 hrnServerScriptExpectZ(http, "HEAD / HTTP/1.1\r\n" TEST_USER_AGENT "\r\n");
                 hrnServerScriptReplyZ(http, "HTTP/1.1 200 OK\r\nTransfer-Encoding: chunked\r\n\r\n");
 
-                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, strNew("HEAD"), strNew("/")), true), "request");
+                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, STRDEF("HEAD"), STRDEF("/")), true), "request");
                 TEST_RESULT_UINT(httpResponseCode(response), 200, "check response code");
                 TEST_RESULT_STR_Z(httpResponseReason(response), "OK", "check response message");
                 TEST_RESULT_BOOL(httpResponseEof(response), true, "io is eof");
@@ -449,7 +540,7 @@ testRun(void)
 
                 hrnServerScriptClose(http);
 
-                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, strNew("HEAD"), strNew("/")), true), "request");
+                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, STRDEF("HEAD"), STRDEF("/")), true), "request");
                 TEST_RESULT_UINT(httpResponseCode(response), 200, "check response code");
                 TEST_RESULT_STR_Z(httpResponseReason(response), "OK", "check response message");
                 TEST_RESULT_BOOL(httpResponseEof(response), true, "io is eof");
@@ -478,7 +569,7 @@ testRun(void)
                 hrnServerScriptExpectZ(http, "GET / HTTP/1.1\r\n" TEST_USER_AGENT "\r\n");
                 hrnServerScriptReplyZ(http, "HTTP/1.1 404 Not Found\r\n\r\n");
 
-                TEST_ASSIGN(request, httpRequestNewP(client, strNew("GET"), strNew("/")), "request");
+                TEST_ASSIGN(request, httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), "request");
                 TEST_ASSIGN(response, httpRequestResponse(request, false), "response");
                 TEST_RESULT_UINT(httpResponseCode(response), 404, "check response code");
                 TEST_RESULT_BOOL(httpResponseCodeOk(response), false, "check response code error");
@@ -489,7 +580,7 @@ testRun(void)
                 TEST_ERROR(
                     httpRequestError(request, response), ProtocolError,
                     "HTTP request failed with 404 (Not Found):\n"
-                    "*** URI/Query ***:\n"
+                    "*** Path/Query ***:\n"
                     "/");
 
                 // -----------------------------------------------------------------------------------------------------------------
@@ -501,13 +592,13 @@ testRun(void)
                 StringList *headerRedact = strLstNew();
                 strLstAdd(headerRedact, STRDEF("hdr2"));
                 headerRequest = httpHeaderNew(headerRedact);
-                httpHeaderAdd(headerRequest, strNew("hdr1"), strNew("1"));
-                httpHeaderAdd(headerRequest, strNew("hdr2"), strNew("2"));
+                httpHeaderAdd(headerRequest, STRDEF("hdr1"), STRDEF("1"));
+                httpHeaderAdd(headerRequest, STRDEF("hdr2"), STRDEF("2"));
 
                 TEST_ASSIGN(
                     request,
                     httpRequestNewP(
-                        client, strNew("GET"), strNew("/"), .query = httpQueryAdd(httpQueryNewP(), STRDEF("a"), STRDEF("b")),
+                        client, STRDEF("GET"), STRDEF("/"), .query = httpQueryAdd(httpQueryNewP(), STRDEF("a"), STRDEF("b")),
                         .header = headerRequest),
                     "request");
                 TEST_ASSIGN(response, httpRequestResponse(request, false), "response");
@@ -520,7 +611,7 @@ testRun(void)
                 TEST_ERROR(
                     httpRequestError(request, response), ProtocolError,
                     "HTTP request failed with 403:\n"
-                    "*** URI/Query ***:\n"
+                    "*** Path/Query ***:\n"
                     "/?a=b\n"
                     "*** Request Headers ***:\n"
                     "hdr1: 1\n"
@@ -546,8 +637,8 @@ testRun(void)
                     response,
                     httpRequestResponse(
                         httpRequestNewP(
-                            client, strNew("GET"), strNew("/path/file 1.txt"),
-                            .header = httpHeaderAdd(httpHeaderNew(NULL), strNew("content-length"), strNew("30")),
+                            client, STRDEF("GET"), httpUriEncode(STRDEF("/path/file 1.txt"), true),
+                            .header = httpHeaderAdd(httpHeaderNew(NULL), STRDEF("content-length"), STRDEF("30")),
                             .content = BUFSTRDEF("012345678901234567890123456789")), true),
                     "request");
                 TEST_RESULT_STR_Z(
@@ -570,7 +661,9 @@ testRun(void)
                 hrnServerScriptReplyZ(http, "HTTP/1.1 200 OK\r\ncontent-length:32\r\n\r\n01234567890123456789012345678901");
 
                 TEST_ASSIGN(
-                    response, httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/path/file 1.txt")), true),
+                    response,
+                    httpRequestResponse(
+                        httpRequestNewP(client, STRDEF("GET"), httpUriEncode(STRDEF("/path/file 1.txt"), true)), true),
                     "request");
                 TEST_RESULT_STR_Z(strNewBuf(httpResponseContent(response)),  "01234567890123456789012345678901", "check response");
                 TEST_RESULT_UINT(httpResponseRead(response, bufNew(1), true), 0, "call internal read to check eof");
@@ -584,7 +677,9 @@ testRun(void)
                 hrnServerScriptClose(http);
 
                 TEST_ASSIGN(
-                    response, httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/path/file 1.txt")), false),
+                    response,
+                    httpRequestResponse(
+                        httpRequestNewP(client, STRDEF("GET"), httpUriEncode(STRDEF("/path/file 1.txt"), true)), false),
                     "request");
                 TEST_RESULT_PTR_NE(response->session, NULL, "session is busy");
                 TEST_ERROR(ioRead(httpResponseIoRead(response), bufNew(32)), FileReadError, "unexpected EOF reading HTTP content");
@@ -603,7 +698,7 @@ testRun(void)
                     "10\r\n0123456789012345\r\n"
                     "0\r\n\r\n");
 
-                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, strNew("GET"), strNew("/")), false), "request");
+                TEST_ASSIGN(response, httpRequestResponse(httpRequestNewP(client, STRDEF("GET"), STRDEF("/")), false), "request");
                 TEST_RESULT_STR_Z(
                     httpHeaderToLog(httpResponseHeader(response)),  "{transfer-encoding: 'chunked'}", "check response headers");
 
@@ -618,15 +713,15 @@ testRun(void)
                 hrnServerScriptClose(http);
                 hrnServerScriptEnd(http);
             }
-            HARNESS_FORK_PARENT_END();
+            HRN_FORK_PARENT_END();
         }
-        HARNESS_FORK_END();
+        HRN_FORK_END();
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("statistics exist");
 
-        TEST_RESULT_BOOL(varLstSize(kvKeyList(statToKv())) > 0, true, "check");
+        TEST_RESULT_BOOL(varLstEmpty(kvKeyList(statToKv())), false, "check");
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
