@@ -6,26 +6,43 @@ Test Strings
 // Declare a static const string for testing
 STRING_STATIC(TEST_STRING, "a very interesting string!");
 
+/***********************************************************************************************************************************
+Test enum and function to ensure 64-bit enums work properly
+***********************************************************************************************************************************/
+typedef enum
+{
+    testStringIdEnumAes256Cbc = STRID5("aes-256-cbc", 0xc43dfbbcdcca10),
+    testStringIdEnumRemote = STRID6("remote", 0x1543cd1521),
+    testStringIdEnumTest = STRID5("test", 0xa4cb40),
+} TestStringIdEnum;
+
+static TestStringIdEnum
+testStringIdEnumFunc(TestStringIdEnum testEnum)
+{
+    return testEnum;
+}
+
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
 
     // *****************************************************************************************************************************
-    if (testBegin("strNew(), strNewBuf(), strNewN(), strEmpty(), strZ(), strZNull(), strSize(), and strFree()"))
+    if (testBegin("strNew*(), strEmpty(), strZ(), strZNull(), strSize(), and strFree()"))
     {
         // We don't want this struct to grow since there are generally a lot of strings, so make sure it doesn't grow without us
         // knowing about it
-        TEST_RESULT_UINT(sizeof(StringConst), TEST_64BIT() ? 16 : 12, "check StringConst struct size");
+        TEST_RESULT_UINT(sizeof(StringPub), TEST_64BIT() ? 16 : 12, "check StringConst struct size");
 
         // Test the size macro
         TEST_RESULT_VOID(CHECK_SIZE(555), "valid size");
         TEST_ERROR(CHECK_SIZE(STRING_SIZE_MAX + 1), AssertError, "string size must be <= 1073741824 bytes");
 
-        String *string = strNew("static string");
+        String *string = strNewZ("static string");
+        TEST_RESULT_BOOL(string->pub.buffer == (char *)(string + 1), true, "string has fixed buffer");
         TEST_RESULT_STR_Z(string, "static string", "new with static string");
         TEST_RESULT_UINT(strSize(string), 13, "check size");
         TEST_RESULT_BOOL(strEmpty(string), false, "is not empty");
@@ -35,7 +52,7 @@ testRun(void)
         TEST_RESULT_VOID(strFree(string), "free string");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_RESULT_STR_Z(strNewN("testmorestring", 4), "test", "new string with size limit");
+        TEST_RESULT_STR_Z(strNewZN("testmorestring", 4), "test", "new string with size limit");
 
         // -------------------------------------------------------------------------------------------------------------------------
         Buffer *buffer = bufNew(8);
@@ -44,6 +61,11 @@ testRun(void)
 
         TEST_RESULT_STR_Z(strNewBuf(buffer), "12345678", "new string from buffer");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("new from double");
+
+        TEST_RESULT_STR_Z(strNewDbl(999.1), "999.1", "new");
+
         // -------------------------------------------------------------------------------------------------------------------------
         string = strNewFmt("formatted %s %04d", "string", 1);
         TEST_RESULT_STR_Z(string, "formatted string 0001", "new with formatted string");
@@ -55,9 +77,16 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("empty string is allocated extra space");
 
-        TEST_ASSIGN(string, strNew(""), "new empty string");
-        TEST_RESULT_UINT(string->size, 0, "    check size");
-        TEST_RESULT_UINT(string->extra, 64, "    check extra");
+        TEST_ASSIGN(string, strNew(), "new empty string");
+        TEST_RESULT_UINT(string->pub.size, 0, "check size");
+        TEST_RESULT_UINT(string->pub.extra, 0, "check extra");
+        TEST_RESULT_PTR(string->pub.buffer, EMPTY_STR->pub.buffer, "check buffer");
+        TEST_RESULT_VOID(strFree(string), "free string");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strNewEncode()");
+
+        TEST_RESULT_STR_Z(strNewEncode(encodeBase64, BUFSTRDEF("zz")), "eno=", "encode base64");
     }
 
     // *****************************************************************************************************************************
@@ -94,21 +123,36 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("strCat(), strCatChr(), and strCatFmt()"))
+    if (testBegin("strCat*()"))
     {
-        String *string = strNew("XXXX");
-        String *string2 = strNew("ZZZZ");
+        String *string = strCatZ(strNew(), "XXX");
+        String *string2 = strNewZ("ZZZZ");
 
+        TEST_RESULT_STR_Z(strCatN(string, STRDEF("XX"), 1), "XXXX", "cat N chars");
+        TEST_RESULT_UINT(string->pub.extra, 60, "check extra");
+        TEST_RESULT_STR_Z(strCatZ(string, ""), "XXXX", "cat empty string");
+        TEST_RESULT_UINT(string->pub.extra, 60, "check extra");
+        TEST_RESULT_STR_Z(strCatEncode(string, encodeBase64, BUFSTRDEF("")), "XXXX", "cat empty encode");
+        TEST_RESULT_UINT(string->pub.extra, 60, "check extra");
         TEST_RESULT_STR_Z(strCat(string, STRDEF("YYYY")), "XXXXYYYY", "cat string");
-        TEST_RESULT_UINT(string->extra, 60, "check extra");
-        TEST_RESULT_STR_Z(strCatFmt(string, "%05d", 777), "XXXXYYYY00777", "cat formatted string");
-        TEST_RESULT_UINT(string->extra, 55, "check extra");
-        TEST_RESULT_STR_Z(strCatChr(string, '!'), "XXXXYYYY00777!", "cat chr");
-        TEST_RESULT_UINT(string->extra, 54, "check extra");
+        TEST_RESULT_UINT(string->pub.extra, 56, "check extra");
+        TEST_RESULT_STR_Z(strCatZN(string, NULL, 0), "XXXXYYYY", "cat 0");
+        TEST_RESULT_UINT(string->pub.extra, 56, "check extra");
+        TEST_RESULT_STR_Z(strCatBuf(string, BUFSTRDEF("?")), "XXXXYYYY?", "cat buf");
+        TEST_RESULT_UINT(string->pub.extra, 55, "check extra");
+        TEST_RESULT_STR_Z(strCatFmt(string, "%05d", 777), "XXXXYYYY?00777", "cat formatted string");
+        TEST_RESULT_UINT(string->pub.extra, 50, "check extra");
+        TEST_RESULT_STR_Z(strCatChr(string, '!'), "XXXXYYYY?00777!", "cat chr");
+        TEST_RESULT_UINT(string->pub.extra, 49, "check extra");
         TEST_RESULT_STR_Z(
             strCatZN(string, "$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*", 55),
-            "XXXXYYYY00777!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$", "cat chr");
-        TEST_RESULT_UINT(string->extra, 34, "check extra");
+            "XXXXYYYY?00777!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$", "cat chr");
+        TEST_RESULT_UINT(string->pub.extra, 35, "check extra");
+        TEST_RESULT_STR_Z(
+            strCatEncode(string, encodeBase64, BUFSTRDEF("zzzzz")),
+            "XXXXYYYY?00777!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$enp6eno=", "cat encode");
+        TEST_RESULT_UINT(string->pub.extra, 27, "check extra");
+        TEST_RESULT_VOID(strFree(string), "free string");
 
         TEST_RESULT_STR_Z(string2, "ZZZZ", "check unaltered string");
     }
@@ -171,23 +215,23 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("strFirstUpper(), strFirstLower(), strUpper(), strLower()"))
     {
-        TEST_RESULT_STR_Z(strFirstUpper(strNew("")), "", "empty first upper");
-        TEST_RESULT_STR_Z(strFirstUpper(strNew("aaa")), "Aaa", "first upper");
-        TEST_RESULT_STR_Z(strFirstUpper(strNew("Aaa")), "Aaa", "first already upper");
+        TEST_RESULT_STR_Z(strFirstUpper(strNewZ("")), "", "empty first upper");
+        TEST_RESULT_STR_Z(strFirstUpper(strNewZ("aaa")), "Aaa", "first upper");
+        TEST_RESULT_STR_Z(strFirstUpper(strNewZ("Aaa")), "Aaa", "first already upper");
 
-        TEST_RESULT_STR_Z(strFirstLower(strNew("")), "", "empty first lower");
-        TEST_RESULT_STR_Z(strFirstLower(strNew("AAA")), "aAA", "first lower");
-        TEST_RESULT_STR_Z(strFirstLower(strNew("aAA")), "aAA", "first already lower");
+        TEST_RESULT_STR_Z(strFirstLower(strNew()), "", "empty first lower");
+        TEST_RESULT_STR_Z(strFirstLower(strNewZ("AAA")), "aAA", "first lower");
+        TEST_RESULT_STR_Z(strFirstLower(strNewZ("aAA")), "aAA", "first already lower");
 
-        TEST_RESULT_STR_Z(strLower(strNew("K123aBc")), "k123abc", "all lower");
-        TEST_RESULT_STR_Z(strLower(strNew("k123abc")), "k123abc", "already lower");
-        TEST_RESULT_STR_Z(strLower(strNew("C")), "c", "char lower");
-        TEST_RESULT_STR_Z(strLower(strNew("")), "", "empty lower");
+        TEST_RESULT_STR_Z(strLower(strNewZ("K123aBc")), "k123abc", "all lower");
+        TEST_RESULT_STR_Z(strLower(strNewZ("k123abc")), "k123abc", "already lower");
+        TEST_RESULT_STR_Z(strLower(strNewZ("C")), "c", "char lower");
+        TEST_RESULT_STR_Z(strLower(strNew()), "", "empty lower");
 
-        TEST_RESULT_STR_Z(strUpper(strNew("K123aBc")), "K123ABC", "all upper");
-        TEST_RESULT_STR_Z(strUpper(strNew("K123ABC")), "K123ABC", "already upper");
-        TEST_RESULT_STR_Z(strUpper(strNew("c")), "C", "char upper");
-        TEST_RESULT_STR_Z(strUpper(strNew("")), "", "empty upper");
+        TEST_RESULT_STR_Z(strUpper(strNewZ("K123aBc")), "K123ABC", "all upper");
+        TEST_RESULT_STR_Z(strUpper(strNewZ("K123ABC")), "K123ABC", "already upper");
+        TEST_RESULT_STR_Z(strUpper(strNewZ("c")), "C", "char upper");
+        TEST_RESULT_STR_Z(strUpper(strNew()), "", "empty upper");
     }
 
     // *****************************************************************************************************************************
@@ -199,7 +243,7 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("strReplaceChr()"))
     {
-        TEST_RESULT_STR_Z(strReplaceChr(strNew("ABCD"), 'B', 'R'), "ARCD", "replace chr");
+        TEST_RESULT_STR_Z(strReplaceChr(strNewZ("ABCD"), 'B', 'R'), "ARCD", "replace chr");
     }
 
     // *****************************************************************************************************************************
@@ -214,14 +258,14 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("strTrim()"))
     {
-        TEST_RESULT_STR_Z(strTrim(strNew("")), "", "trim empty");
-        TEST_RESULT_STR_Z(strTrim(strNew("X")), "X", "no trim (one char)");
-        TEST_RESULT_STR_Z(strTrim(strNew("no-trim")), "no-trim", "no trim (string)");
-        TEST_RESULT_STR_Z(strTrim(strNew(" \t\r\n")), "", "all whitespace");
-        TEST_RESULT_STR_Z(strTrim(strNew(" \tbegin-only")), "begin-only", "trim begin");
-        TEST_RESULT_STR_Z(strTrim(strNew("end-only\t ")), "end-only", "trim end");
-        TEST_RESULT_STR_Z(strTrim(strNew("\n\rboth\r\n")), "both", "trim both");
-        TEST_RESULT_STR_Z(strTrim(strNew("begin \r\n\tend")), "begin \r\n\tend", "ignore whitespace in middle");
+        TEST_RESULT_STR_Z(strTrim(strNewZ("")), "", "trim empty");
+        TEST_RESULT_STR_Z(strTrim(strNewZ("X")), "X", "no trim (one char)");
+        TEST_RESULT_STR_Z(strTrim(strNewZ("no-trim")), "no-trim", "no trim (string)");
+        TEST_RESULT_STR_Z(strTrim(strNewZ(" \t\r\n")), "", "all whitespace");
+        TEST_RESULT_STR_Z(strTrim(strNewZ(" \tbegin-only")), "begin-only", "trim begin");
+        TEST_RESULT_STR_Z(strTrim(strNewZ("end-only\t ")), "end-only", "trim end");
+        TEST_RESULT_STR_Z(strTrim(strNewZ("\n\rboth\r\n")), "both", "trim both");
+        TEST_RESULT_STR_Z(strTrim(strNewZ("begin \r\n\tend")), "begin \r\n\tend", "ignore whitespace in middle");
     }
 
     // *****************************************************************************************************************************
@@ -232,11 +276,11 @@ testRun(void)
         TEST_RESULT_INT(strChr(STRDEF("abcd"), 'i'), -1, "i not found");
         TEST_RESULT_INT(strChr(STRDEF(""), 'x'), -1, "empty string - x not found");
 
-        String *val = strNew("abcdef");
+        String *val = strCatZ(strNew(), "abcdef");
         TEST_ERROR(
             strTrunc(val, (int)(strSize(val) + 1)), AssertError,
-            "assertion 'idx >= 0 && (size_t)idx <= this->size' failed");
-        TEST_ERROR(strTrunc(val, -1), AssertError, "assertion 'idx >= 0 && (size_t)idx <= this->size' failed");
+            "assertion 'idx >= 0 && (size_t)idx <= strSize(this)' failed");
+        TEST_ERROR(strTrunc(val, -1), AssertError, "assertion 'idx >= 0 && (size_t)idx <= strSize(this)' failed");
 
         TEST_RESULT_STR_Z(strTrunc(val, strChr(val, 'd')), "abc", "simple string truncated");
         strCatZ(val, "\r\n to end");
@@ -295,7 +339,8 @@ testRun(void)
                 else
                 {
                     TEST_RESULT_STR(
-                        strLstAdd(list, strNewFmt("STR%02d", listIdx)), strNewFmt("STR%02d", listIdx), "add item %d", listIdx);
+                        strLstAdd(list, strNewFmt("STR%02d", listIdx)), strNewFmt("STR%02d", listIdx),
+                        strZ(strNewFmt("add item %d", listIdx)));
                 }
             }
 
@@ -314,9 +359,7 @@ testRun(void)
                 TEST_RESULT_STR(strLstGet(list, listIdx), NULL, "check null item");
             }
             else
-            {
-                TEST_RESULT_STR(strLstGet(list, listIdx), strNewFmt("STR%02u", listIdx), "check item %u", listIdx);
-            }
+                TEST_RESULT_STR(strLstGet(list, listIdx), strNewFmt("STR%02u", listIdx), strZ(strNewFmt("check item %u", listIdx)));
         }
 
         TEST_RESULT_VOID(strLstFree(list), "free string list");
@@ -327,14 +370,16 @@ testRun(void)
         TEST_ASSIGN(list, strLstNew(), "new list");
         TEST_RESULT_VOID(strLstAddIfMissing(list, STRDEF("item1")), "add item 1");
         TEST_RESULT_UINT(strLstSize(list), 1, "check size");
-        TEST_RESULT_BOOL(strLstExistsZ(list, "item1"), true, "check exists");
-        TEST_RESULT_BOOL(strLstExistsZ(list, NULL), false, "check null exists");
+        TEST_RESULT_BOOL(strLstExists(list, STRDEF("item1")), true, "check exists");
+        TEST_RESULT_BOOL(strLstExists(list, NULL), false, "check null exists");
         TEST_RESULT_VOID(strLstAddIfMissing(list, STRDEF("item1")), "add item 1 again");
         TEST_RESULT_UINT(strLstSize(list), 1, "check size");
+        TEST_RESULT_BOOL(strLstEmpty(list), false, "    not empty");
 
         TEST_RESULT_BOOL(strLstRemove(list, STRDEF("item1")), true, "remove item 1");
         TEST_RESULT_BOOL(strLstRemove(list, STRDEF("item1")), false, "remove item 1 fails");
         TEST_RESULT_UINT(strLstSize(list), 0, "    check size");
+        TEST_RESULT_BOOL(strLstEmpty(list), true, "    empty");
     }
 
     // *****************************************************************************************************************************
@@ -345,24 +390,6 @@ testRun(void)
         TEST_RESULT_STR_Z(strLstJoin(strLstNewSplit(STRDEF("item1, item2"), STRDEF(", ")), ", "), "item1, item2", "two items");
     }
 
-    // *****************************************************************************************************************************
-    if (testBegin("strLstNewSplitSize()"))
-    {
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewSplitSize(STRDEF(""), STRDEF(" "), 0), ", "), "", "empty list");
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewSplitSizeZ(STRDEF("abc def"), " ", 3), "-"), "abc-def", "two items");
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewSplitSizeZ(STRDEF("abc def"), " ", 4), "-"), "abc-def", "one items");
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewSplitSizeZ(STRDEF("abc def ghi"), " ", 4), "-"), "abc-def-ghi", "three items");
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewSplitSizeZ(STRDEF("abc def ghi"), " ", 8), "-"), "abc def-ghi", "three items");
-        TEST_RESULT_STR_Z(strLstJoin(strLstNewSplitSizeZ(STRDEF("abc def "), " ", 4), "-"), "abc-def ", "two items");
-
-        TEST_RESULT_STR_Z(
-            strLstJoin(strLstNewSplitSize(STRDEF("this is a short sentence"), STRDEF(" "), 10), "\n"),
-            "this is a\n"
-            "short\n"
-            "sentence",
-            "empty list");
-    }
-
     // *****************************************************************************************************************************
     if (testBegin("strLstNewVarLst()"))
     {
@@ -404,7 +431,7 @@ testRun(void)
             }
             else
             {
-                TEST_RESULT_Z_STR(szList[listIdx], strNewFmt("STR%02u", listIdx), "check item %u", listIdx);
+                TEST_RESULT_Z_STR(szList[listIdx], strNewFmt("STR%02u", listIdx), strZ(strNewFmt("check item %u", listIdx)));
             }
         }
 
@@ -414,7 +441,7 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("strLstExists() and strLstExistsZ()"))
+    if (testBegin("strLstExists()"))
     {
         StringList *list = strLstNew();
         strLstAddZ(list, "A");
@@ -422,8 +449,6 @@ testRun(void)
 
         TEST_RESULT_BOOL(strLstExists(list, STRDEF("B")), false, "string does not exist");
         TEST_RESULT_BOOL(strLstExists(list, STRDEF("C")), true, "string exists");
-        TEST_RESULT_BOOL(strLstExistsZ(list, "B"), false, "string does not exist");
-        TEST_RESULT_BOOL(strLstExistsZ(list, "C"), true, "string exists");
     }
 
     // *****************************************************************************************************************************
@@ -508,6 +533,161 @@ testRun(void)
         TEST_RESULT_STR_Z(strLstJoin(strLstSort(list, sortOrderAsc), ", "), "a, b, c", "sort ascending");
     }
 
+    // *****************************************************************************************************************************
+    if (testBegin("STRID*(), strIdTo*(), and strIdFrom*()"))
+    {
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdFromZN()");
+
+        #define TEST_STR5ID1                                        (stringIdBit5 | (uint16_t)('a' - 96) << 4)
+        #define TEST_STR5ID2                                        (TEST_STR5ID1 | (uint16_t)('b' - 96) << 9)
+        #define TEST_STR5ID3                                        ((uint32_t)TEST_STR5ID2 | (uint32_t)('c' - 96) << 14)
+        #define TEST_STR5ID4                                        (TEST_STR5ID3 | (uint32_t)('-' - 18) << 19)
+        #define TEST_STR5ID5                                        (TEST_STR5ID4 | (uint32_t)('z' - 96) << 24)
+        #define TEST_STR5ID6                                        ((uint64_t)TEST_STR5ID5 | (uint64_t)('k' - 96) << 29)
+        #define TEST_STR5ID7                                        (TEST_STR5ID6 | (uint64_t)('z' - 96) << 34)
+        #define TEST_STR5ID8                                        (TEST_STR5ID7 | (uint64_t)('2' - 22) << 39)
+        #define TEST_STR5ID9                                        (TEST_STR5ID8 | (uint64_t)('-' - 18) << 44)
+        #define TEST_STR5ID10                                       (TEST_STR5ID9 | (uint64_t)('y' - 96) << 49)
+        #define TEST_STR5ID11                                       (TEST_STR5ID10 | (uint64_t)('5' - 24) << 54)
+        #define TEST_STR5ID12                                       (TEST_STR5ID11 | (uint64_t)('6' - 24) << 59)
+        #define TEST_STR5ID13                                       (TEST_STR5ID12 | STRING_ID_PREFIX)
+
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit5, "a", 1), TEST_STR5ID1, "5 bits 1 char");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit5, "abc-zk", 6), TEST_STR5ID6, "5 bits 6 chars");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit5, "abc-zkz2-y56", 12), TEST_STR5ID12, "5 bits 12 chars");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit5, "abc-zkz2-y56?", 13), TEST_STR5ID13, "5 bits 13 chars");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit5, "abc-zkz2-y56??", 14), TEST_STR5ID13, "5 bits 14 chars");
+
+        TEST_ERROR(strIdFromZN(stringIdBit5, "AB", 2), FormatError, "'A' is invalid for 5-bit encoding in 'AB'");
+
+        #define TEST_STR6ID1                                        (stringIdBit6 | (uint16_t)('a' - 96) << 4)
+        #define TEST_STR6ID2                                        (TEST_STR6ID1 | (uint16_t)('b' - 96) << 10)
+        #define TEST_STR6ID3                                        ((uint32_t)TEST_STR6ID2 | (uint32_t)('C' - 27) << 16)
+        #define TEST_STR6ID4                                        (TEST_STR6ID3 | (uint32_t)('-' - 18) << 22)
+        #define TEST_STR6ID5                                        ((uint64_t)TEST_STR6ID4 | (uint64_t)('4' - 20) << 28)
+        #define TEST_STR6ID6                                        (TEST_STR6ID5 | (uint64_t)('0' - 20) << 34)
+        #define TEST_STR6ID7                                        (TEST_STR6ID6 | (uint64_t)('M' - 27) << 40)
+        #define TEST_STR6ID8                                        (TEST_STR6ID7 | (uint64_t)('z' - 96) << 46)
+        #define TEST_STR6ID9                                        (TEST_STR6ID8 | (uint64_t)('Z' - 27) << 52)
+        #define TEST_STR6ID10                                       (TEST_STR6ID9 | (uint64_t)('9' - 20) << 58)
+        #define TEST_STR6ID11                                       (TEST_STR6ID10 | STRING_ID_PREFIX)
+
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit6, "a", 1), TEST_STR6ID1, "6 bits 1 char");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit6, "abC-4", 5), TEST_STR6ID5, "6 bits 5 chars");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit6, "abC-40MzZ9", 10), TEST_STR6ID10, "6 bits 10 chars");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit6, "abC-40MzZ9?", 11), TEST_STR6ID11, "6 bits 11 chars");
+        TEST_RESULT_UINT(strIdFromZN(stringIdBit6, "abC-40MzZ9??", 12), TEST_STR6ID11, "6 bits 12 chars");
+
+        TEST_ERROR(strIdFromZN(stringIdBit6, "|B", 2), FormatError, "'|' is invalid for 6-bit encoding in '|B'");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("STRID*()");
+
+        TEST_RESULT_UINT(STRID5("a", TEST_STR5ID1), TEST_STR5ID1, "STRID5()");
+        TEST_RESULT_UINT(STRID6("abC-4", TEST_STR6ID5), TEST_STR6ID5, "STRID6()");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdFromStr()");
+
+        TEST_RESULT_UINT(strIdFromStr(stringIdBit5, STRDEF("abc-")), TEST_STR5ID4, "5 bits 4 chars");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdFromZ()");
+
+        TEST_RESULT_UINT(strIdFromZ(stringIdBit6, "abC-"), TEST_STR6ID4, "6 bits 4 chars");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdToZN()");
+
+        char buffer5[] = "XXXXXXXXXXXXX";
+
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID1, buffer5), 1, "5 bits 1 char");
+        TEST_RESULT_Z(buffer5, "aXXXXXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID2, buffer5), 2, "5 bits 2 chars");
+        TEST_RESULT_Z(buffer5, "abXXXXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID3, buffer5), 3, "5 bits 3 chars");
+        TEST_RESULT_Z(buffer5, "abcXXXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID4, buffer5), 4, "5 bits 4 chars");
+        TEST_RESULT_Z(buffer5, "abc-XXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID5, buffer5), 5, "5 bits 5 chars");
+        TEST_RESULT_Z(buffer5, "abc-zXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID6, buffer5), 6, "5 bits 6 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID7, buffer5), 7, "5 bits 7 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkzXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID8, buffer5), 8, "5 bits 8 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkz2XXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID9, buffer5), 9, "5 bits 9 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkz2-XXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID10, buffer5), 10, "5 bits 10 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkz2-yXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID11, buffer5), 11, "5 bits 11 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkz2-y5XX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID12, buffer5), 12, "5 bits 12 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkz2-y56X", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR5ID13, buffer5), 13, "5 bits 13 chars");
+        TEST_RESULT_Z(buffer5, "abc-zkz2-y56+", "    check");
+
+        char buffer6[] = "XXXXXXXXXXX";
+
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID1, buffer6), 1, "6 bits 1 char");
+        TEST_RESULT_Z(buffer6, "aXXXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID2, buffer6), 2, "6 bits 2 chars");
+        TEST_RESULT_Z(buffer6, "abXXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID3, buffer6), 3, "6 bits 3 chars");
+        TEST_RESULT_Z(buffer6, "abCXXXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID4, buffer6), 4, "6 bits 4 chars");
+        TEST_RESULT_Z(buffer6, "abC-XXXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID5, buffer6), 5, "6 bits 5 chars");
+        TEST_RESULT_Z(buffer6, "abC-4XXXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID6, buffer6), 6, "6 bits 6 chars");
+        TEST_RESULT_Z(buffer6, "abC-40XXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID7, buffer6), 7, "6 bits 7 chars");
+        TEST_RESULT_Z(buffer6, "abC-40MXXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID8, buffer6), 8, "6 bits 8 chars");
+        TEST_RESULT_Z(buffer6, "abC-40MzXXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID9, buffer6), 9, "6 bits 9 chars");
+        TEST_RESULT_Z(buffer6, "abC-40MzZXX", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID10, buffer6), 10, "6 bits 10 chars");
+        TEST_RESULT_Z(buffer6, "abC-40MzZ9X", "    check");
+        TEST_RESULT_UINT(strIdToZN(TEST_STR6ID11, buffer6), 11, "6 bits 11 chars");
+        TEST_RESULT_Z(buffer6, "abC-40MzZ9+", "    check");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdToStr()");
+
+        TEST_RESULT_STR_Z(strIdToStr(TEST_STR5ID1), "a", "5 bits 1 char");
+        TEST_RESULT_STR_Z(strIdToStr(TEST_STR5ID8), "abc-zkz2", "5 bits 8 chars");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdToStr()");
+
+        char buffer[STRID_MAX + 1];
+
+        TEST_RESULT_UINT(strIdToZ(TEST_STR5ID1, buffer), 1, "5 bits 1 char");
+        TEST_RESULT_Z(buffer, "a", "    check");
+        TEST_RESULT_UINT(strIdToZ(TEST_STR5ID4, buffer), 4, "4 chars");
+        TEST_RESULT_Z(buffer, "abc-", "    check");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("64-bit enum");
+
+        TEST_RESULT_STR_Z(strIdToStr(testStringIdEnumFunc(testStringIdEnumAes256Cbc)), "aes-256-cbc", "pass to enum param");
+
+        TestStringIdEnum testEnum = testStringIdEnumRemote;
+        TEST_RESULT_STR_Z(strIdToStr(testEnum), "remote", "assign to enum");
+
+        TEST_RESULT_STR_Z(strIdToStr(testStringIdEnumTest), "test", "pass to StringId param");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("strIdToLog()");
+
+        TEST_RESULT_UINT(strIdToLog(TEST_STR5ID2, buffer, sizeof(buffer)), 2, "string id with limited buffer");
+        TEST_RESULT_UINT(strlen(buffer), 2, "    check length");
+        TEST_RESULT_Z(buffer, "ab", "    check buffer");
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("strLstToLog()"))
     {
@@ -515,13 +695,13 @@ testRun(void)
 
         TEST_RESULT_STR_Z(strLstToLog(list), "{[]}", "format empty list");
 
-        strLstInsertZ(list, 0, "item3");
+        strLstInsert(list, 0, STRDEF("item3"));
         TEST_RESULT_STR_Z(strLstToLog(list), "{[\"item3\"]}", "format 1 item list");
 
         strLstInsert(list, 0, STRDEF("item1"));
-        strLstInsertZ(list, 1, "item2");
+        strLstInsert(list, 1, STRDEF("item2"));
         TEST_RESULT_STR_Z(strLstToLog(list), "{[\"item1\", \"item2\", \"item3\"]}", "format 3 item list");
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
