@@ -1,15 +1,15 @@
 /***********************************************************************************************************************************
 Test Database
 ***********************************************************************************************************************************/
+#include "common/io/fdRead.h"
+#include "common/io/fdWrite.h"
+#include "common/type/json.h"
+
 #include "common/harnessConfig.h"
 #include "common/harnessFork.h"
 #include "common/harnessLog.h"
 #include "common/harnessPq.h"
 
-#include "common/io/fdRead.h"
-#include "common/io/fdWrite.h"
-#include "common/type/json.h"
-
 /***********************************************************************************************************************************
 Macro to check that replay is making progress -- this does not seem useful enough to be included in the pq harness header
 ***********************************************************************************************************************************/
@@ -48,7 +48,7 @@ Macro to check that replay is making progress -- this does not seem useful enoug
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
@@ -59,23 +59,18 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("Db and dbProtocol()"))
     {
-        HARNESS_FORK_BEGIN()
+        HRN_FORK_BEGIN()
         {
-            HARNESS_FORK_CHILD_BEGIN(0, true)
+            HRN_FORK_CHILD_BEGIN()
             {
-                IoRead *read = ioFdReadNew(strNew("client read"), HARNESS_FORK_CHILD_READ(), 2000);
-                ioReadOpen(read);
-                IoWrite *write = ioFdWriteNew(strNew("client write"), HARNESS_FORK_CHILD_WRITE(), 2000);
-                ioWriteOpen(write);
-
                 // Set options
                 StringList *argList = strLstNew();
-                strLstAddZ(argList, "--stanza=test1");
-                strLstAddZ(argList, "--pg1-path=/path/to/pg");
-                strLstAddZ(argList, "--pg1-database=testdb");
-                strLstAddZ(argList, "--" CFGOPT_REMOTE_TYPE "=" PROTOCOL_REMOTE_TYPE_PG);
-                strLstAddZ(argList, "--process=0");
-                harnessCfgLoadRole(cfgCmdBackup, cfgCmdRoleRemote, argList);
+                hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+                hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, TEST_PATH "/pg");
+                hrnCfgArgKeyRawZ(argList, cfgOptPgDatabase, 1,  "testdb");
+                hrnCfgArgRawStrId(argList, cfgOptRemoteType, protocolStorageTypePg);
+                hrnCfgArgRawZ(argList, cfgOptProcess, "0");
+                HRN_CFG_LOAD(cfgCmdBackup, argList, .role = cfgCmdRoleRemote);
 
                 // Set script
                 harnessPqScriptSet((HarnessPq [])
@@ -83,13 +78,13 @@ testRun(void)
                     HRNPQ_MACRO_OPEN(1, "dbname='testdb' port=5432"),
                     HRNPQ_MACRO_SET_SEARCH_PATH(1),
                     HRNPQ_MACRO_SET_CLIENT_ENCODING(1),
-                    HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_84, "/pgdata", NULL, NULL),
+                    HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_84, TEST_PATH "/pg", NULL, NULL),
                     HRNPQ_MACRO_CLOSE(1),
 
                     HRNPQ_MACRO_OPEN(1, "dbname='testdb' port=5432"),
                     HRNPQ_MACRO_SET_SEARCH_PATH(1),
                     HRNPQ_MACRO_SET_CLIENT_ENCODING(1),
-                    HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_84, "/pgdata", NULL, NULL),
+                    HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_84, TEST_PATH "/pg", NULL, NULL),
                     HRNPQ_MACRO_WAL_SWITCH(1, "xlog", "000000030000000200000003"),
                     HRNPQ_MACRO_CLOSE(1),
 
@@ -99,53 +94,92 @@ testRun(void)
                 // Create server
                 ProtocolServer *server = NULL;
 
-                TEST_ASSIGN(server, protocolServerNew(strNew("db test server"), strNew("test"), read, write), "create server");
-                TEST_RESULT_VOID(protocolServerHandlerAdd(server, dbProtocol), "add handler");
-                TEST_RESULT_VOID(protocolServerProcess(server, NULL), "run process loop");
+                TEST_ASSIGN(
+                    server,
+                    protocolServerNew(STRDEF("db test server"), STRDEF("test"), HRN_FORK_CHILD_READ(), HRN_FORK_CHILD_WRITE()),
+                    "create server");
+
+                static const ProtocolServerHandler commandHandler[] = {PROTOCOL_SERVER_HANDLER_DB_LIST};
+
+                TEST_RESULT_VOID(
+                    protocolServerProcess(server, NULL, commandHandler, PROTOCOL_SERVER_HANDLER_LIST_SIZE(commandHandler)),
+                    "run process loop");
                 TEST_RESULT_VOID(protocolServerFree(server), "free server");
             }
-            HARNESS_FORK_CHILD_END();
+            HRN_FORK_CHILD_END();
 
-            HARNESS_FORK_PARENT_BEGIN()
+            HRN_FORK_PARENT_BEGIN()
             {
-                IoRead *read = ioFdReadNew(strNew("server read"), HARNESS_FORK_PARENT_READ_PROCESS(0), 2000);
-                ioReadOpen(read);
-                IoWrite *write = ioFdWriteNew(strNew("server write"), HARNESS_FORK_PARENT_WRITE_PROCESS(0), 2000);
-                ioWriteOpen(write);
-
                 // Create client
                 ProtocolClient *client = NULL;
                 Db *db = NULL;
 
-                TEST_ASSIGN(client, protocolClientNew(strNew("db test client"), strNew("test"), read, write), "create client");
+                TEST_ASSIGN(
+                    client,
+                    protocolClientNew(STRDEF("db test client"), STRDEF("test"), HRN_FORK_PARENT_READ(0), HRN_FORK_PARENT_WRITE(0)),
+                    "create client");
 
-                // Open and free database
-                TEST_ASSIGN(db, dbNew(NULL, client, strNew("test")), "create db");
-                TEST_RESULT_VOID(dbOpen(db), "open db");
-                TEST_RESULT_VOID(dbFree(db), "free db");
+                TRY_BEGIN()
+                {
+                    // -------------------------------------------------------------------------------------------------------------
+                    TEST_TITLE("open and free database");
 
-                // Open the database, but don't free it so the server is forced to do it on shutdown
-                TEST_ASSIGN(db, dbNew(NULL, client, strNew("test")), "create db");
-                TEST_RESULT_VOID(dbOpen(db), "open db");
-                TEST_RESULT_STR_Z(dbWalSwitch(db), "000000030000000200000003", "    wal switch");
-                TEST_RESULT_VOID(memContextCallbackClear(db->memContext), "clear context so close is not called");
+                    TEST_ASSIGN(db, dbNew(NULL, client, STRDEF("test")), "create db");
 
-                TEST_RESULT_VOID(protocolClientFree(client), "free client");
+                    TRY_BEGIN()
+                    {
+                        TEST_RESULT_VOID(dbOpen(db), "open db");
+                        TEST_RESULT_UINT(db->remoteIdx, 0, "check remote idx");
+                        TEST_RESULT_VOID(dbFree(db), "free db");
+                        db = NULL;
+                    }
+                    CATCH_ANY()
+                    {
+                        // Free on error
+                        dbFree(db);
+                    }
+                    TRY_END();
+
+                    // -------------------------------------------------------------------------------------------------------------
+                    TEST_TITLE("remote commands");
+
+                    TEST_ASSIGN(db, dbNew(NULL, client, STRDEF("test")), "create db");
+
+                    TRY_BEGIN()
+                    {
+                        TEST_RESULT_VOID(dbOpen(db), "open db");
+                        TEST_RESULT_UINT(db->remoteIdx, 1, "check idx");
+                        TEST_RESULT_STR_Z(dbWalSwitch(db), "000000030000000200000003", "wal switch");
+                        TEST_RESULT_VOID(memContextCallbackClear(db->pub.memContext), "clear context so close is not called");
+                    }
+                    FINALLY()
+                    {
+                        // Clear the context callback so the server frees the db on exit
+                        memContextCallbackClear(db->pub.memContext);
+                    }
+                    TRY_END();
+                }
+                FINALLY()
+                {
+                    // Free on error
+                    protocolClientFree(client);
+                }
+                TRY_END();
             }
-            HARNESS_FORK_PARENT_END();
+            HRN_FORK_PARENT_END();
         }
-        HARNESS_FORK_END();
+        HRN_FORK_END();
     }
 
     // *****************************************************************************************************************************
     if (testBegin("dbBackupStart(), dbBackupStop(), dbTime(), dbList(), dbTablespaceList(), and dbReplayWait()"))
     {
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "--stanza=test1");
-        strLstAddZ(argList, "--repo1-retention-full=1");
-        strLstAddZ(argList, "--pg1-path=/pg1");
-        strLstAddZ(argList, "--pg1-database=backupdb");
-        harnessCfgLoad(cfgCmdBackup, argList);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionFull, 1, "1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, TEST_PATH "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgDatabase, 1,  "backupdb");
+        HRN_CFG_LOAD(cfgCmdBackup, argList);
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("error when unable to select any pg_settings");
@@ -201,7 +235,7 @@ testRun(void)
         harnessPqScriptSet((HarnessPq [])
         {
             // Connect to primary
-            HRNPQ_MACRO_OPEN_LE_91(1, "dbname='backupdb' port=5432", PG_VERSION_83, "/pg1", NULL, NULL),
+            HRNPQ_MACRO_OPEN_LE_91(1, "dbname='backupdb' port=5432", PG_VERSION_83, TEST_PATH "/pg1", NULL, NULL),
 
             // Get advisory lock
             HRNPQ_MACRO_ADVISORY_LOCK(1, true),
@@ -228,7 +262,7 @@ testRun(void)
         harnessPqScriptSet((HarnessPq [])
         {
             // Connect to primary
-            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='backupdb' port=5432", PG_VERSION_95, "/pg1", false, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='backupdb' port=5432", PG_VERSION_95, TEST_PATH "/pg1", false, NULL, NULL),
 
             // Get start time
             HRNPQ_MACRO_TIME_QUERY(1, 1000),
@@ -284,7 +318,7 @@ testRun(void)
         harnessPqScriptSet((HarnessPq [])
         {
             // Connect to primary
-            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='backupdb' port=5432", PG_VERSION_95, "/pg1", false, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='backupdb' port=5432", PG_VERSION_95, TEST_PATH "/pg1", false, NULL, NULL),
 
             // Start backup when backup is in progress
             HRNPQ_MACRO_ADVISORY_LOCK(1, true),
@@ -323,7 +357,7 @@ testRun(void)
         harnessPqScriptSet((HarnessPq [])
         {
             // Connect to primary
-            HRNPQ_MACRO_OPEN_GE_96(1, "dbname='backupdb' port=5432", PG_VERSION_96, "/pg1", false, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_96(1, "dbname='backupdb' port=5432", PG_VERSION_96, TEST_PATH "/pg1", false, NULL, NULL),
 
             // Start backup
             HRNPQ_MACRO_ADVISORY_LOCK(1, true),
@@ -356,20 +390,20 @@ testRun(void)
         TEST_TITLE("PostgreSQL 9.5 start backup from standby");
 
         argList = strLstNew();
-        strLstAddZ(argList, "--stanza=test1");
-        strLstAddZ(argList, "--repo1-retention-full=1");
-        strLstAddZ(argList, "--pg1-path=/pg1");
-        strLstAddZ(argList, "--pg2-path=/pg2");
-        strLstAddZ(argList, "--pg2-port=5433");
-        harnessCfgLoad(cfgCmdBackup, argList);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionFull, 1, "1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, TEST_PATH "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 2, TEST_PATH "/pg2");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPort, 2, "5433");
+        HRN_CFG_LOAD(cfgCmdBackup, argList);
 
         harnessPqScriptSet((HarnessPq [])
         {
             // Connect to primary
-            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_95, "/pg1", false, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_95, TEST_PATH "/pg1", false, NULL, NULL),
 
             // Connect to standby
-            HRNPQ_MACRO_OPEN_GE_92(2, "dbname='postgres' port=5433", PG_VERSION_95, "/pg2", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(2, "dbname='postgres' port=5433", PG_VERSION_95, TEST_PATH "/pg2", true, NULL, NULL),
 
             // Start backup
             HRNPQ_MACRO_ADVISORY_LOCK(1, true),
@@ -401,10 +435,10 @@ testRun(void)
         harnessPqScriptSet((HarnessPq [])
         {
             // Connect to primary
-            HRNPQ_MACRO_OPEN_GE_96(1, "dbname='postgres' port=5432", PG_VERSION_10, "/pg1", false, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_96(1, "dbname='postgres' port=5432", PG_VERSION_10, TEST_PATH "/pg1", false, NULL, NULL),
 
             // Connect to standby
-            HRNPQ_MACRO_OPEN_GE_96(2, "dbname='postgres' port=5433", PG_VERSION_10, "/pg2", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_96(2, "dbname='postgres' port=5433", PG_VERSION_10, TEST_PATH "/pg2", true, NULL, NULL),
 
             // Start backup
             HRNPQ_MACRO_ADVISORY_LOCK(1, true),
@@ -437,10 +471,11 @@ testRun(void)
             HRNPQ_MACRO_REPLAY_TARGET_REACHED_PROGRESS_GE_10(2, "5/5", false, "5/3", "5/3", false, 250),
             HRNPQ_MACRO_REPLAY_TARGET_REACHED_PROGRESS_GE_10(2, "5/5", false, "5/3", "5/3", false, 0),
 
-            // Checkpoint target not reached
+            // Checkpoint target timeout waiting for sync
             HRNPQ_MACRO_REPLAY_TARGET_REACHED_GE_10(2, "5/5", true, "5/5"),
             HRNPQ_MACRO_CHECKPOINT(2),
-            HRNPQ_MACRO_CHECKPOINT_TARGET_REACHED_GE_10(2, "5/5", false, "5/4"),
+            HRNPQ_MACRO_CHECKPOINT_TARGET_REACHED_GE_10(2, "5/5", false, "5/4", 250),
+            HRNPQ_MACRO_CHECKPOINT_TARGET_REACHED_GE_10(2, "5/5", false, "5/4", 0),
 
             // Wait for standby to sync
             HRNPQ_MACRO_REPLAY_TARGET_REACHED_GE_10(2, "5/5", false, "5/3"),
@@ -448,7 +483,7 @@ testRun(void)
             HRNPQ_MACRO_REPLAY_TARGET_REACHED_PROGRESS_GE_10(2, "5/5", false, "5/4", "5/3", true, 0),
             HRNPQ_MACRO_REPLAY_TARGET_REACHED_PROGRESS_GE_10(2, "5/5", true, "5/5", "5/4", true, 0),
             HRNPQ_MACRO_CHECKPOINT(2),
-            HRNPQ_MACRO_CHECKPOINT_TARGET_REACHED_GE_10(2, "5/5", true, "X/X"),
+            HRNPQ_MACRO_CHECKPOINT_TARGET_REACHED_GE_10(2, "5/5", true, "X/X", 0),
 
             // Close standby
             HRNPQ_MACRO_CLOSE(2),
@@ -473,11 +508,13 @@ testRun(void)
 
         TEST_ERROR(
             dbReplayWait(db.standby, STRDEF("5/5"), 200), ArchiveTimeoutError,
-            "timeout before standby replayed to 5/5 - only reached 5/3");
+            "timeout before standby replayed to 5/5 - only reached 5/3\n"
+            "HINT: is replication running and current on the standby?\n"
+            "HINT: disable the 'backup-standby' option to backup directly from the primary.");
 
         TEST_ERROR(
-            dbReplayWait(db.standby, STRDEF("5/5"), 1000), ArchiveTimeoutError,
-            "the checkpoint lsn 5/4 is less than the target lsn 5/5 even though the replay lsn is 5/5");
+            dbReplayWait(db.standby, STRDEF("5/5"), 200), ArchiveTimeoutError,
+            "timeout before standby checkpoint lsn reached 5/5 - only reached 5/4");
 
         TEST_RESULT_VOID(dbReplayWait(db.standby, STRDEF("5/5"), 1000), "sync standby");
 
@@ -493,14 +530,15 @@ testRun(void)
     {
         DbGetResult result = {0};
 
-        // Error connecting to primary
-        // -------------------------------------------------------------------------------------------------------------------------
         StringList *argList = strLstNew();
-        strLstAddZ(argList, "--stanza=test1");
-        strLstAddZ(argList, "--repo1-retention-full=1");
-        strLstAddZ(argList, "--pg1-path=/path/to/pg");
-        strLstAddZ(argList, "--pg1-user=bob");
-        harnessCfgLoad(cfgCmdBackup, argList);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionFull, 1, "1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, TEST_PATH "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgUser, 1, "bob");
+        HRN_CFG_LOAD(cfgCmdBackup, argList);
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("error connecting to primary");
 
         harnessPqScriptSet((HarnessPq [])
         {
@@ -512,18 +550,19 @@ testRun(void)
         });
 
         TEST_ERROR(dbGet(true, true, false), DbConnectError, "unable to find primary cluster - cannot proceed");
-        harnessLogResult(
+        TEST_RESULT_LOG(
             "P00   WARN: unable to check pg-1: [DbConnectError] unable to connect to 'dbname='postgres' port=5432 user='bob'':"
                 " error");
 
-        // Only cluster is a standby
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("only available cluster is a standby");
+
         harnessPqScriptSet((HarnessPq [])
         {
             HRNPQ_MACRO_OPEN(1, "dbname='postgres' port=5432 user='bob'"),
             HRNPQ_MACRO_SET_SEARCH_PATH(1),
             HRNPQ_MACRO_SET_CLIENT_ENCODING(1),
-            HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_94, "/pgdata", NULL, NULL),
+            HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_94, TEST_PATH "/pg", NULL, NULL),
             HRNPQ_MACRO_SET_APPLICATION_NAME(1),
             HRNPQ_MACRO_IS_STANDBY_QUERY(1, true),
             HRNPQ_MACRO_CLOSE(1),
@@ -540,7 +579,7 @@ testRun(void)
             HRNPQ_MACRO_OPEN(1, "dbname='postgres' port=5432 user='bob'"),
             HRNPQ_MACRO_SET_SEARCH_PATH(1),
             HRNPQ_MACRO_SET_CLIENT_ENCODING(1),
-            HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_94, "/pgdata", NULL, NULL),
+            HRNPQ_MACRO_VALIDATE_QUERY(1, PG_VERSION_94, TEST_PATH "/pg", NULL, NULL),
             HRNPQ_MACRO_SET_APPLICATION_NAME(1),
             HRNPQ_MACRO_IS_STANDBY_QUERY(1, false),
             HRNPQ_MACRO_CLOSE(1),
@@ -549,40 +588,42 @@ testRun(void)
 
         TEST_ERROR(dbGet(false, false, true), DbConnectError, "unable to find standby cluster - cannot proceed");
 
-        // Primary cluster found
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("primary cluster found");
+
         harnessPqScriptSet((HarnessPq [])
         {
-            HRNPQ_MACRO_OPEN_LE_91(1, "dbname='postgres' port=5432 user='bob'", PG_VERSION_84, "/pgdata", NULL, NULL),
+            HRNPQ_MACRO_OPEN_LE_91(1, "dbname='postgres' port=5432 user='bob'", PG_VERSION_84, TEST_PATH "/pg1", NULL, NULL),
             HRNPQ_MACRO_CLOSE(1),
             HRNPQ_MACRO_DONE()
         });
 
         TEST_ASSIGN(result, dbGet(true, true, false), "get primary only");
 
-        TEST_RESULT_INT(result.primaryIdx, 0, "    check primary id");
-        TEST_RESULT_BOOL(result.primary != NULL, true, "    check primary");
-        TEST_RESULT_INT(result.standbyIdx, 0, "    check standby id");
-        TEST_RESULT_BOOL(result.standby == NULL, true, "    check standby");
-        TEST_RESULT_INT(dbPgVersion(result.primary), PG_VERSION_84, "    version set");
-        TEST_RESULT_STR_Z(dbPgDataPath(result.primary), "/pgdata", "    path set");
+        TEST_RESULT_INT(result.primaryIdx, 0, "check primary id");
+        TEST_RESULT_BOOL(result.primary != NULL, true, "check primary");
+        TEST_RESULT_INT(result.standbyIdx, 0, "check standby id");
+        TEST_RESULT_BOOL(result.standby == NULL, true, "check standby");
+        TEST_RESULT_INT(dbPgVersion(result.primary), PG_VERSION_84, "version set");
+        TEST_RESULT_STR_Z(dbPgDataPath(result.primary), TEST_PATH "/pg1", "path set");
 
         TEST_RESULT_VOID(dbFree(result.primary), "free primary");
 
-        // More than one primary found
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("more than one primary found");
+
         argList = strLstNew();
-        strLstAddZ(argList, "--stanza=test1");
-        strLstAddZ(argList, "--repo1-retention-full=1");
-        strLstAddZ(argList, "--pg1-path=/path/to/pg1");
-        strLstAddZ(argList, "--pg8-path=/path/to/pg2");
-        strLstAddZ(argList, "--pg8-port=5433");
-        harnessCfgLoad(cfgCmdBackup, argList);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionFull, 1, "1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, TEST_PATH "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 8, TEST_PATH "/pg8");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPort, 8, "5433");
+        HRN_CFG_LOAD(cfgCmdBackup, argList);
 
         harnessPqScriptSet((HarnessPq [])
         {
-            HRNPQ_MACRO_OPEN_LE_91(1, "dbname='postgres' port=5432", PG_VERSION_84, "/pgdata", NULL, NULL),
-            HRNPQ_MACRO_OPEN_LE_91(8, "dbname='postgres' port=5433", PG_VERSION_84, "/pgdata", NULL, NULL),
+            HRNPQ_MACRO_OPEN_LE_91(1, "dbname='postgres' port=5432", PG_VERSION_84, TEST_PATH "/pg1", NULL, NULL),
+            HRNPQ_MACRO_OPEN_LE_91(8, "dbname='postgres' port=5433", PG_VERSION_84, TEST_PATH "/pg8", NULL, NULL),
 
             HRNPQ_MACRO_CLOSE(1),
             HRNPQ_MACRO_CLOSE(8),
@@ -592,12 +633,13 @@ testRun(void)
 
         TEST_ERROR(dbGet(true, true, false), DbConnectError, "more than one primary cluster found");
 
-        // Two standbys found but no primary
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("two standbys found but no primary");
+
         harnessPqScriptSet((HarnessPq [])
         {
-            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_92, "/pgdata", true, NULL, NULL),
-            HRNPQ_MACRO_OPEN_GE_92(8, "dbname='postgres' port=5433", PG_VERSION_92, "/pgdata", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_92, TEST_PATH "/pg1", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(8, "dbname='postgres' port=5433", PG_VERSION_92, TEST_PATH "/pg8", true, NULL, NULL),
 
             HRNPQ_MACRO_CLOSE(8),
             HRNPQ_MACRO_CLOSE(1),
@@ -607,12 +649,13 @@ testRun(void)
 
         TEST_ERROR(dbGet(false, true, false), DbConnectError, "unable to find primary cluster - cannot proceed");
 
-        // Two standbys and primary not required
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("two standbys and primary not required");
+
         harnessPqScriptSet((HarnessPq [])
         {
-            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_92, "/pgdata", true, NULL, NULL),
-            HRNPQ_MACRO_OPEN_GE_92(8, "dbname='postgres' port=5433", PG_VERSION_92, "/pgdata", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_92, TEST_PATH "/pg1", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(8, "dbname='postgres' port=5433", PG_VERSION_92, TEST_PATH "/pg8", true, NULL, NULL),
 
             HRNPQ_MACRO_CLOSE(8),
             HRNPQ_MACRO_CLOSE(1),
@@ -622,31 +665,32 @@ testRun(void)
 
         TEST_ASSIGN(result, dbGet(false, false, false), "get standbys");
 
-        TEST_RESULT_INT(result.primaryIdx, 0, "    check primary id");
-        TEST_RESULT_BOOL(result.primary == NULL, true, "    check primary");
-        TEST_RESULT_INT(result.standbyIdx, 0, "    check standby id");
-        TEST_RESULT_BOOL(result.standby != NULL, true, "    check standby");
+        TEST_RESULT_INT(result.primaryIdx, 0, "check primary id");
+        TEST_RESULT_BOOL(result.primary == NULL, true, "check primary");
+        TEST_RESULT_INT(result.standbyIdx, 0, "check standby id");
+        TEST_RESULT_BOOL(result.standby != NULL, true, "check standby");
 
         TEST_RESULT_VOID(dbFree(result.standby), "free standby");
 
-        // Primary and standby found
         // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("primary and standby found");
+
         argList = strLstNew();
-        strLstAddZ(argList, "--stanza=test1");
-        strLstAddZ(argList, "--repo1-retention-full=1");
-        strLstAddZ(argList, "--pg1-path=/path/to/pg1");
-        strLstAddZ(argList, "--pg4-path=/path/to/pg4");
-        strLstAddZ(argList, "--pg4-port=5433");
-        strLstAddZ(argList, "--pg5-host=localhost");
-        strLstAdd(argList, strNewFmt("--pg5-host-user=%s", testUser()));
-        strLstAddZ(argList, "--pg5-path=/path/to/pg5");
-        strLstAddZ(argList, "--pg8-path=/path/to/pg8");
-        strLstAddZ(argList, "--pg8-port=5434");
-        harnessCfgLoad(cfgCmdBackup, argList);
+        hrnCfgArgRawZ(argList, cfgOptStanza, "test1");
+        hrnCfgArgKeyRawZ(argList, cfgOptRepoRetentionFull, 1, "1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 1, TEST_PATH "/pg1");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 4, TEST_PATH "/pg4");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPort, 4, "5433");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgHost, 5, "localhost");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgHostUser, 5, TEST_USER);
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 5, TEST_PATH "/pg5");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPath, 8, TEST_PATH "/pg8");
+        hrnCfgArgKeyRawZ(argList, cfgOptPgPort, 8, "5434");
+        HRN_CFG_LOAD(cfgCmdBackup, argList);
 
         harnessPqScriptSet((HarnessPq [])
         {
-            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_92, "/pgdata", true, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(1, "dbname='postgres' port=5432", PG_VERSION_92, TEST_PATH "/pg1", true, NULL, NULL),
 
             // pg-4 error
             {.session = 4, .function = HRNPQ_CONNECTDB, .param = "[\"dbname='postgres' port=5433\"]"},
@@ -654,7 +698,7 @@ testRun(void)
             {.session = 4, .function = HRNPQ_ERRORMESSAGE, .resultZ = "error"},
             {.session = 4, .function = HRNPQ_FINISH},
 
-            HRNPQ_MACRO_OPEN_GE_92(8, "dbname='postgres' port=5434", PG_VERSION_92, "/pgdata", false, NULL, NULL),
+            HRNPQ_MACRO_OPEN_GE_92(8, "dbname='postgres' port=5434", PG_VERSION_92, TEST_PATH "/pg8", false, NULL, NULL),
 
             HRNPQ_MACRO_CREATE_RESTORE_POINT(8, "2/3"),
             HRNPQ_MACRO_WAL_SWITCH(8, "xlog", "000000010000000200000003"),
@@ -667,23 +711,23 @@ testRun(void)
 
         TEST_ASSIGN(result, dbGet(false, true, false), "get primary and standy");
 
-        hrnLogReplaceAdd("No such file or directory.*$", NULL, "NO SUCH FILE OR DIRECTORY", false);
+        hrnLogReplaceAdd("(could not connect to server|connection to server on socket).*$", NULL, "PG ERROR", false);
         TEST_RESULT_LOG(
             "P00   WARN: unable to check pg-4: [DbConnectError] unable to connect to 'dbname='postgres' port=5433': error\n"
-            "P00   WARN: unable to check pg-5: [DbConnectError] raised from remote-0 protocol on 'localhost':"
-                " unable to connect to 'dbname='postgres' port=5432': could not connect to server: [NO SUCH FILE OR DIRECTORY]");
+            "P00   WARN: unable to check pg-5: [DbConnectError] raised from remote-0 ssh protocol on 'localhost':"
+                " unable to connect to 'dbname='postgres' port=5432': [PG ERROR]");
 
-        TEST_RESULT_INT(result.primaryIdx, 3, "    check primary idx");
-        TEST_RESULT_BOOL(result.primary != NULL, true, "    check primary");
-        TEST_RESULT_STR_Z(dbArchiveMode(result.primary), "on", "    dbArchiveMode");
-        TEST_RESULT_STR_Z(dbArchiveCommand(result.primary), PROJECT_BIN, "    dbArchiveCommand");
-        TEST_RESULT_STR_Z(dbWalSwitch(result.primary), "000000010000000200000003", "    wal switch");
-        TEST_RESULT_INT(result.standbyIdx, 0, "    check standby id");
-        TEST_RESULT_BOOL(result.standby != NULL, true, "    check standby");
+        TEST_RESULT_INT(result.primaryIdx, 3, "check primary idx");
+        TEST_RESULT_BOOL(result.primary != NULL, true, "check primary");
+        TEST_RESULT_STR_Z(dbArchiveMode(result.primary), "on", "dbArchiveMode");
+        TEST_RESULT_STR_Z(dbArchiveCommand(result.primary), PROJECT_BIN, "dbArchiveCommand");
+        TEST_RESULT_STR_Z(dbWalSwitch(result.primary), "000000010000000200000003", "wal switch");
+        TEST_RESULT_INT(result.standbyIdx, 0, "check standby id");
+        TEST_RESULT_BOOL(result.standby != NULL, true, "check standby");
 
         TEST_RESULT_VOID(dbFree(result.primary), "free primary");
         TEST_RESULT_VOID(dbFree(result.standby), "free standby");
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
