@@ -14,7 +14,7 @@ stress testing as needed.
 #include "common/crypto/hash.h"
 #include "common/compress/gz/compress.h"
 #include "common/compress/lz4/compress.h"
-#include "common/io/filter/filter.intern.h"
+#include "common/io/filter/filter.h"
 #include "common/io/filter/sink.h"
 #include "common/io/bufferRead.h"
 #include "common/io/bufferWrite.h"
@@ -33,11 +33,14 @@ Dummy callback functions
 static void
 storageTestDummyInfoListCallback(void *data, const StorageInfo *info)
 {
-    (void)data;
     (void)info;
 
-    // Do some work in the mem context to blow up the total time if this is not efficient
+    // Do some work in the mem context to blow up the total time if this is not efficient, i.e. if the current mem context is not
+    // being freed regularly
     memResize(memNew(16), 32);
+
+    // Increment callback total
+    (*(uint64_t *)data)++;
 }
 
 /***********************************************************************************************************************************
@@ -115,9 +118,9 @@ testIoRateNew(uint64_t bytesPerSec)
 {
     IoFilter *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("TestIoRate")
+    OBJ_NEW_BEGIN(TestIoRate)
     {
-        TestIoRate *driver = memNew(sizeof(TestIoRate));
+        TestIoRate *driver = OBJ_NEW_ALLOC();
 
         *driver = (TestIoRate)
         {
@@ -125,9 +128,9 @@ testIoRateNew(uint64_t bytesPerSec)
             .bytesPerSec = bytesPerSec,
         };
 
-        this = ioFilterNewP(strNew("TestIoRate"), driver, NULL, .in = testIoRateProcess);
+        this = ioFilterNewP(STRID5("test-io-rate", 0x2d032dbd3ba4cb40), driver, NULL, .in = testIoRateProcess);
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     return this;
 }
@@ -135,7 +138,7 @@ testIoRateNew(uint64_t bytesPerSec)
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
@@ -143,20 +146,22 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("storageInfoList()"))
     {
-        // One million files represents a fairly large cluster
-        CHECK(testScale() <= 2000);
-        uint64_t fileTotal = (uint64_t)1000000 * testScale();
+        TEST_TITLE_FMT("list %d million files", TEST_SCALE);
 
-        HARNESS_FORK_BEGIN()
+        // One million files represents a fairly large cluster
+        CHECK(TEST_SCALE <= 2000);
+        uint64_t fileTotal = (uint64_t)1000000 * TEST_SCALE;
+
+        HRN_FORK_BEGIN(.timeout = 60000)
         {
-            HARNESS_FORK_CHILD_BEGIN(0, true)
+            HRN_FORK_CHILD_BEGIN()
             {
                 // Create a basic configuration so the remote storage driver can determine the storage type
                 StringList *argList = strLstNew();
                 strLstAddZ(argList, "--" CFGOPT_STANZA "=test");
                 strLstAddZ(argList, "--" CFGOPT_PROCESS "=0");
-                strLstAddZ(argList, "--" CFGOPT_REMOTE_TYPE "=" PROTOCOL_REMOTE_TYPE_REPO);
-                harnessCfgLoadRole(cfgCmdArchivePush, cfgCmdRoleRemote, argList);
+                hrnCfgArgRawStrId(argList, cfgOptRemoteType, protocolStorageTypeRepo);
+                HRN_CFG_LOAD(cfgCmdArchivePush, argList, .role = cfgCmdRoleRemote);
 
                 // Create a driver to test remote performance of storageInfoList() and inject it into storageRepo()
                 StorageTestPerfInfoList driver =
@@ -167,32 +172,27 @@ testRun(void)
 
                 driver.interface.infoList = storageTestPerfInfoList;
 
-                storageHelper.storageRepo = memNew(sizeof(Storage *));
-                storageHelper.storageRepo[0] = storageNew(
-                    STRDEF("TEST"), STRDEF("/"), 0, 0, false, NULL, &driver, driver.interface);
+                Storage *storageTest = storageNew(
+                    strIdFromZ(stringIdBit6, "test"), STRDEF("/"), 0, 0, false, NULL, &driver, driver.interface);
+                storageHelper.storageRepoWrite = memNew(sizeof(Storage *));
+                storageHelper.storageRepoWrite[0] = storageTest;
+
+                TEST_RESULT_PTR(storageRepoWrite(), storageTest, "check test storage is used");
 
                 // Setup handler for remote storage protocol
-                IoRead *read = ioFdReadNew(strNew("storage server read"), HARNESS_FORK_CHILD_READ(), 60000);
-                ioReadOpen(read);
-                IoWrite *write = ioFdWriteNew(strNew("storage server write"), HARNESS_FORK_CHILD_WRITE(), 1000);
-                ioWriteOpen(write);
-
-                ProtocolServer *server = protocolServerNew(strNew("storage test server"), strNew("test"), read, write);
-                protocolServerHandlerAdd(server, storageRemoteProtocol);
-                protocolServerProcess(server, NULL);
+                ProtocolServer *server = protocolServerNew(
+                    STRDEF("storage test server"), STRDEF("test"), HRN_FORK_CHILD_READ(), HRN_FORK_CHILD_WRITE());
 
+                static const ProtocolServerHandler commandHandler[] = {PROTOCOL_SERVER_HANDLER_STORAGE_REMOTE_LIST};
+                protocolServerProcess(server, NULL, commandHandler, PROTOCOL_SERVER_HANDLER_LIST_SIZE(commandHandler));
             }
-            HARNESS_FORK_CHILD_END();
+            HRN_FORK_CHILD_END();
 
-            HARNESS_FORK_PARENT_BEGIN()
+            HRN_FORK_PARENT_BEGIN()
             {
                 // Create client
-                IoRead *read = ioFdReadNew(strNew("storage client read"), HARNESS_FORK_PARENT_READ_PROCESS(0), 60000);
-                ioReadOpen(read);
-                IoWrite *write = ioFdWriteNew(strNew("storage client write"), HARNESS_FORK_PARENT_WRITE_PROCESS(0), 1000);
-                ioWriteOpen(write);
-
-                ProtocolClient *client = protocolClientNew(strNew("storage test client"), strNew("test"), read, write);
+                ProtocolClient *client = protocolClientNew(
+                    STRDEF("storage test client"), STRDEF("test"), HRN_FORK_PARENT_READ(0), HRN_FORK_PARENT_WRITE(0));
 
                 // Create remote storage
                 Storage *storageRemote = storageRemoteNew(
@@ -201,18 +201,22 @@ testRun(void)
                 TimeMSec timeBegin = timeMSec();
 
                 // Storage info list
+                uint64_t fileCallbackTotal = 0;
+
                 TEST_RESULT_VOID(
-                    storageInfoListP(storageRemote, NULL, storageTestDummyInfoListCallback, NULL),
-                    "list %" PRIu64 " remote files", fileTotal);
+                    storageInfoListP(storageRemote, NULL, storageTestDummyInfoListCallback, &fileCallbackTotal),
+                    "list remote files");
+
+                TEST_RESULT_UINT(fileCallbackTotal, fileTotal, "check callback total");
 
                 TEST_LOG_FMT("list transferred in %ums", (unsigned int)(timeMSec() - timeBegin));
 
                 // Free client
                 protocolClientFree(client);
             }
-            HARNESS_FORK_PARENT_END();
+            HRN_FORK_PARENT_END();
         }
-        HARNESS_FORK_END();
+        HRN_FORK_END();
     }
 
     // *****************************************************************************************************************************
@@ -222,8 +226,8 @@ testRun(void)
         ioBufferSizeSet(4 * 1024 * 1024);
 
         // 1MB is a fairly normal table size
-        CHECK(testScale() <= 1024 * 1024 * 1024);
-        uint64_t blockTotal = (uint64_t)1 * testScale();
+        CHECK(TEST_SCALE <= 1024 * 1024 * 1024);
+        uint64_t blockTotal = (uint64_t)1 * TEST_SCALE;
 
         // Set iteration
         unsigned int iteration = 1;
@@ -233,7 +237,7 @@ testRun(void)
         uint64_t rateOut = 0; // MB/s (0 disables)
 
         // Get the sample pages from disk
-        Buffer *block = storageGetP(storageNewReadP(storagePosixNewP(STR(testRepoPath())), STRDEF("test/data/filecopy.table.bin")));
+        Buffer *block = storageGetP(storageNewReadP(storagePosixNewP(HRN_PATH_REPO_STR), STRDEF("test/data/filecopy.table.bin")));
         ASSERT(bufUsed(block) == 1024 * 1024);
 
         // Build the input buffer
@@ -383,5 +387,5 @@ testRun(void)
 #endif // HAVE_LIBLZ4
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
