@@ -61,6 +61,8 @@ typedef struct
     uint64_t fileTotal;
 } StorageTestManifestNewBuild;
 
+STRING_STATIC(TEST_MANIFEST_PATH_USER_STR,                          "test");
+
 static StorageInfo
 storageTestManifestNewBuildInfo(THIS_VOID, const String *file, StorageInfoLevel level, StorageInterfaceInfoParam param)
 {
@@ -74,8 +76,8 @@ storageTestManifestNewBuildInfo(THIS_VOID, const String *file, StorageInfoLevel
         .mode = 0600,
         .userId = 100,
         .groupId = 100,
-        .user = STRDEF("test"),
-        .group = STRDEF("test"),
+        .user = TEST_MANIFEST_PATH_USER_STR,
+        .group = TEST_MANIFEST_PATH_USER_STR,
     };
 
     if (strEq(file, STRDEF("/pg")))
@@ -145,7 +147,7 @@ storageTestManifestNewBuildInfoList(
 /***********************************************************************************************************************************
 Test Run
 ***********************************************************************************************************************************/
-void
+static void
 testRun(void)
 {
     FUNCTION_HARNESS_VOID();
@@ -153,8 +155,8 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("lstFind()"))
     {
-        CHECK(testScale() <= 10000);
-        int testMax = 100000 * (int)testScale();
+        CHECK(TEST_SCALE <= 10000);
+        int testMax = 100000 * (int)TEST_SCALE;
 
         // Generate a large list of values (use int instead of string so there fewer allocations)
         List *list = lstNewP(sizeof(int), .comparator = testComparator);
@@ -187,13 +189,40 @@ testRun(void)
         TEST_LOG_FMT("desc search completed in %ums", (unsigned int)(timeMSec() - timeBegin));
     }
 
+    // *****************************************************************************************************************************
+    if (testBegin("lstRemoveIdx()"))
+    {
+        CHECK(TEST_SCALE <= 10000);
+        int testMax = 1000000 * (int)TEST_SCALE;
+
+        // Generate a large list of values (use int instead of string so there fewer allocations)
+        List *list = lstNewP(sizeof(int));
+
+        for (int listIdx = 0; listIdx < testMax; listIdx++)
+            lstAdd(list, &listIdx);
+
+        CHECK(lstSize(list) == (unsigned int)testMax);
+
+        TEST_LOG_FMT("generated %d item list", testMax);
+
+        // Remove all values from index 0
+        TimeMSec timeBegin = timeMSec();
+
+        for (int listIdx = 0; listIdx < testMax; listIdx++)
+            lstRemoveIdx(list, 0);
+
+        TEST_LOG_FMT("remove completed in %ums", (unsigned int)(timeMSec() - timeBegin));
+
+        CHECK(lstEmpty(list));
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("iniLoad()"))
     {
-        CHECK(testScale() <= 10000);
+        CHECK(TEST_SCALE <= 10000);
 
-        String *iniStr = strNew("[section1]\n");
-        unsigned int iniMax = 100000 * (unsigned int)testScale();
+        String *iniStr = strCatZ(strNew(), "[section1]\n");
+        unsigned int iniMax = 100000 * (unsigned int)TEST_SCALE;
 
         for (unsigned int keyIdx = 0; keyIdx < iniMax; keyIdx++)
             strCatFmt(iniStr, "key%u=\"value%u\"\n", keyIdx, keyIdx);
@@ -213,33 +242,32 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("manifestNewBuild()/manifestNewLoad()/manifestSave()"))
     {
-        CHECK(testScale() <= 1000000);
+        CHECK(TEST_SCALE <= 1000000);
 
         // Create a storage driver to test manifest build with an arbitrary number of files
         StorageTestManifestNewBuild driver =
         {
             .interface = storageInterfaceTestDummy,
-            .fileTotal = 100000 * (unsigned int)testScale(),
+            .fileTotal = 100000 * (unsigned int)TEST_SCALE,
         };
 
         driver.interface.info = storageTestManifestNewBuildInfo;
         driver.interface.infoList = storageTestManifestNewBuildInfoList;
 
-        Storage *storagePg = storageNew(STRDEF("TEST"), STRDEF("/pg"), 0, 0, false, NULL, &driver, driver.interface);
+        const Storage *const storagePg = storageNew(
+            strIdFromZ(stringIdBit6, "test"), STRDEF("/pg"), 0, 0, false, NULL, &driver, driver.interface);
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("build manifest");
 
-        MemContext *testContext = memContextNew("test");
+        MemContext *testContext = memContextNewP("test");
         memContextKeep();
         Manifest *manifest = NULL;
         TimeMSec timeBegin = timeMSec();
 
         MEM_CONTEXT_BEGIN(testContext)
         {
-            TEST_ASSIGN(
-                manifest, manifestNewBuild(storagePg, PG_VERSION_91, 999999999, false, false, NULL, NULL),
-                "build with %" PRIu64 " files", driver.fileTotal);
+            TEST_ASSIGN(manifest, manifestNewBuild(storagePg, PG_VERSION_91, 999999999, false, false, NULL, NULL), "build files");
         }
         MEM_CONTEXT_END();
 
@@ -263,7 +291,7 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("load manifest");
 
-        testContext = memContextNew("test");
+        testContext = memContextNewP("test");
         memContextKeep();
         timeBegin = timeMSec();
 
@@ -296,7 +324,7 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("statistics collector"))
     {
-        CHECK(testScale() <= 1000000);
+        CHECK(TEST_SCALE <= 1000000);
 
         // Setup a list of stats to use for testing
         #define TEST_STAT_TOTAL 100
@@ -305,7 +333,7 @@ testRun(void)
         for (unsigned int statIdx = 0; statIdx < TEST_STAT_TOTAL; statIdx++)
             statList[statIdx] = strNewFmt("STAT%u", statIdx);
 
-        uint64_t runTotal = (uint64_t)testScale() * (uint64_t)100000;
+        uint64_t runTotal = (uint64_t)TEST_SCALE * (uint64_t)100000;
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE_FMT("update %d stats %" PRIu64 " times", TEST_STAT_TOTAL, runTotal);
@@ -328,10 +356,10 @@ testRun(void)
         for (unsigned int statIdx = 0; statIdx < TEST_STAT_TOTAL; statIdx++)
         {
             TEST_RESULT_UINT(
-                varUInt64(kvGet(varKv(kvGet(statKv, VARSTR(statList[statIdx]))), STAT_VALUE_TOTAL_VAR)), runTotal, "check stat %u",
-                statIdx);
+                varUInt64(kvGet(varKv(kvGet(statKv, VARSTR(statList[statIdx]))), STAT_VALUE_TOTAL_VAR)), runTotal,
+                strZ(strNewFmt("check stat %u", statIdx)));
         }
     }
 
-    FUNCTION_HARNESS_RESULT_VOID();
+    FUNCTION_HARNESS_RETURN_VOID();
 }
